@@ -17,6 +17,7 @@
 #include "load_gl_extensions.h"
 #include "map.h"
 #include "missiles.h"
+#include "named_colours.h"
 #include "new_actors.h"
 #include "platform.h"
 #include "shadows.h"
@@ -34,13 +35,6 @@
 #include "fsaa/fsaa.h"
 #endif	/* FSAA */
 
-#ifdef ELC
-#define DRAW_ORTHO_INGAME_NORMAL(x, y, z, our_string, max_lines)	draw_ortho_ingame_string(x, y, z, (const Uint8*)our_string, max_lines, INGAME_FONT_X_LEN*10.0, INGAME_FONT_Y_LEN*10.0)
-#define DRAW_INGAME_NORMAL(x, y, our_string, max_lines)	draw_ingame_string(x, y, (const Uint8*)our_string, max_lines, INGAME_FONT_X_LEN, INGAME_FONT_Y_LEN)
-#define DRAW_INGAME_SMALL(x, y, our_string, max_lines)	draw_ingame_string(x, y, (const Uint8*)our_string, max_lines, SMALL_INGAME_FONT_X_LEN, SMALL_INGAME_FONT_Y_LEN)
-#define DRAW_INGAME_ALT(x, y, our_string, max_lines)	draw_ingame_string(x, y, (const Uint8*)our_string, max_lines, ALT_INGAME_FONT_X_LEN, ALT_INGAME_FONT_Y_LEN)
-#endif
-
 actor *actors_list[MAX_ACTORS];
 int max_actors=0;
 SDL_mutex *actors_lists_mutex = NULL;	//used for locking between the timer and main threads
@@ -50,7 +44,7 @@ actor_types actors_defs[MAX_ACTOR_DEFS];
 
 attached_actors_types attached_actors_defs[MAX_ACTOR_DEFS];
 
-void draw_actor_overtext( actor* actor_ptr ); /* forward declaration */
+static void draw_actor_overtext(actor* actor_ptr, double x, double y, double z); /* forward declaration */
 
 int no_near_actors=0;
 #ifdef NEW_SOUND
@@ -146,7 +140,8 @@ int add_actor (int actor_type, char * skin_name, float x_pos, float y_pos, float
 
 	our_actor = calloc(1, sizeof(actor));
 
-	memset(our_actor->current_displayed_text, 0, MAX_CURRENT_DISPLAYED_TEXT_LEN);
+	memset(our_actor->current_displayed_text, 0, sizeof(our_actor->current_displayed_text));
+	our_actor->current_displayed_text_lines = 0;
 	our_actor->current_displayed_text_time_left =  0;
 
 	our_actor->is_enhanced_model=0;
@@ -213,7 +208,7 @@ int add_actor (int actor_type, char * skin_name, float x_pos, float y_pos, float
 	for(k=0;k<MAX_EMOTE_FRAME;k++) our_actor->cur_emote.frames[k].anim_index=-1;
 	our_actor->cur_emote.idle.anim_index=-1;
 	our_actor->cur_emote_sound_cookie=0;
-	
+
 
 
 
@@ -371,10 +366,28 @@ void remove_actor_attachment(int actor_id)
 	UNLOCK_ACTORS_LISTS();
 }
 
-void set_health_color(float percent, float multiplier, float a)
+static void set_health_color(actor * actor_id, float percent, float multiplier, float a)
 {
 	float r,g;
 
+	if (actor_id != NULL)
+	{
+		// Only a subset of actors have health bars, so the choice
+		// here is limited.
+		if (actor_id->actor_id == yourself)
+		{
+			if (!dynamic_banner_colour.yourself)
+				percent = 1.0f;
+		}
+		else if (actor_id->is_enhanced_model)
+		{
+			if (!dynamic_banner_colour.other_players)
+				percent = 1.0f;
+		}
+		else if (!dynamic_banner_colour.creatures)
+			percent = 1.0f;
+	}
+
 	r=(1.0f-percent)*2.0f;
 	g=(percent/1.25f)*2.0f;
 
@@ -386,46 +399,70 @@ void set_health_color(float percent, float multiplier, float a)
 	glColor4f(r*multiplier,g*multiplier,0.0f, a);
 }
 
-void set_mana_color(float percent, float multiplier, float a)
+static void set_mana_color(float percent, float multiplier, float a)
 {
-	float c;
+	static int have_colours = 0;
+	static GLfloat full_mana[3], zero_mana[3];
+	GLfloat use_colours[3];
+	size_t i;
 
-	c=0.6f - percent*0.6f;
+	// get the mana colour range
+	if (!have_colours)
+	{
+		elglGetColour3v("banner.mana.full", full_mana);
+		elglGetColour3v("banner.mana.zero", zero_mana);
+		have_colours = 1;
+	}
 
-	if(c<0.0f)c=0.0f;
-	else if(c>1.0f)c=1.0f;
+	// not dynamic so just to full mana colour
+	if (!dynamic_banner_colour.yourself)
+	{
+		glColor4f(multiplier * full_mana[0], multiplier * full_mana[1], full_mana[2], a);
+		return;
+	}
 
-	glColor4f(c,c,2.0f, a);
+	// dynamic so find step between full and zero mana for each colour
+	for (i = 0; i < 3; i++)
+	{
+		use_colours[i] = full_mana[i] + (zero_mana[i] - full_mana[i]) * (1.0f - percent);
+		if (use_colours[i] < 0.0f)
+			use_colours[i] = 0.0f;
+		else if (use_colours[i] > 1.0f)
+			use_colours[i] = 1.0f;
+	}
+
+	glColor4f(multiplier * use_colours[0], multiplier * use_colours[1], use_colours[2], a);
 }
 
 
-void draw_actor_banner(actor * actor_id, float offset_z)
+static void draw_actor_banner(actor * actor_id, float offset_z)
 {
 	unsigned char str[60];
-	unsigned char temp[255];
 	GLdouble model[16],proj[16];
 	GLint view[4];
 
 	GLdouble hx,hy,hz,a_bounce;
+	float name_zoom = font_scales[NAME_FONT];
 	float font_scale = 1.0f/ALT_INGAME_FONT_X_LEN;
+	float stat_font_size = font_scale * ALT_INGAME_FONT_X_LEN; // base it on font_scale - OK so this makes it 1.0f currently
+	float name_font_size = font_scale * SMALL_INGAME_FONT_X_LEN;
 	double healthbar_x=0.0f;
 	double healthbar_y=0.0f;
 	double healthbar_z=offset_z+0.1;
-	double health_str_x_len=ALT_INGAME_FONT_X_LEN*12.0*name_zoom*3*font_scale;
 	double healthbar_x_len_converted=0;
 	double healthbar_x_len_loss=0;
 	double healthbar_x_loss_fade=1.0f;
+	double y_top, y_bottom;
 
 	//we use health bar variables if possible, all the extras we need for ether bar are:
 	double ether_str_x_len = 0;
 	double etherbar_x_len_converted=0;
-	GLdouble ey;
+	GLdouble name_bot_y, health_bot_y, ether_bot_y;
 
 	//some general values valid for whole banner
 	double bar_x_len = 0;
-	double bar_y_len=ALT_INGAME_FONT_Y_LEN*12.0*name_zoom*font_scale;
+	double bar_y_len = get_line_height(NAME_FONT, stat_font_size);
 	float banner_width = 0.0f;
-	int num_lines;
 
 	//define inner display_xxxxx variables to have more control over displaying inside this function
 	//necesary to implement instance mode makes code a bit more easy to understand imho
@@ -529,21 +566,27 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 			 * an exit condition at the beginning of the function */
 			if ((first_person)&&(actor_id->actor_id==yourself)){
 				float x,y;
-				x = window_width/2.0 -(((float)get_string_width(str) * (font_scale*0.17*name_zoom)))*0.5f;
+				x = window_width / 2.0 - 0.5f * (float)get_string_width_zoom(str, NAME_FONT,  font_scale*0.17);
 				y = a_bounce + window_height/2.0-40.0;
-				draw_ortho_ingame_string(x, y, 0, str, 1, font_scale*.14, font_scale*.21);
+				draw_ortho_ingame_string(x, y, 0, str, 1, NAME_FONT, font_scale*.14, font_scale*.14);
 			}
 			else
 			{
 				float font_scale2 = font_scale*powf(1.0f+((float)abs(actor_id->damage)/2.0f)/1000.0f, 4.0);
-				draw_ortho_ingame_string(hx-(((float)get_string_width(str) * (font_scale2*0.17*name_zoom)))*0.5f, a_bounce+hy+10.0f, 0, str, 1, font_scale2*.14, font_scale2*.21);
-			}			glDisable(GL_BLEND);
+				int extra_y = (view_mode_instance && displaying_me) ?view_mode_instance_damage_height * bar_y_len : 0;
+				int lines = (!(view_mode_instance && displaying_me) && (display_hp || display_health_bar) && (display_ether || display_ether_bar)) ? 3 : 2;
+				draw_ortho_ingame_string(hx - 0.5f * (float)get_string_width_zoom(str, NAME_FONT, font_scale2*0.17),
+					a_bounce + hy + extra_y + get_text_height(lines, NAME_FONT, name_zoom), 0, str, 1,
+					NAME_FONT, font_scale2*.14, font_scale2*.14);
+			}
+			glDisable(GL_BLEND);
 		}
 		else
 		{	//No floating messages
 			sprintf((char*)str,"%i",actor_id->damage);
 			glColor3f (1.0f, 0.3f, 0.3f);
-			DRAW_ORTHO_INGAME_NORMAL(-0.1f,healthbar_z/2.0f,0,str,1.0f);
+			draw_ortho_ingame_string(-0.1f, 0.5*healthbar_z, 0.0f, str, 1, NAME_FONT,
+				INGAME_FONT_X_LEN*10.0, INGAME_FONT_X_LEN*10.0);
 		}
 		if (view_mode_instance && im_other_player_show_banner_on_damage && displaying_other_player && !display_hp && !display_health_bar && actor_id->damage>0) {
 			display_hp = 1;
@@ -553,31 +596,29 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 
 	glDepthFunc(GL_LESS);
 
-	//figure out which lines should we display and how many lines total do we show
+	//figure out which lines should we display
 	display_health_line = (actor_id->kind_of_actor != NPC && (display_hp || display_health_bar) && actor_id->cur_health > 0 && actor_id->max_health > 0);
 	display_ether_line = ((display_ether || display_ether_bar) && displaying_me && your_info.ethereal_points.base > 0 );
-	num_lines = display_names + display_health_line + display_ether_line;
 	if (view_mode_instance && displaying_me) {
 		//make your bar a bit more above everything else so you can see it good enough
 		//and got no problems with attacking mobs
 		hy += view_mode_instance_banner_height*bar_y_len;
-	} else if (displaying_me && display_health_line && display_ether_line) {
-		hy += 1.5*bar_y_len;
 	}
 
-	//calculate "y" positions of ether lines
-	ey = hy -(display_health_line * bar_y_len);
+	// calculate the bottom y coord for the displayed info lines, keep centred around a the same point whether 3, 2 or 1 line being displays
+	hy += bar_y_len;
+	name_bot_y = (display_names) ?hy + (display_health_line * bar_y_len / 2) + (display_ether_line * bar_y_len / 2) :0.0;
+	health_bot_y = (display_health_line) ?hy - (display_names * bar_y_len / 2) + (display_ether_line * bar_y_len / 2) :0.0;
+	ether_bot_y = (display_ether_line) ?hy -(display_names * bar_y_len / 2) - (display_health_line * bar_y_len / 2) : 0.0;
+	// printf("hy=%.1lf name=%.1lf health=%.1lf ether=%.1lf\n", hy, name_bot_y, health_bot_y, ether_bot_y);
 
-	// Schmurk: same here, we actually never reach this code
+	// main block that draws the name, health and ether text
 	if (!((first_person)&&(actor_id->actor_id==yourself)))
 	{
-		if(actor_id->actor_name[0] && (display_names || display_health_line || display_ether_line)){
-			set_font(name_font);	// to variable length
-
-			if(display_names){
-				float font_size_x=font_scale*SMALL_INGAME_FONT_X_LEN;
-				float font_size_y=font_scale*SMALL_INGAME_FONT_Y_LEN;
-
+		if(actor_id->actor_name[0] && (display_names || display_health_line || display_ether_line))
+		{
+			if (display_names)
+			{
 				if(actor_id->kind_of_actor==NPC){
 					glColor3f(0.3f,0.8f,1.0f);
 				} else if(actor_id->kind_of_actor==HUMAN || actor_id->kind_of_actor==COMPUTER_CONTROLLED_HUMAN){
@@ -591,9 +632,9 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 				} else {
 					glColor3f(1.0f,1.0f,0.0f);
 				}
-				safe_snprintf ((char*)temp, sizeof (temp), "%s", actor_id->actor_name);
-				banner_width = ((float)get_string_width((unsigned char*)actor_id->actor_name)*(font_size_x*name_zoom))/2.0;
-				draw_ortho_ingame_string(hx-banner_width, hy+bar_y_len/2.0f, hz, temp, 1, font_size_x, font_size_y);
+				banner_width = 0.5 * (float)get_string_width_zoom((unsigned char*)actor_id->actor_name, NAME_FONT, name_font_size);
+				draw_ortho_ingame_string(hx - banner_width, name_bot_y, hz,
+					(const unsigned char*)actor_id->actor_name, 1, NAME_FONT, name_font_size, name_font_size);
 			}
 			if (view_buffs)
 			{
@@ -603,15 +644,16 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 			if(  (!actor_id->dead) && (actor_id->kind_of_actor != NPC) && (display_health_line || display_ether_line)){
 				unsigned char hp[200];
 				unsigned char mana[200];
+				double health_str_x_len = 0.0;
 
 				// make the heath bar the same length as the the health text so they are balanced
 				// use the same length health bar, even if not displaying the health text
 				sprintf((char*)hp,"%u/%u", actor_id->cur_health, actor_id->max_health);
-				health_str_x_len = (float)get_string_width(hp)*(ALT_INGAME_FONT_X_LEN*name_zoom*font_scale);
+				health_str_x_len = (float)get_string_width_zoom(hp, NAME_FONT, stat_font_size);
 				//do the same with mana if we want to display it
 				if (display_ether || display_ether_bar) {
 					sprintf((char*)mana,"%d/%d", your_info.ethereal_points.cur, your_info.ethereal_points.base);
-					ether_str_x_len=(float)get_string_width(mana)*(ALT_INGAME_FONT_X_LEN*name_zoom*font_scale);
+					ether_str_x_len=(float)get_string_width_zoom(mana, NAME_FONT, stat_font_size);
 				}
 				//set bar length to longer one (mana or health) - not really clean solution
 				if (ether_str_x_len > health_str_x_len) {
@@ -641,18 +683,18 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 
 					if (display_hp) {
 						//choose color for the health
-						set_health_color((float)actor_id->cur_health/(float)actor_id->max_health, 1.0f, 1.0f);
-						draw_ortho_ingame_string(hx-disp+hp_off, hy-bar_y_len/3.0f, hz, hp, 1, ALT_INGAME_FONT_X_LEN*font_scale, ALT_INGAME_FONT_Y_LEN*font_scale);
+						set_health_color(actor_id, (float)actor_id->cur_health/(float)actor_id->max_health, 1.0f, 1.0f);
+						draw_ortho_ingame_string(hx - disp + hp_off, health_bot_y,
+							hz, hp, 1, NAME_FONT, stat_font_size, stat_font_size);
 					}
 
 					if (display_ether) {
 						set_mana_color((float)your_info.ethereal_points.cur / (float)your_info.ethereal_points.base, 1.0f, 1.0f);
-						draw_ortho_ingame_string(hx-disp+eth_off, ey-bar_y_len/3.0f, hz, mana, 1, ALT_INGAME_FONT_X_LEN*font_scale, ALT_INGAME_FONT_Y_LEN*font_scale);
+						draw_ortho_ingame_string(hx - disp + eth_off, ether_bot_y,
+							hz, mana, 1, NAME_FONT, stat_font_size, stat_font_size);
 					}
 				}
 			}
-
-			set_font(0);	// back to fixed pitch
 		}
 	}
 
@@ -662,9 +704,11 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 	if(display_health_bar && display_health_line && (!actor_id->dead) && (actor_id->kind_of_actor != NPC)){
 		float percentage = (float)actor_id->cur_health/(float)actor_id->max_health;
 		float off;
-		
-		if(percentage>110.0f) //deal with massive bars by trimming at 110%
-			percentage = 110.0f;
+		float top_y = health_bot_y + bar_y_len / 3.0;
+		float bot_y = health_bot_y + 2 * bar_y_len / 3.0;
+
+		if(percentage>1.1f) //deal with massive bars by trimming at 110%
+			percentage = 1.1f;
 		if (display_hp){
 			off = bar_x_len + 5.0f;
 		} else {
@@ -693,45 +737,44 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 		hx-=off;
 
 		//choose tint color
-		set_health_color(percentage, 0.5f, 1.0f);
+		set_health_color(actor_id, percentage, 0.5f, 1.0f);
 		glBegin(GL_QUADS);
-			glVertex3d(hx,hy,hz);
-			glVertex3d(hx+healthbar_x_len_converted,hy,hz);
+			glVertex3d(hx, top_y, hz);
+			glVertex3d(hx + healthbar_x_len_converted, top_y, hz);
 
-		set_health_color(percentage, 1.0f, 1.0f);
+		set_health_color(actor_id, percentage, 1.0f, 1.0f);
 
-			glVertex3d(hx+healthbar_x_len_converted,hy+bar_y_len/3.0,hz);
-			glVertex3d(hx,hy+bar_y_len/3.0,hz);
+			glVertex3d(hx + healthbar_x_len_converted, bot_y, hz);
+			glVertex3d(hx, bot_y, hz);
 		glEnd();
 
 		if(healthbar_x_len_loss){
 			glEnable(GL_BLEND);
 			glBlendFunc(GL_SRC_ALPHA, GL_ONE);
 
-			set_health_color(percentage, 0.5f, healthbar_x_loss_fade);
+			set_health_color(actor_id, percentage, 0.5f, healthbar_x_loss_fade);
 
 			glBegin(GL_QUADS);
-				glVertex3d(hx+healthbar_x_len_converted, hy, hz);
-				glVertex3d(hx+healthbar_x_len_converted+healthbar_x_len_loss, hy, hz);
+				glVertex3d(hx + healthbar_x_len_converted, top_y, hz);
+				glVertex3d(hx + healthbar_x_len_converted + healthbar_x_len_loss, top_y, hz);
 
-			set_health_color(percentage, 1.0f, healthbar_x_loss_fade);
+			set_health_color(actor_id, percentage, 1.0f, healthbar_x_loss_fade);
 
-				glVertex3d(hx+healthbar_x_len_converted+healthbar_x_len_loss, hy+bar_y_len/3.0,hz);
-				glVertex3d(hx+healthbar_x_len_converted, hy+bar_y_len/3.0,hz);
+				glVertex3d(hx + healthbar_x_len_converted + healthbar_x_len_loss, bot_y, hz);
+				glVertex3d(hx + healthbar_x_len_converted, bot_y, hz);
 			glEnd();
 
 			glDisable(GL_BLEND);
 		}
 
-
 		//draw the frame
 		glDepthFunc(GL_LEQUAL);
-		glColor3f (0.0f, 0.0f, 0.0f);
+		glColor3f(0.0f, 0.0f, 0.0f);
 		glBegin(GL_LINE_LOOP);
-			glVertex3f (hx-1.0, hy-1.0, hz);
-			glVertex3f (hx+bar_x_len+1.0, hy-1.0,hz);
-			glVertex3f (hx+bar_x_len+1.0, hy+bar_y_len/3.0+1.0,hz);
-			glVertex3f (hx-1.0, hy+bar_y_len/3.0+1.0,hz);
+			glVertex3f(hx - 1.0, top_y - 1.0, hz);
+			glVertex3f(hx + bar_x_len + 1.0, top_y - 1.0, hz);
+			glVertex3f(hx + bar_x_len + 1.0, bot_y + 1.0, hz);
+			glVertex3f(hx - 1.0, bot_y + 1.0, hz);
 		glEnd();
 
 		hx+=off;
@@ -740,6 +783,8 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 	if (display_ether_bar && display_ether_line) {
 		float percentage = (float)your_info.ethereal_points.cur / (float)your_info.ethereal_points.base;
 		float off;
+		float top_y = ether_bot_y + bar_y_len / 3.0;
+		float bot_y = ether_bot_y + 2 * bar_y_len / 3.0;
 		if(percentage>1.1f) // limit bar length to +10%
 			percentage = 1.1f;
 		if (display_ether){
@@ -755,56 +800,55 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 		set_mana_color(percentage, 0.5f, 1.0f);
 		etherbar_x_len_converted = percentage * bar_x_len;
 		glBegin(GL_QUADS);
-			glVertex3d(hx,ey,hz);
-			glVertex3d(hx+etherbar_x_len_converted,ey,hz);
+			glVertex3d(hx, top_y, hz);
+			glVertex3d(hx + etherbar_x_len_converted, top_y, hz);
 
 		set_mana_color(percentage, 1.0f, 1.0f);
 
-			glVertex3d(hx+etherbar_x_len_converted,ey+bar_y_len/3.0,hz);
-			glVertex3d(hx,ey+bar_y_len/3.0,hz);
+			glVertex3d(hx + etherbar_x_len_converted, bot_y, hz);
+			glVertex3d(hx, bot_y, hz);
 		glEnd();
-		set_health_color(percentage, 1.0f, 1.0f);
+		set_health_color(actor_id, percentage, 1.0f, 1.0f);
 		glDepthFunc(GL_LEQUAL);
 		glColor3f (0.0f, 0.0f, 0.0f);
 		glBegin(GL_LINE_LOOP);
-			glVertex3f (hx-1.0, ey-1.0 , hz);
-			glVertex3f (hx+bar_x_len+1.0, ey-1.0,hz);
-			glVertex3f (hx+bar_x_len+1.0, ey+bar_y_len/3.0+1.0,hz);
-			glVertex3f (hx-1.0, ey+bar_y_len/3.0+1.0,hz);
+			glVertex3f (hx - 1.0, top_y - 1.0 , hz);
+			glVertex3f (hx + bar_x_len + 1.0, top_y - 1.0, hz);
+			glVertex3f (hx + bar_x_len + 1.0, bot_y + 1.0, hz);
+			glVertex3f (hx - 1.0, bot_y + 1.0, hz);
 		glEnd();
 		hx+=off;
 	}
 
 	// draw the alpha background (if ness)
+	y_bottom = ((ether_bot_y > 0) ?ether_bot_y :((health_bot_y > 0) ?health_bot_y : name_bot_y)) - 0.2 * bar_y_len;
+	y_top = ((name_bot_y > 0) ?name_bot_y :((health_bot_y > 0) ?health_bot_y : ether_bot_y)) + 1.2 * bar_y_len;
 	if (display_banner_alpha && banner_width > 0) {
 		//if banner width > 0 there MUST be something displayed in the banner
-		float start_y = hy;
-		start_y  += ((!display_health_line && !display_ether_line && display_names) ?bar_y_len-6.0 :-5.0);
-		start_y  -= (num_lines == 3 || (num_lines==2 && !display_names)) ? bar_y_len:0.0;
-		banner_width += 3;
+		banner_width += name_zoom * 3;
 		glEnable(GL_BLEND);
 		glBlendFunc(GL_ONE, GL_SRC_ALPHA);
 		glColor4f(0.0f, 0.0f, 0.0f, 0.6f);
 		glBegin(GL_QUADS);
-			glVertex3f (hx-banner_width, start_y, hz + 0.0001);
-			glVertex3f (hx+banner_width, start_y, hz + 0.0001);
-			glVertex3f (hx+banner_width, start_y+bar_y_len*num_lines+2, hz + 0.0001);
-			glVertex3f (hx-banner_width, start_y+bar_y_len*num_lines+2, hz + 0.0001);
+			glVertex3f(hx-banner_width, y_bottom, hz + 0.0001);
+			glVertex3f(hx+banner_width, y_bottom, hz + 0.0001);
+			glVertex3f(hx+banner_width, y_top, hz + 0.0001);
+			glVertex3f(hx-banner_width, y_top, hz + 0.0001);
 		glEnd();
 		glDisable(GL_BLEND);
 	}
 
 	glEnable(GL_TEXTURE_2D);
 
+	if ((actor_id->current_displayed_text_time_left>0)&&(actor_id->current_displayed_text[0] != 0)){
+		draw_actor_overtext(actor_id, hx, y_top, hz);
+	}
+
 	glMatrixMode(GL_PROJECTION);
 	glPopMatrix();
 	glMatrixMode(GL_MODELVIEW);
 	glPopMatrix();
 
-	if ((actor_id->current_displayed_text_time_left>0)&&(actor_id->current_displayed_text[0] != 0)){
-		draw_actor_overtext( actor_id );
-	}
-
 	if(floatingmessages_enabled)drawactor_floatingmessages(actor_id->actor_id, healthbar_z);
 
 	/* set cm_mouse_over_banner true if the mouse is over your banner, or a box where it might be */
@@ -812,11 +856,9 @@ void draw_actor_banner(actor * actor_id, float offset_z)
 	{
 		/* use the same calculation as for the alpha background but have a fallback if no banner shown */
 		int xoff = (banner_width > 0) ?banner_width: 60;
-		float start_y = hy;
-		start_y  += ((!display_health_line && !display_ether_line && display_names) ?bar_y_len-6.0 :-5.0);
-		start_y  -= (num_lines == 3 || (num_lines==2 && !display_names)) ? bar_y_len:0.0;
+		int yoff = (banner_width > 0) ?y_top - y_bottom: bar_y_len;
 		if ((mouse_x > hx-xoff) && (mouse_x < hx+xoff) &&
-			(window_height-mouse_y > start_y) && (window_height-mouse_y < start_y+bar_y_len*((num_lines>0)?num_lines:3)))
+			((window_height - mouse_y) > y_bottom) && ((window_height - mouse_y) < (y_bottom + yoff)))
 			cm_mouse_over_banner = 1;
 		else
 			cm_mouse_over_banner = 0;
@@ -829,43 +871,50 @@ CHECK_GL_ERRORS();
 }
 
 
-void draw_bubble(float x_left, float x_right, float x_leg_left, float x_leg_right, float y_top, float y_bottom, float y_actor)
+static void draw_bubble(float x_left, float x_right, float x_leg_left, float x_leg_right,
+	float y_top, float y_bottom, float y_actor, float z, float r)
 {
-	const float r=0.1f;
-	const float mul=M_PI/180.0f;
+	const float mul = M_PI/180.0f;
 	int angle;
 
 	glEnable(GL_BLEND);
 	glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
 	glBlendFunc(GL_NONE, GL_SRC_ALPHA);
-	glBegin(GL_POLYGON);
-
-	for(angle=90;angle<180;angle+=10){
-		float rad=-mul*angle;
-		glVertex3f(x_left+cos(rad)*r-r, 0.01f, y_bottom+r+sin(rad)*r);
-	}
-
-	for(angle=180;angle<270;angle+=10){
-		float rad=-mul*angle;
-		glVertex3f(x_left+cos(rad)*r-r, 0.01f, y_top-r+sin(rad)*r);
-	}
-
-	for(angle=270;angle<360;angle+=10){
-		float rad=-mul*angle;
-		glVertex3f(x_right+cos(rad)*r+r, 0.01f, y_top-r+sin(rad)*r);
-	}
-
-	for(angle=0;angle<90;angle+=10){
-		float rad=-mul*angle;
-		glVertex3f(x_right+cos(rad)*r+r, 0.01f, y_bottom+sin(rad)*r+r);
-	}
-
-	glEnd();
 
 	glBegin(GL_POLYGON);
-		glVertex3f(x_leg_right, 0.01f, y_bottom+0.02);
-		glVertex3f(x_leg_right, 0.01f, y_actor);
-		glVertex3f(x_leg_left, 0.01f, y_bottom+0.02);
+	for (angle = 0; angle <= 90; angle += 10)
+	{
+		float rad = mul * angle;
+		glVertex3f(x_right - r + cos(rad)*r, y_top - r + sin(rad)*r, z);
+	}
+
+	for (angle = 90;angle <= 180; angle += 10)
+	{
+		float rad = mul*angle;
+		glVertex3f(x_left + r + cos(rad)*r, y_top - r + sin(rad)*r, z);
+	}
+
+	for(angle = 180; angle < 270; angle += 10)
+	{
+		float rad = mul*angle;
+		glVertex3f(x_left + r + cos(rad)*r, y_bottom + r + sin(rad)*r, z);
+	}
+	// Explicitly include the vertices where the leg attaches to the bubble, to prevent a gap
+	// between the bubble and the leg
+	glVertex3f(x_leg_left, y_bottom, z);
+	glVertex3f(x_leg_right, y_bottom, z);
+
+	for (angle = 270; angle <= 360; angle += 10)
+	{
+		float rad = mul*angle;
+		glVertex3f(x_right - r + cos(rad)*r, y_bottom + r + sin(rad)*r, z);
+	}
+	glEnd(); // GL_POLYGON
+
+	glBegin(GL_TRIANGLES);
+		glVertex3f(x_leg_left, y_bottom, z);
+		glVertex3f(x_leg_right, y_actor, z);
+		glVertex3f(x_leg_right, y_bottom, z);
 	glEnd();
 
 	glDisable(GL_BLEND);
@@ -875,51 +924,66 @@ CHECK_GL_ERRORS();
 }
 
 //-- Logan Dugenoux [5/26/2004]
-void draw_actor_overtext( actor* actor_ptr )
+static void draw_actor_overtext(actor* actor_ptr, double x, double y, double z)
 {
-	float z, w, h;
-	float x_left, x_right, x_leg_left, x_leg_right, y_top, y_bottom, y_actor;
-	float textwidth;
-	float textheight;
+	int lines = min2i(actor_ptr->current_displayed_text_lines, MAX_CURRENT_DISPLAYED_TEXT_LINES);
+	float font_scale = 0.14f / ALT_INGAME_FONT_X_LEN;
+	float x_left, x_right, x_leg_left, x_leg_right, y_top, y_bottom;
+	int text_width, line_height, text_height;
 	float margin;
 
-	//-- decrease display time
-	actor_ptr->current_displayed_text_time_left -= (cur_time-last_time);
+	actor *me = get_our_actor();
+	if (me && me != actor_ptr)
+	{
+		const float s_rx = sin(rx * M_PI / 180);
+		const float c_rx = cos(rx * M_PI / 180);
+		const float s_rz = sin(rz * M_PI / 180);
+		const float c_rz = cos(rz * M_PI / 180);
+		float cam_x = me->x_pos + zoom_level*camera_distance * s_rx * s_rz;
+		float cam_y = me->y_pos + zoom_level*camera_distance * s_rx * c_rz;
+		float cam_z = me->z_pos + zoom_level*camera_distance * c_rx;
+		double dx, dy, dz, actor_dsq, me_dsq;
+		dx = actor_ptr->x_pos - cam_x;
+		dy = actor_ptr->y_pos - cam_y;
+		dz = actor_ptr->z_pos - cam_z;
+		actor_dsq = dx*dx + dy*dy + dz*dz;
+		me_dsq = zoom_level*zoom_level * camera_distance*camera_distance;
+		font_scale *= 0.5 + 0.5 * me_dsq / actor_dsq;
+	}
 
-	textwidth = ((float)get_string_width((unsigned char*)(actor_ptr->current_displayed_text))*(SMALL_INGAME_FONT_X_LEN*zoom_level*name_zoom/3.0))/12.0;
-	textheight = (0.06f*zoom_level/3.0)*4;
-	margin = 0.02f*zoom_level;
-	z = 1.2f;// distance over the player
-	if (actor_ptr->sitting)		z = 0.8f; // close if he's sitting
-	w = textwidth+margin*2;
-	h = textheight+margin*2;
+	get_buf_dimensions((const unsigned char*)actor_ptr->current_displayed_text,
+		strlen(actor_ptr->current_displayed_text),
+		CHAT_FONT, font_scale, &text_width, &text_height);
+	line_height = get_line_height(CHAT_FONT, font_scale);
 
-	x_left=-w/2.0f;
-	x_right=w/2.0f;
-	x_leg_left=-0.3f;
-	x_leg_right=0.0f;
+	margin = 0.5 * line_height;
 
-	y_top=z+0.7f+h;
-	y_bottom=z+0.7f;
-	y_actor=z+0.2f;
+	x_left = x - 0.5*text_width - margin;
+	x_right = x + 0.5*text_width + margin;
+	x_leg_left = x - margin;
+	x_leg_right = x;
+
+	y_bottom = y + 1.5*margin;
+	y_top = y_bottom + text_height + 2*margin;
 
 	glDisable(GL_TEXTURE_2D);
-
-	draw_bubble(x_left+0.01f, x_right-0.01f, x_leg_left, x_leg_right, y_top-0.01f, y_bottom+0.01f, y_actor+0.01f);
-
+	draw_bubble(x_left, x_right, x_leg_left, x_leg_right, y_top, y_bottom, y, z+0.0001f, 1.5*margin);
 	glEnable(GL_TEXTURE_2D);
 
 	//---
 	// Draw text
-	glColor3f(0.77f,0.57f,0.39f);
 
-	DRAW_INGAME_SMALL(x_left+margin, y_bottom+margin,actor_ptr->current_displayed_text,1);
+	draw_ortho_ingame_string(x - 0.5f * text_width, y_bottom + margin + text_height - line_height, z,
+		(const unsigned char*)actor_ptr->current_displayed_text, lines,
+		CHAT_FONT, font_scale, font_scale);
 
-	//glDepthFunc(GL_LESS);
-	if (actor_ptr->current_displayed_text_time_left<=0)
+	//-- decrease display time
+	actor_ptr->current_displayed_text_time_left -= (cur_time-last_time);
+	if (actor_ptr->current_displayed_text_time_left <= 0)
 	{	// clear if needed
 		actor_ptr->current_displayed_text_time_left = 0;
-		actor_ptr->current_displayed_text[0] = 0;
+		actor_ptr->current_displayed_text_lines = 0;
+		actor_ptr->current_displayed_text[0] = '\0';
 	}
 #ifdef OPENGL_TRACE
 CHECK_GL_ERRORS();
@@ -1000,7 +1064,7 @@ CHECK_GL_ERRORS();
 #endif //OPENGL_TRACE
 }
 
-static __inline__ void draw_actor_banner_new(actor * actor_id)
+static void draw_actor_banner_new(actor * actor_id)
 {
 	float x_pos, y_pos, z_pos;
 	float healthbar_z;
@@ -1566,7 +1630,11 @@ void add_actor_from_server (const char *in_data, int len)
 
 	i= add_actor(actor_type, actors_defs[actor_type].skin_name, f_x_pos, f_y_pos, 0.0, f_z_rot, scale, 0, 0, 0, 0, 0, 0, 0, actor_id);
 
-	if(i==-1) return;//A nasty error occured and we couldn't add the actor. Ignore it.
+	if(i==-1)
+	{
+		UNLOCK_ACTORS_LISTS();
+		return;//A nasty error occured and we couldn't add the actor. Ignore it.
+	}
 
 	//The actors list is locked when we get here...
 
@@ -1623,7 +1691,7 @@ void add_actor_from_server (const char *in_data, int len)
 		{
 			LOG_ERROR("%s (%d): %s/%d\n", bad_actor_name_length, actors_list[i]->actor_type,&in_data[17], (int)strlen(&in_data[17]));
 		}
-	else my_strncp(actors_list[i]->actor_name,&in_data[17],30);
+	else safe_strncpy(actors_list[i]->actor_name,&in_data[17],30);
 
 	if (attachment_type >= 0)
 		add_actor_attachment(actor_id, attachment_type);
@@ -1675,14 +1743,14 @@ void add_actor_from_server (const char *in_data, int len)
 //--- LoganDugenoux [5/25/2004]
 #define MS_PER_CHAR	200
 #define MINI_BUBBLE_MS	500
-void	add_displayed_text_to_actor( actor * actor_ptr, const char* text )
+void add_displayed_text_to_actor(actor *actor_ptr, const char* text)
 {
-	int len_to_add;
-	len_to_add = strlen(text);
-	safe_snprintf(actor_ptr->current_displayed_text, sizeof(actor_ptr->current_displayed_text), "%s", text);
-	actor_ptr->current_displayed_text_time_left = len_to_add*MS_PER_CHAR;
-
-	actor_ptr->current_displayed_text_time_left += MINI_BUBBLE_MS;
+	char *dest = actor_ptr->current_displayed_text;
+	const size_t size = sizeof(actor_ptr->current_displayed_text);
+	safe_snprintf(dest, size, "%s", text);
+	actor_ptr->current_displayed_text_lines = reset_soft_breaks((unsigned char*)dest, strlen(dest),
+		size, CHAT_FONT, 1.0, window_width/3, NULL, NULL);
+	actor_ptr->current_displayed_text_time_left = MINI_BUBBLE_MS + strlen(text) * MS_PER_CHAR;
 }
 
 //--- LoganDugenoux [5/25/2004]
