@@ -14,6 +14,7 @@
 #include <SDL_mutex.h>
 #include "bbox_tree.h"
 #include "cal_types.h"
+#include "chat.h"
 #include "client_serv.h"
 #include "platform.h"
 #include "tiles.h"
@@ -33,7 +34,6 @@ extern "C" {
 extern int yourself; 	/*!< This variable holds the actor_id (as the server sees it, not the position in the actors_list) of your character.*/
 extern int you_sit; 	/*!< Specifies if you are currently sitting down.*/
 extern int sit_lock; 	/*!< The sit_lock variable holds you in a sitting position.*/
-extern float name_zoom; /*!< The name_zoom defines how large the text used for drawing the names should be*/
 extern int use_alpha_banner;	/*!< Use_alpha_banner defines if an alpha background is drawn behind the name/health banner.*/
 
 /*!
@@ -48,8 +48,10 @@ extern int use_alpha_banner;	/*!< Use_alpha_banner defines if an alpha backgroun
 #define PKABLE_COMPUTER_CONTROLLED 5	/*!< Draw the actors name in red*/
 /*! \} */
 
-/*! Max text len to display into bubbles overhead*/
-#define MAX_CURRENT_DISPLAYED_TEXT_LEN	60
+/*! The maximum number of lines in the overhead text */
+#define MAX_CURRENT_DISPLAYED_TEXT_LINES 3
+/*! Max text len to display into bubbles overhead */
+#define MAX_CURRENT_DISPLAYED_TEXT_LEN (MAX_TEXT_MESSAGE_LENGTH + MAX_CURRENT_DISPLAYED_TEXT_LINES + 1)
 
 // default duration in ms of a step when an actor is walking
 #define DEFAULT_STEP_DURATION 250
@@ -631,13 +633,14 @@ typedef struct
 	char ghost;		/*!< Sets the actor type to ghost (Disable lightning, enable blending (GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA))*/
 	char has_alpha;		/*!< is alpha blending needed for this actor? */
 	int kind_of_actor;	/*!< Defines the kind_of_actor (NPC, HUMAN, COMPUTER_CONTROLLED_HUMAN, PKABLE, PKABLE_COMPUTER_CONTROLLED)*/
-	Uint32 buffs;		/*!<Contains the buffs on this actor as bits (currently only invisibility)*/
+	Uint32 buffs;		/*!< Contains the buffs on this actor as bits (currently only invisibility)*/
 	/*! \} */
 
 	/*! \name Overhead text (text bubbles)*/
 	/*! \{ */
 	char current_displayed_text[MAX_CURRENT_DISPLAYED_TEXT_LEN]; /*!< If the text is displayed in a bubble over the actor, this holds the text*/
-	int current_displayed_text_time_left;	/*!< Defines the remaining time the overhead text should be displayed*/
+	int current_displayed_text_lines;     /*!< The number of lines of text */
+	int current_displayed_text_time_left; /*!< Defines the remaining time the overhead text should be displayed*/
 	/*! \} */
 
 	/*! \name Unused variables*/
@@ -682,24 +685,11 @@ extern attached_actors_types attached_actors_defs[MAX_ACTOR_DEFS]; /*!< The defi
 static __inline__ int is_actor_barehanded(actor *act, int hand){
 	if(hand==EMOTE_BARE_L)
 		return (act->cur_shield==SHIELD_NONE||act->cur_shield==QUIVER_ARROWS||act->cur_shield==QUIVER_BOLTS);
-	else 
+	else
 		return (act->cur_weapon==WEAPON_NONE||act->cur_weapon==GLOVE_FUR||act->cur_weapon==GLOVE_LEATHER);
 }
 
 
-/*!
- * \ingroup	display_actors
- * \brief	Draws the actors banner (healthbar, name, etc)
- *
- * 		This function is used for drawing the healthbar, the name, the damage, the healthpoints (cur/max) and the text bubbles
- *
- * \param	actor_id Is a pointer to the actor we wish to draw
- * \param	offset_z Is the z offset, found by the current MD2 frames max_z.
- *
- * \callgraph
- */
-void draw_actor_banner(actor * actor_id, float offset_z);
-
 /*!
  * \ingroup	display_actors
  * \brief	The main actor loop - draws all actors within range
@@ -748,14 +738,14 @@ extern SDL_threadID have_actors_lock;
 #define	LOCK_ACTORS_LISTS() 	\
 	{\
 		fprintf(stderr,"Last locked by: %s %s %d\n",__FILE__,__FUNCTION__,__LINE__);\
-		if(SDL_LockMutex(actors_lists_mutex)==-1) {fprintf(stderr,"We're fucked!! The mutex on %s %s %d was not locked even though we asked it to!\n",__FILE__,__FUNCTION__,__LINE__); abort(); }\
+		if(SDL_LockMutex(actors_lists_mutex)==-1) {fprintf(stderr,"The mutex on %s %s %d was not locked even though we asked it to!\n",__FILE__,__FUNCTION__,__LINE__); abort(); }\
 		assert(have_actors_lock==0); have_actors_lock=SDL_ThreadID(); \
 	}
 #define	UNLOCK_ACTORS_LISTS() 	\
 	{\
 		fprintf(stderr,"Last unlocked by: %s %s %d\n",__FILE__,__FUNCTION__,__LINE__);\
 		assert(have_actors_lock); assert(have_actors_lock==SDL_ThreadID()); have_actors_lock=0; \
-		if(SDL_UnlockMutex(actors_lists_mutex)==-1)  {fprintf(stderr,"We're fucked!! The mutex on %s %s %d was not unlocked even though we asked it to!\n",__FILE__,__FUNCTION__,__LINE__); abort(); }\
+		if(SDL_UnlockMutex(actors_lists_mutex)==-1)  {fprintf(stderr,"The mutex on %s %s %d was not unlocked even though we asked it to!\n",__FILE__,__FUNCTION__,__LINE__); abort(); }\
 	}
 /*! @} */
 #else
@@ -880,14 +870,14 @@ void draw_actor_without_banner(actor * actor_id, Uint32 use_lightning, Uint32 us
 /*!
  * \brief Remember this last summoned creature.
  *
- * \param The name of the summoned creature
+ * \param summoned_name The name of the summoned creature
  */
 void remember_new_summoned(const char *summoned_name);
 
 /*!
  * \brief Check if a new actor is the last summoned by the player.  The actor mutex must be already held.
  *
- * \param Pointer to the new actors
+ * \param new_actor Pointer to the new actor
  */
 void check_if_new_actor_last_summoned(actor *new_actor);
 
