@@ -37,9 +37,6 @@ typedef struct
 #endif // ONGOING_BAG_EFFECT
 } bag;
 
-int ground_items_win= -1;
-int ground_items_menu_x=400;
-int ground_items_menu_y=20;
 int ground_items_visible_grid_rows = 10;
 int ground_items_visible_grid_cols = 5;
 int items_auto_get_all = 0;
@@ -50,6 +47,8 @@ static bag bag_list[NUM_BAGS];
 static int GRIDSIZE = 33;
 static int ground_items_grid_rows = 10;
 static int ground_items_grid_cols = 5;
+static int grid_sep_x = 0;
+static int grid_sep_y = 0;
 static const int min_grid_rows = 4;
 static const int min_grid_cols = 2;
 static const char *item_desc_str = NULL;
@@ -151,6 +150,31 @@ static float get_bag_tilt(float pos_x, float pos_y, int bag_id, int map_x, int m
 		digest[6]) + cosf(digest[7])) / 5.0f) * 30;
 }
 
+void revalidate_ground_bag_window(void)
+{
+	// never opened a bag window so nothing to do
+	if (get_id_MW(MW_BAGS) < 0)
+		return;
+
+	// if we are over an open bag, refresh its contents
+	if (your_actor && get_show_window_MW(MW_BAGS))
+	{
+		size_t i;
+		for (i = 0; i < NUM_BAGS; i++)
+			if ((bag_list[i].obj_3d_id != -1) &&
+				(bag_list[i].x == (your_actor->x_pos * 2)) &&
+				(bag_list[i].y == (your_actor->y_pos * 2)))
+			{
+				Uint8 str[2] = { INSPECT_BAG, i };
+				my_tcp_send(str, 2);
+				return;
+			}
+	}
+
+	// either we have an open window but no bag, or a previously opened window - clear the state
+	client_close_bag();
+}
+
 void put_bag_on_ground(int bag_x,int bag_y,int bag_id)
 {
 	float x,y,z;
@@ -264,10 +288,8 @@ void add_bags_from_list (const Uint8 *data)
 
 		// Now, find a place into the bags list, so we can destroy the bag properly
 		if (bag_list[bag_id].obj_3d_id != -1) {
-			char buf[256];
 			// oops, slot already taken!
-			safe_snprintf(buf, sizeof(buf), "Oops, trying to add an existing bag! id=%d\n", bag_id);
-			LOG_ERROR(buf);
+			LOG_ERROR("Oops, trying to add an existing bag! id=%d\n", bag_id);
 			return;
 		}
 
@@ -283,7 +305,8 @@ void add_bags_from_list (const Uint8 *data)
 
 void remove_item_from_ground(Uint8 pos)
 {
-	ground_item_list[pos].quantity= 0;
+	if (pos < ITEMS_PER_BAG)
+		ground_item_list[pos].quantity= 0;
 }
 
 void remove_bag(int bag_id)
@@ -368,10 +391,8 @@ int find_and_open_closest_bag(int tile_x, int tile_y, float max_distance)
 
 	if (found_bag != -1)
 	{
-		Uint8 str[4];
-		str[0]= INSPECT_BAG;
-		str[1]= found_bag;
-		my_tcp_send(my_socket,str, 2);
+		Uint8 str[2] = { INSPECT_BAG, found_bag };
+		my_tcp_send(str, 2);
 		return 1;
 	}
 
@@ -381,12 +402,11 @@ int find_and_open_closest_bag(int tile_x, int tile_y, float max_distance)
 void open_bag(int object_id)
 {
 	int i;
-	Uint8 str[4];
 	for(i=0;i<NUM_BAGS;i++){
-		if(bag_list[i].obj_3d_id==object_id){
-			str[0]= INSPECT_BAG;
-			str[1]= i;
-			my_tcp_send(my_socket,str,2);
+		if(bag_list[i].obj_3d_id==object_id)
+		{
+			Uint8 str[2] = { INSPECT_BAG, i };
+			my_tcp_send(str, 2);
 			return;
 		}
 	}
@@ -409,14 +429,13 @@ void get_bag_item (const Uint8 *data)
 
 static void pick_up_all_items(void)
 {
-	Uint8 str[20];
 	int itempos;
 	for(itempos = 0; itempos < ITEMS_PER_BAG; itempos++){
-		if(ground_item_list[itempos].quantity){
-			str[0]=PICK_UP_ITEM;
-			str[1]=itempos;
+		if(ground_item_list[itempos].quantity)
+		{
+			Uint8 str[6] = { PICK_UP_ITEM, itempos };
 			*((Uint32 *)(str+2))=SDL_SwapLE32(ground_item_list[itempos].quantity);
-			my_tcp_send(my_socket,str,6);
+			my_tcp_send(str, 6);
 		}
 	}
 }
@@ -429,7 +448,7 @@ void items_get_bag(int x, int y)
 		if(bag_list[pos].x != 0 && bag_list[pos].y != 0 &&
 			bag_list[pos].x == x && bag_list[pos].y == y)
 		{
-			if(get_show_window(ground_items_win))
+			if(get_show_window_MW(MW_BAGS))
 				pick_up_all_items();
 			else
 			{
@@ -505,7 +524,7 @@ static int display_ground_items_handler(window_info *win)
 	int i;
 	static Uint8 resizing = 0;
 	int yoffset = get_window_scroll_pos(win->window_id);
-	int but_text_x, but_text_y;
+	int but_text_y;
 
 	/* if resizing wait until we stop */
 	if (win->resized)
@@ -513,11 +532,11 @@ static int display_ground_items_handler(window_info *win)
 	/* once we stop, snap the window to the new grid size */
 	else if (resizing)
 	{
-		int new_width = (ground_items_grid_cols+1)*GRIDSIZE;
-		int new_rows = (win->len_y+GRIDSIZE/2)/GRIDSIZE;
+		int new_width = grid_sep_x + (ground_items_grid_cols + 1) * GRIDSIZE;
+		int new_rows = (win->len_y - 2 * grid_sep_y + GRIDSIZE / 2) / GRIDSIZE;
 		int max_rows = (ITEMS_PER_BAG + ground_items_grid_cols - 1) / ground_items_grid_cols;
 		resizing = 0;
-		resize_window (win->window_id, new_width, ((new_rows > max_rows) ?max_rows :new_rows)*GRIDSIZE);
+		resize_window (win->window_id, new_width, 2 * grid_sep_y + ((new_rows > max_rows) ?max_rows :new_rows) * GRIDSIZE);
 		yoffset = get_window_scroll_pos(win->window_id);
 	}
 
@@ -525,10 +544,9 @@ static int display_ground_items_handler(window_info *win)
 
 	// write "get all" in the "get all" box :)
 	strap_word(get_all_str,my_str);
-	glColor3f(0.77f,0.57f,0.39f);
-	but_text_x = (int)(0.5 + ((GRIDSIZE - (float)(3 * win->small_font_len_x)) / 2.0));
+	glColor3fv(gui_color);
 	but_text_y = (int)(0.5 + ((GRIDSIZE - (float)(2 * win->small_font_len_y)) / 2.0));
-	draw_string_small_zoomed(win->len_x+but_text_x-GRIDSIZE, win->box_size+but_text_y+yoffset, (unsigned char*)my_str, 2, win->current_scale);
+	draw_string_small_zoomed_centered(win->len_x-GRIDSIZE/2, win->box_size+but_text_y+yoffset, (unsigned char*)my_str, 2, win->current_scale);
 
 	glColor3f(1.0f,1.0f,1.0f);
 	//ok, now let's draw the objects...
@@ -545,9 +563,9 @@ static int display_ground_items_handler(window_info *win)
 
 			//get the x and y
 			cur_pos=i;
-			x_start=GRIDSIZE*(cur_pos%ground_items_grid_cols)+1;
+			x_start = grid_sep_x + GRIDSIZE * (cur_pos % ground_items_grid_cols) + 1;
 			x_end=x_start+GRIDSIZE-1;
-			y_start=GRIDSIZE*(cur_pos/ground_items_grid_cols);
+			y_start = grid_sep_y +  + GRIDSIZE * (cur_pos / ground_items_grid_cols);
 			y_end=y_start+GRIDSIZE-1;
 
 			//get the texture this item belongs to
@@ -559,7 +577,7 @@ static int display_ground_items_handler(window_info *win)
 			glEnd();
 
 			safe_snprintf(str,sizeof(str),"%i",ground_item_list[i].quantity);
-			y_end -= -3 + ((i&1) ?GRIDSIZE-1 : ((use_large) ?win->default_font_len_y :win->small_font_len_y));
+			y_end -= ((i & 1) ?GRIDSIZE-1 : ((use_large) ?win->default_font_len_y :win->small_font_len_y));
 			if (use_large)
 				draw_string_shadowed_zoomed(x_start,y_end,(unsigned char*)str,1,1.0f,1.0f,1.0f,0.0f,0.0f,0.0f, win->current_scale);
 			else
@@ -573,16 +591,16 @@ static int display_ground_items_handler(window_info *win)
 	// cards
 	glDisable(GL_TEXTURE_2D);
 
-	glColor3f(0.77f,0.57f,0.39f);
+	glColor3fv(gui_color);
 	/* if a full grid render in one go */
 	if (ground_items_grid_cols*ground_items_grid_rows == ITEMS_PER_BAG)
-		rendergrid(ground_items_grid_cols,ground_items_grid_rows,0,0,GRIDSIZE,GRIDSIZE);
+		rendergrid(ground_items_grid_cols, ground_items_grid_rows, grid_sep_x, grid_sep_y, GRIDSIZE, GRIDSIZE);
 	/* otherwise don't render the extra slots */
 	else
 	{
 		int remainder = ITEMS_PER_BAG - (ground_items_grid_cols*(ground_items_grid_rows-1));
-		rendergrid(ground_items_grid_cols,ground_items_grid_rows-1,0,0,GRIDSIZE,GRIDSIZE);
-		rendergrid(remainder, 1, 0, GRIDSIZE*(ground_items_grid_rows-1), GRIDSIZE, GRIDSIZE);
+		rendergrid(ground_items_grid_cols, ground_items_grid_rows - 1, grid_sep_x, grid_sep_y, GRIDSIZE, GRIDSIZE);
+		rendergrid(remainder, 1, grid_sep_x, grid_sep_y + GRIDSIZE*(ground_items_grid_rows-1), GRIDSIZE, GRIDSIZE);
 	}
 
 	glBegin(GL_LINE_LOOP);
@@ -605,7 +623,7 @@ CHECK_GL_ERRORS();
 static int click_ground_items_handler(window_info *win, int mx, int my, Uint32 flags)
 {
 	int pos;
-	Uint8 str[10];
+	Uint8 str[6];
 	int right_click = flags & ELW_RIGHT_MOUSE;
 	int ctrl_on = flags & KMOD_CTRL;
 	int yoffset = get_window_scroll_pos(win->window_id);
@@ -618,12 +636,11 @@ static int click_ground_items_handler(window_info *win, int mx, int my, Uint32 f
 	if(right_click) {
 		if(item_dragged != -1) {
 			item_dragged = -1;
-		} else if(item_action_mode == ACTION_LOOK) {
-			item_action_mode = ACTION_WALK;
-		} else {
-			item_action_mode = ACTION_LOOK;
+			return 1;
+		} else if(is_gamewin_look_action()) {
+			clear_gamewin_look_action();
+			return 1;
 		}
-		return 1;
 	}
 
 	// see if we clicked on the "Get All" box
@@ -633,7 +650,8 @@ static int click_ground_items_handler(window_info *win, int mx, int my, Uint32 f
 		return 1;
 	}
 
-	pos = (my<0) ?-1 :get_mouse_pos_in_grid(mx,my+yoffset+1,ground_items_grid_cols,ground_items_grid_rows,0,0,GRIDSIZE,GRIDSIZE);
+	pos = (my<0) ?-1 :get_mouse_pos_in_grid(mx, my + yoffset + 1,
+		ground_items_grid_cols, ground_items_grid_rows, grid_sep_x, grid_sep_y, GRIDSIZE, GRIDSIZE);
 
 	if(pos==-1 || pos>=ITEMS_PER_BAG){
 	} else
@@ -642,13 +660,13 @@ static int click_ground_items_handler(window_info *win, int mx, int my, Uint32 f
 			str[0] = DROP_ITEM;
 			str[1] = item_dragged;
 			*((Uint32 *) (str + 2)) = SDL_SwapLE32(item_quantity);
-			my_tcp_send(my_socket, str, 6);
+			my_tcp_send(str, 6);
 			do_drop_item_sound();
 		}
-	} else if(item_action_mode==ACTION_LOOK) {
+	} else if(right_click || is_gamewin_look_action()) {
 		str[0]= LOOK_AT_GROUND_ITEM;
 		str[1]= ground_item_list[pos].pos;
-		my_tcp_send(my_socket,str,2);
+		my_tcp_send(str, 2);
 	} else {
 		int quantity;
 		quantity= ground_item_list[pos].quantity;
@@ -657,7 +675,7 @@ static int click_ground_items_handler(window_info *win, int mx, int my, Uint32 f
 		str[0]= PICK_UP_ITEM;
 		str[1]= ground_item_list[pos].pos;
 		*((Uint32 *)(str+2))= SDL_SwapLE32(quantity);
-		my_tcp_send(my_socket,str,6);
+		my_tcp_send(str, 6);
 		do_get_item_sound();
 	}
 
@@ -667,14 +685,15 @@ static int click_ground_items_handler(window_info *win, int mx, int my, Uint32 f
 
 static int mouseover_ground_items_handler(window_info *win, int mx, int my) {
 	int yoffset = get_window_scroll_pos(win->window_id);
-	int pos = (yoffset>my) ?-1 :get_mouse_pos_in_grid(mx, my+1, ground_items_grid_cols, ground_items_grid_rows, 0, 0, GRIDSIZE, GRIDSIZE);
+	int pos = (yoffset>my) ?-1 :get_mouse_pos_in_grid(mx, my + 1, ground_items_grid_cols,
+		ground_items_grid_rows, grid_sep_x, grid_sep_y, GRIDSIZE, GRIDSIZE);
 
 	if(pos!=-1 && pos<ITEMS_PER_BAG && ground_item_list[pos].quantity) {
 		Uint16 item_id = ground_item_list[pos].id;
 		int image_id = ground_item_list[pos].image_id;
 		if (show_item_desc_text && item_info_available() && (get_item_count(item_id, image_id) == 1))
 			item_desc_str = get_item_description(item_id, image_id);
-		if(item_action_mode==ACTION_LOOK) {
+		if(is_gamewin_look_action()) {
 			elwin_mouse=CURSOR_EYE;
 		} else {
 			elwin_mouse=CURSOR_PICK;
@@ -689,8 +708,8 @@ static int mouseover_ground_items_handler(window_info *win, int mx, int my) {
 /* dynamically adjust the grid and the scroll bar when the window resizes */
 static int resize_ground_items_handler(window_info *win, int width, int height)
 {
-	ground_items_visible_grid_cols = (win->len_x / GRIDSIZE) - 1;
-	ground_items_visible_grid_rows = (win->len_y / GRIDSIZE);
+	ground_items_visible_grid_cols = ((win->len_x - grid_sep_x) / GRIDSIZE) - 1;
+	ground_items_visible_grid_rows = ((win->len_y - 2 * grid_sep_y) / GRIDSIZE);
 
 	/* let the width lead */
 	ground_items_grid_cols = ground_items_visible_grid_cols;
@@ -707,7 +726,7 @@ static int resize_ground_items_handler(window_info *win, int width, int height)
 			ground_items_grid_cols++;
 	}
 
-	set_window_scroll_len(win->window_id, ground_items_grid_rows*GRIDSIZE-win->len_y);
+	set_window_scroll_len(win->window_id, ground_items_grid_rows * GRIDSIZE + 2 * grid_sep_y - win->len_y);
 	return 0;
 }
 
@@ -721,37 +740,60 @@ static int ui_scale_ground_items_handler(window_info * win)
 		ground_items_visible_grid_rows = 10;
 	}
 	GRIDSIZE = (int)(0.5 + 33 * win->current_scale);
-	set_window_min_size(win->window_id, (min_grid_cols+1)*GRIDSIZE, min_grid_rows*GRIDSIZE);
+	grid_sep_x = (int)(0.5 + 2 * win->current_scale);
+	grid_sep_y = (int)(0.5 + 2 * win->current_scale);
+	set_window_min_size(win->window_id, grid_sep_x + (min_grid_cols + 1) * GRIDSIZE, 2 * grid_sep_y + min_grid_rows * GRIDSIZE);
 	set_window_scroll_inc(win->window_id, GRIDSIZE/3);
 	set_window_scroll_yoffset(win->window_id, GRIDSIZE);
 	set_window_scroll_pos(win->window_id, current_scroll_pos * GRIDSIZE/3);
-	resize_window(win->window_id, (ground_items_visible_grid_cols+1)*GRIDSIZE, ground_items_visible_grid_rows*GRIDSIZE);
+	resize_window(win->window_id, grid_sep_x + (ground_items_visible_grid_cols + 1) * GRIDSIZE,
+		2 * grid_sep_y + ground_items_visible_grid_rows * GRIDSIZE);
+	return 1;
+}
+
+void server_close_bag(void)
+{
+	hide_window_MW(MW_BAGS);
+	clear_groundlist();
+	clear_was_open_MW(MW_BAGS);
+}
+
+void client_close_bag(void)
+{
+	const unsigned char protocol_close_bag = S_CLOSE_BAG;
+	my_tcp_send(&protocol_close_bag, 1);
+	server_close_bag();
+}
+
+static int close_handler(window_info *win)
+{
+	client_close_bag();
 	return 1;
 }
 
 static void draw_pick_up_menu(void)
 {
-	if(ground_items_win < 0){
-		int our_root_win = -1;
-		if (!windows_on_top) {
-			our_root_win = game_root_win;
-		}
-		ground_items_win= create_window(win_bag, our_root_win, 0, ground_items_menu_x, ground_items_menu_y,
-			0, 0, ELW_USE_UISCALE|ELW_SCROLLABLE|ELW_RESIZEABLE|ELW_WIN_DEFAULT);
+	int ground_items_win = get_id_MW(MW_BAGS);
 
-		set_window_custom_scale(ground_items_win, &custom_scale_factors.bags);
+	if(ground_items_win < 0){
+		ground_items_win = create_window(win_bag, (not_on_top_now(MW_BAGS) ?game_root_win : -1), 0, get_pos_x_MW(MW_BAGS), get_pos_y_MW(MW_BAGS),
+			0, 0, ELW_USE_UISCALE|ELW_SCROLLABLE|ELW_RESIZEABLE|ELW_WIN_DEFAULT);
+		set_id_MW(MW_BAGS, ground_items_win);
+
+		set_window_custom_scale(ground_items_win, MW_BAGS);
 		set_window_handler(ground_items_win, ELW_HANDLER_DISPLAY, &display_ground_items_handler );
 		set_window_handler(ground_items_win, ELW_HANDLER_PRE_DISPLAY, &pre_display_ground_items_handler );
 		set_window_handler(ground_items_win, ELW_HANDLER_CLICK, &click_ground_items_handler );
 		set_window_handler(ground_items_win, ELW_HANDLER_MOUSEOVER, &mouseover_ground_items_handler );
 		set_window_handler(ground_items_win, ELW_HANDLER_RESIZE, &resize_ground_items_handler );
-		set_window_handler(ground_items_win, ELW_HANDLER_CLOSE, &clear_groundlist );
+		set_window_handler(ground_items_win, ELW_HANDLER_CLOSE, &close_handler );
 		set_window_handler(ground_items_win, ELW_HANDLER_UI_SCALE, &ui_scale_ground_items_handler );
 
 		if (ground_items_win >=0 && ground_items_win < windows_list.num_windows)
 			ui_scale_ground_items_handler(&windows_list.window[ground_items_win]);
 		else
 			return;
+		check_proportional_move(MW_BAGS);
 
 	} else {
 		show_window(ground_items_win);
