@@ -48,14 +48,14 @@
  G, and B in them, will always result in white.  They work well for magic and
  light sources.  "Non-glowing" particles blend with an average (as with
  glowing, weighted proportional to transparency).  Infinite accumulation of
- non-glowing particles results purely in the color of the particle itself. 
+ non-glowing particles results purely in the color of the particle itself.
  They work well for things like dust, debris, and smoke.
 
  All effects (except fireflies and leaves/petals, which don't need it) have a
  level of detail flag.  This is the maximum level of detail to use.  The
  number of particles that the effect will use is roughly proportional to its
  level of detail, although different effects may use more or less particles
- than others.  Naturally, lower level of detail is faster but poorer quality. 
+ than others.  Naturally, lower level of detail is faster but poorer quality.
  Eye Candy also has a built-in particle limit.  As you near this limit, it
  automatically tells effects to lower their level of detail.  Any new
  particles that they create will be done according to the new LOD.  Lastly,
@@ -103,7 +103,7 @@
 #ifndef NOMINMAX
 #define NOMINMAX
 #endif
-#include <windows.h>     
+#include <windows.h>
 #include <float.h>
 #else
 #include <sys/time.h>
@@ -295,14 +295,13 @@ namespace ec
 				y = 0.0f;
 				z = 0.0f;
 			}
-			;
 			Vec3(coord_t _x, coord_t _y, coord_t _z)
 			{
 				x = _x;
 				y = _y;
 				z = _z;
 			}
-			;
+			Vec3(const Vec3& v) = default;
 			~Vec3()
 			{
 			}
@@ -314,7 +313,7 @@ namespace ec
 				y += rhs.y;
 				z += rhs.z;
 
-				if (!is_valid()) 
+				if (!is_valid())
 				{
 					x = 0.0;
 					y = 0.0;
@@ -331,7 +330,7 @@ namespace ec
 				y -= rhs.y;
 				z -= rhs.z;
 
-				if (!is_valid()) 
+				if (!is_valid())
 				{
 					x = 0.0;
 					y = 0.0;
@@ -413,15 +412,7 @@ namespace ec
 			}
 			;
 
-			Vec3 operator=(const Vec3 rhs)
-			{
-				x = rhs.x;
-				y = rhs.y;
-				z = rhs.z;
-
-				return *this;
-			}
-			;
+			Vec3& operator=(const Vec3& rhs) = default;
 
 			Vec3 operator-()
 			{
@@ -1150,7 +1141,7 @@ namespace ec
 	};
 
 	/*!
-	 \brief A gradient mover which whirls/pinches particles. 
+	 \brief A gradient mover which whirls/pinches particles.
 
 	 If spiral speed is equal to -pinch_rate, particles will follow a rough
 	 circle.  A greater magnitude pinch and they'll go inwards.  A lesser
@@ -1221,7 +1212,7 @@ namespace ec
 	};
 
 	/*!
-	 \brief A bounding range composed of a series of angles and radii forming a 
+	 \brief A bounding range composed of a series of angles and radii forming a
 	 polygon that is smoothly interpolated.
 	 */
 
@@ -1797,7 +1788,7 @@ namespace ec
 			}
 			;
 
-			virtual Vec3 get_force_gradient(Particle& p) = 0;
+			virtual Vec3 get_force_gradient(Particle& p) const = 0;
 
 			coord_t max_distance;
 			coord_t max_distance_squared;
@@ -1822,7 +1813,7 @@ namespace ec
 			}
 			;
 
-			virtual Vec3 get_force_gradient(Particle& p);
+			virtual Vec3 get_force_gradient(Particle& p) const;
 
 			Vec3* pos;
 	};
@@ -1848,7 +1839,7 @@ namespace ec
 			}
 			;
 
-			virtual Vec3 get_force_gradient(Particle& p);
+			virtual Vec3 get_force_gradient(Particle& p) const;
 
 			Vec3* pos;
 			coord_t bottom;
@@ -1868,7 +1859,7 @@ namespace ec
 			}
 			;
 
-			virtual Vec3 get_force_gradient(Particle& p);
+			virtual Vec3 get_force_gradient(Particle& p) const;
 
 			Vec3* start;
 			Vec3* end;
@@ -1894,7 +1885,7 @@ namespace ec
 			}
 			;
 
-			virtual Vec3 get_force_gradient(Particle& p);
+			virtual Vec3 get_force_gradient(Particle& p) const;
 
 			Vec3* pos;
 	};
@@ -1938,7 +1929,7 @@ namespace ec
 			}
 			;
 
-			virtual Vec3 get_force_gradient(Particle& p);
+			virtual Vec3 get_force_gradient(Particle& p) const;
 
 			Vec3 start;
 			Vec3 end;
@@ -2022,13 +2013,11 @@ namespace ec
 			virtual bool idle(const Uint64 usec) = 0;
 			virtual void draw(const Uint64 usec)
 			{
-				for (std::map<Particle*, bool>::iterator iter2 =
-					particles.begin(); iter2 != particles.end(); iter2++)
+				for (auto& iter2: particles)
 				{
-					for (std::vector<Obstruction*>::iterator iter =
-						obstructions->begin(); iter != obstructions->end(); iter++)
+					for (auto obstruction: *obstructions)
 					{
-						(*iter)->get_force_gradient(*(iter2->first));
+						obstruction->get_force_gradient(*iter2.first);
 					}
 				}
 			}
@@ -2050,11 +2039,11 @@ namespace ec
 			virtual Uint64 get_expire_time()
 			{	return 0x8000000000000000ull;};
 
-#ifdef CLUSTER_INSIDES  
+#ifdef CLUSTER_INSIDES
 			short getCluster () const
 			{
 				if (!pos)
-				return 0;
+					return 0;
 				return get_cluster (int (pos->x / 0.5f), int (-pos->z / 0.5f));
 			}
 
@@ -2091,7 +2080,7 @@ namespace ec
 
 		 The EyeCandy object (there should only ever be one) encapsulates all of the
 		 effects and particles that will occur in a program.  There are numerous
-		 options, flags, and settings that can be set for the eye candy object. 
+		 options, flags, and settings that can be set for the eye candy object.
 		 A few critical notes:
 
 		 1) When initializing the object, be sure to load_textures().
@@ -2107,6 +2096,8 @@ namespace ec
 		class EyeCandy
 		{
 			public:
+			//! Linear attenuation factor for lights used in eye candy effects
+			static constexpr const float light_linear_attenuation = 1.0;
 
 			EyeCandy();
 			EyeCandy(int _max_particles);
@@ -2238,11 +2229,11 @@ namespace ec
 			~Logger()
 			{	delete rdbuf();};
 
-			void log_text(const std::string message)
+			void log_text(const std::string& message)
 			{	((LoggerBuf*)rdbuf())->logs.push_back(message);};
-			void log_warning(const std::string message)
+			void log_warning(const std::string& message)
 			{	log_text("WARNING: " + message + "\n");};
-			void log_error(const std::string message)
+			void log_error(const std::string& message)
 			{	log_text("ERROR: " + message + "\n"); ec_error_status = true;};
 			std::vector<std::string> fetch()
 			{	const std::vector<std::string> ret(((LoggerBuf*)rdbuf())->logs); ((LoggerBuf*)rdbuf())->logs.clear(); return ret;};
