@@ -17,6 +17,16 @@
 #include "fsaa/fsaa.h"
 #endif	/* FSAA */
 
+typedef struct
+{
+	int profile;
+	int major;
+	int minor;
+	char version_string[80];
+} GLContextInfo;
+
+static GLContextInfo gl_context_info = { 0, 0, 0, { 0 } };
+
 int window_width = 640;
 int window_height = 480;
 static float window_highdpi_scale_width = 1.0f;
@@ -26,15 +36,13 @@ SDL_Window *el_gl_window = NULL;
 static SDL_GLContext el_gl_context = NULL;
 static SDL_Surface *icon_bmp = NULL;
 static SDL_version el_gl_linked;
+static int log_next_window_resize = 0;
 
 int bpp = 0;
 int have_stencil = 1;
 int video_mode = 0;
 int video_user_width = 640;
 int video_user_height = 480;
-#ifdef WINDOWS
-int disable_window_adjustment = 0;
-#endif
 int full_screen = 0;
 
 int use_compiled_vertex_array = 0;
@@ -43,7 +51,9 @@ int use_frame_buffer = 0;
 int use_mipmaps = 0;
 int use_draw_range_elements = 1;
 float anisotropic_filter = 1.0f;
+int enable_screensaver = 0;
 int disable_gamma_adjust = 0;
+int disable_focus_clickthrough = 0;
 float gamma_var = 1.00f;
 float perspective = 0.15f;
 float near_plane = 0.1f; // don't cut off anything
@@ -51,7 +61,7 @@ float far_plane = 100.0;   // LOD helper. Cull distant objects. Lower value == h
 float far_reflection_plane = 100.0;   // LOD helper. Cull distant reflected objects. Lower value == higher framerates.
 int gl_extensions_loaded = 0;
 
-static void setup_video_mode(int fs, int mode)
+static void get_window_size(int mode, int *width, int *height)
 {
 	/* Video mode 0 is user defined size (via video_user_width and video_user_height)
 	 * Video mode 1 and above are defined in the video_modes array where mode 1 is at position 0
@@ -66,103 +76,8 @@ static void setup_video_mode(int fs, int mode)
 	if (index < 0 || index >= video_modes_count)
 		index = 0;
 
-	if (fs) // Fullscreen
-	{
-		if (mode == 0)
-		{
-			window_width = video_user_width;
-			window_height = video_user_height;
-			bpp = 0;
-		} else {
-			window_width = video_modes[index].width;
-			window_height = video_modes[index].height;
-			bpp = video_modes[index].bpp;
-		}
-	} 
-	else // Windowed mode
-	{
-		int new_width = video_modes[index].width;
-		int new_height = video_modes[index].height;
-
-		if (mode == 0)
-		{
-			new_width = video_user_width;
-			new_height = video_user_height;
-		} 
-#ifdef WINDOWS
-		else if (!disable_window_adjustment)
-		{
-			// Window size magic:
-			// Try to get the work area and adjust the window to that size minus the border size
-
-			HWND hwnd;
-			HMONITOR monitor;
-			MONITORINFO monitorInfo;
-			int monitor_width = 0;
-			int monitor_height = 0;
-
-			hwnd = GetActiveWindow();
-			// Get the monitor closest to the window (if we already have one)
-			monitor = MonitorFromWindow(hwnd, MONITOR_DEFAULTTOPRIMARY);
-
-			// Get the size of the work area
-			monitorInfo.cbSize = sizeof(monitorInfo);
-			GetMonitorInfo(monitor, &monitorInfo);
-			monitor_width = monitorInfo.rcWork.right - monitorInfo.rcWork.left;
-			monitor_height = monitorInfo.rcWork.bottom - monitorInfo.rcWork.top;
-
-			// Clip the window size to the work area
-			if (new_width >= monitor_width)
-				new_width = monitor_width;
-			if (new_height >= monitor_height)
-				new_height = monitor_height;
-
-			// try to move the window to make everything visible
-			if (hwnd != NULL)
-			{
-				WINDOWPLACEMENT wpl;
-				int dx, dy;
-
-				wpl.length = sizeof(wpl);
-				GetWindowPlacement(hwnd, &wpl);
-
-				dx = wpl.rcNormalPosition.left + new_width - monitorInfo.rcWork.right;
-				dy = wpl.rcNormalPosition.top + new_height - monitorInfo.rcWork.bottom;
-
-				if (dx < 0) dx = 0;
-				if (dy < 0) dy = 0;
-
-				if (dx || dy) {
-					wpl.rcNormalPosition.left -= dx;
-					wpl.rcNormalPosition.top -= dy;
-					wpl.showCmd = SW_SHOWNORMAL;
-					SetWindowPlacement(hwnd, &wpl);
-				}
-			}
-
-			// Adjust the window size to fit into the border and title bar
-			new_width -= 2 * GetSystemMetrics(SM_CXFIXEDFRAME);
-			new_height -= 2 * GetSystemMetrics(SM_CYFIXEDFRAME) + GetSystemMetrics(SM_CYCAPTION);
-		}
-#endif
-
-		if (window_width != new_width || window_height != new_height)
-		{
-			char modestr[100];
-			char str[100];
-			safe_snprintf(modestr, sizeof(modestr), "%dx%d", new_width, new_height);
-			safe_snprintf(str, sizeof(str), window_size_adjusted_str, modestr);
-			LOG_TO_CONSOLE(c_yellow1,str);
-			LOG_DEBUG("%s",str);
-		}
-
-		window_width = new_width;
-		window_height = new_height;
-		bpp = 0; // autodetect
-	}
-#ifndef WINDOWS
-	bpp=0;//under X, we can't change the desktop BPP
-#endif
+	*width = (mode == 0) ?video_user_width :video_modes[index].width;
+	*height = (mode == 0) ?video_user_height :video_modes[index].height;
 }
 
 static void load_window_icon(void)
@@ -180,13 +95,100 @@ static void load_window_icon(void)
 	free(str_buf);
 }
 
+// check if the window fits within the available space and resize if not
+// check if the window is on-screen and move it if not
+enum size_and_position_enum { DO_SIZE = 0, DO_POSITION, DO_SIZE_AND_POSITION };
+static void fix_window_size_and_position(enum size_and_position_enum mode)
+{
+#if SDL_VERSION_ATLEAST(2, 0, 5)
+	SDL_Rect usable_rect;
+	int display_index = SDL_GetWindowDisplayIndex(el_gl_window);
+	if (SDL_VERSIONNUM(el_gl_linked.major, el_gl_linked.minor, el_gl_linked.patch) < 2005)
+		return;
+	if ((display_index >= 0) && (SDL_GetDisplayUsableBounds(display_index, &usable_rect) == 0))
+	{
+		int top, left, bottom, right;
+		if (SDL_GetWindowBordersSize(el_gl_window, &top, &left, &bottom, &right) == 0)
+		{
+			int ww, wh;
+			SDL_GL_GetDrawableSize(el_gl_window, &ww, &wh);
+
+			if ((mode == DO_SIZE) || (mode == DO_SIZE_AND_POSITION))
+			{
+				int do_resize = 0;
+				if ((ww + left + right) > usable_rect.w)
+				{
+					ww = usable_rect.w - left - right;
+					do_resize = 1;
+				}
+				if ((wh + top + bottom) > usable_rect.h)
+				{
+					wh = usable_rect.h - top - bottom;
+					do_resize = 1;
+				}
+				if (do_resize)
+				{
+					SDL_SetWindowSize(el_gl_window, ww, wh);
+					SDL_RestoreWindow(el_gl_window);
+				}
+			}
+
+			if ((mode == DO_POSITION) || (mode == DO_SIZE_AND_POSITION))
+			{
+				int posx, posy;
+				SDL_GetWindowPosition(el_gl_window, &posx, &posy);
+				if (((posx - left) < usable_rect.x) ||
+					((posy - top) < usable_rect.y) ||
+					((posx + ww + left + right) > (usable_rect.x + usable_rect.w)) ||
+					((posy + wh + top + bottom) > (usable_rect.y + usable_rect.h)))
+				{
+					posx = usable_rect.x + left + (usable_rect.w - ww - left - right) / 2;
+					posy = usable_rect.y + top + (usable_rect.h - wh - top - bottom) / 2;
+					SDL_SetWindowPosition(el_gl_window, posx, posy);
+				}
+			}
+		}
+	}
+#endif
+}
+
+void update_SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH(void)
+{
+#if SDL_VERSION_ATLEAST(2, 0, 5)
+	if (SDL_VERSIONNUM(el_gl_linked.major, el_gl_linked.minor, el_gl_linked.patch) >= 2005)
+	{
+		if (disable_focus_clickthrough)
+			SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "0");
+		else
+			SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "1");
+	}
+#endif
+}
+
 void init_video(void)
 {
-	char str[400];
+	int versions_to_try[][3] = {
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 3, 3 }, // OpenGL 3.3 compatibility profile
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 3, 2 }, // OpenGL 3.2 compatibility profile
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 3, 1 }, // OpenGL 3.1
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 3, 0 }, // OpenGL 3.0
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 2, 1 }, // OpenGL 2.1
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 2, 0 }, // OpenGL 2.0. Not sure if lower versions will still work
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 1, 5 }, // OpenGL 1.5
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 1, 4 }, // OpenGL 1.4
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 1, 3 }, // OpenGL 1.3
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 1, 2 }, // OpenGL 1.2
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 1, 1 }, // OpenGL 1.1
+		{ SDL_GL_CONTEXT_PROFILE_COMPATIBILITY, 1, 0 }, // OpenGL 1.0
+	};
+	int nr_versions_to_try = sizeof(versions_to_try) / sizeof(*versions_to_try);
+
+	int target_width = 0, target_height = 0;
 	int rgb_size[3];
 	Uint32 flags;
 
-	setup_video_mode(full_screen, video_mode);
+	if (!full_screen)
+		get_window_size(video_mode, &target_width, &target_height);
 
 	if (SDL_Init(SDL_INIT_VIDEO) < 0)
 	{
@@ -267,11 +269,12 @@ void init_video(void)
 #ifdef	FSAA
 	if (fsaa > 1)
 	{
+		char str[400];
 		SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
 		SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, fsaa);
 		glDisable(GL_MULTISAMPLE);
 
-		el_gl_window = SDL_CreateWindow("Eternal Lands", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, (full_screen)?0:window_width, (full_screen)?0:window_height, flags);
+		el_gl_window = SDL_CreateWindow("Eternal Lands", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, target_width, target_height, flags);
 		if (el_gl_window == NULL)
 		{
 			safe_snprintf(str, sizeof(str), "Can't use fsaa mode x%d, disabling it.", fsaa);
@@ -287,7 +290,7 @@ void init_video(void)
 	//try to find a stencil buffer
 	if (el_gl_window == NULL)
 	{
-		el_gl_window = SDL_CreateWindow("Eternal Lands", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, (full_screen)?0:window_width, (full_screen)?0:window_height, flags);
+		el_gl_window = SDL_CreateWindow("Eternal Lands", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, target_width, target_height, flags);
 		if (el_gl_window == NULL)
 		{
 			LOG_TO_CONSOLE(c_red1,no_hardware_stencil_str);
@@ -299,7 +302,7 @@ void init_video(void)
             }
 			SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,16);
 			SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE,0);
-			el_gl_window = SDL_CreateWindow("Eternal Lands", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, (full_screen)?0:window_width, (full_screen)?0:window_height, flags);
+			el_gl_window = SDL_CreateWindow("Eternal Lands", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, target_width, target_height, flags);
 			if (el_gl_window == NULL)
 			{
 				LOG_ERROR("%s: %s\n", fail_opengl_mode, SDL_GetError());
@@ -311,7 +314,27 @@ void init_video(void)
 		}
 	}
 
-	el_gl_context = SDL_GL_CreateContext(el_gl_window);
+	for (int i = 0; i < nr_versions_to_try; ++i)
+	{
+		SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, versions_to_try[i][0]);
+		SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, versions_to_try[i][1]);
+		SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, versions_to_try[i][2]);
+		el_gl_context = SDL_GL_CreateContext(el_gl_window);
+		if (el_gl_context)
+		{
+			// OpenGL deprecated a lot of features we use in OpenGL 3.0, and removed them in
+			// 3.1. OpenGL. OpenGL 3.2 introduced compatibility profiles, and 3.1 has the
+			// ARB_compativility extension, but some systems fail with an OpenGL context >= 3.0
+			// even when they report compatibility, or when the context is OpenGL 3.0 which should
+			// still support the old functionality, Therefore, try a deprecated function here,
+			// and if it fails, downgrade to a lower version context.
+			const GLubyte* extensions_str = glGetString(GL_EXTENSIONS);
+			DO_CHECK_GL_ERRORS();
+			if (extensions_str)
+				// That seemd to work, hopefully we're good
+				break;
+		}
+	}
 	if (el_gl_context == NULL)
 	{
 		LOG_ERROR("%s: %s\n", "SDL_GL_CreateContext() Failed", SDL_GetError());
@@ -329,11 +352,34 @@ void init_video(void)
 			DO_CHECK_GL_ERRORS();
 			LOG_ERROR("%s: %s\n", error_str, SDL_GetError());
 			SDL_Quit();
-			FATAL_ERROR_WINDOW(error_str);
+			FATAL_ERROR_WINDOW("%s", error_str);
 			exit(1);
 		}
 	}
 
+	// Set the GL verion info, so we can use different code paths for rendering if necessary
+	SDL_GL_GetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, &gl_context_info.profile);
+	SDL_GL_GetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, &gl_context_info.major);
+	SDL_GL_GetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, &gl_context_info.minor);
+	switch (gl_context_info.profile)
+	{
+		case SDL_GL_CONTEXT_PROFILE_CORE:
+			safe_snprintf(gl_context_info.version_string, sizeof(gl_context_info.version_string),
+				"OpenGL Core %d.%d", gl_context_info.major, gl_context_info.minor);
+			break;
+		case SDL_GL_CONTEXT_PROFILE_COMPATIBILITY:
+			safe_snprintf(gl_context_info.version_string, sizeof(gl_context_info.version_string),
+				"OpenGL Compatibility %d.%d", gl_context_info.major, gl_context_info.minor);
+			break;
+		case SDL_GL_CONTEXT_PROFILE_ES:
+			safe_snprintf(gl_context_info.version_string, sizeof(gl_context_info.version_string),
+				"OpenGL ES %d.%d", gl_context_info.major, gl_context_info.minor);
+			break;
+		default:
+			safe_snprintf(gl_context_info.version_string, sizeof(gl_context_info.version_string),
+				"Unknown %d.%d", gl_context_info.major, gl_context_info.minor);
+	}
+
 	// set the minimum size for the window, this is too small perhaps but a config option
 	SDL_SetWindowMinimumSize(el_gl_window, 640,  480);
 
@@ -348,18 +394,24 @@ void init_video(void)
 		}
 	}
 
+	if (!full_screen)
+		fix_window_size_and_position(DO_SIZE_AND_POSITION);
+
 	// get the windos size, these variables are used globaly
 	update_window_size_and_scale();
+	// even though no windows have been created, their starting position need to be adjusted
+	if (!full_screen)
+		move_windows_proportionally((float)window_width / (float)target_width, (float)window_height / (float)target_height);
 
 	// enable V-SYNC, choosing active as a preference
 	if (SDL_GL_SetSwapInterval(-1) < 0)
 		SDL_GL_SetSwapInterval(1);
 
-	// set the hint that clicks that focus the window, pass through for action too
-#if SDL_VERSION_ATLEAST(2, 0, 5)
-	if (SDL_VERSIONNUM(el_gl_linked.major, el_gl_linked.minor, el_gl_linked.patch) >= 2005)
-		SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "1");
-#endif
+	// set the hint that determines if clicks that focus the window, pass through for action too
+	update_SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH();
+
+	if (enable_screensaver)
+		SDL_EnableScreenSaver();
 
 	// set the gamma of we are controlling it
 	if (!disable_gamma_adjust)
@@ -952,22 +1004,42 @@ void resize_root_window(void)
 
 int switch_video(int mode, int full_screen)
 {
-	Uint32 old_window_width = window_width, old_window_height = window_height;
-	video_mode=mode;
-	setup_video_mode(full_screen, mode);
-	SDL_RestoreWindow(el_gl_window);
+	video_mode = mode;
 	if (full_screen)
 		SDL_SetWindowFullscreen(el_gl_window, SDL_WINDOW_FULLSCREEN_DESKTOP);
 	else
 	{
-		SDL_SetWindowFullscreen(el_gl_window, 0);
-		SDL_SetWindowSize(el_gl_window, window_width, window_height);
+		int target_width, target_height;
+		get_window_size(mode, &target_width, &target_height);
+		set_client_window_size(target_width, target_height);
 	}
-	update_window_size_and_scale();
-	resize_all_root_windows(old_window_width, window_width, old_window_height, window_height);
 	return 1;
 }
 
+//	Set the window size as specified without changing the window mode.
+void set_client_window_size(int width, int height)
+{
+	// limit the window size to the actual available space
+#if SDL_VERSION_ATLEAST(2, 0, 5)
+	if (SDL_VERSIONNUM(el_gl_linked.major, el_gl_linked.minor, el_gl_linked.patch) >= 2005)
+	{
+		SDL_Rect rect;
+		int display_index = SDL_GetWindowDisplayIndex(el_gl_window);
+		int top, left, bottom, right;
+		SDL_GetWindowBordersSize(el_gl_window, &top, &left, &bottom, &right);
+		SDL_GetDisplayUsableBounds(display_index, &rect);
+		width = (width > (rect.w - left - right)) ?rect.w - left - right :width;
+		height = (height > (rect.h - top - bottom)) ?rect.h - top - bottom :height;
+	}
+#endif
+
+	log_next_window_resize = 1;
+	SDL_RestoreWindow(el_gl_window);
+	SDL_SetWindowFullscreen(el_gl_window, 0);
+	SDL_SetWindowSize(el_gl_window, width, height);
+	fix_window_size_and_position(DO_POSITION);
+}
+
 //	Get a single value for highhdpi scaling.
 float get_highdpi_scale(void)
 {
@@ -986,12 +1058,24 @@ void highdpi_scale(int *width, int *height)
 //	You need to ration of SDL_GL_GetDrawableSize()/SDL_GetWindowSize() to scale the mouse location.
 void update_window_size_and_scale(void)
 {
+	float old_width = window_highdpi_scale_width, old_height = window_highdpi_scale_height;
 	int non_dpi_w, non_dpi_h;
 	SDL_GetWindowSize(el_gl_window, &non_dpi_w, &non_dpi_h);
 	SDL_GL_GetDrawableSize(el_gl_window, &window_width, &window_height);
 	window_highdpi_scale_width = (float)window_width / (float)non_dpi_w;
 	window_highdpi_scale_height = (float)window_height / (float)non_dpi_h;
-	update_highdpi_auto_scaling();
+	if ((old_width != window_highdpi_scale_width) || (old_height != window_highdpi_scale_height))
+		update_highdpi_auto_scaling();
+	if (log_next_window_resize)
+	{
+		char modestr[100];
+		char str[100];
+		safe_snprintf(modestr, sizeof(modestr), "%dx%d", window_width, window_height);
+		safe_snprintf(str, sizeof(str), window_size_adjusted_str, modestr);
+		LOG_TO_CONSOLE(c_yellow1, str);
+		LOG_DEBUG("%s",str);
+		log_next_window_resize = 0;
+	}
 }
 
 void toggle_full_screen(void)
@@ -1012,3 +1096,55 @@ int print_gl_errors(const char *file, int line)
 	return anyErr;
 }
 
+void gl_window_cleanup(void)
+{
+	if (el_gl_window != NULL)
+	{
+		SDL_DestroyWindow(el_gl_window);
+		el_gl_window = NULL;
+	}
+	if (icon_bmp != NULL)
+	{
+		SDL_FreeSurface(icon_bmp);
+		icon_bmp = NULL;
+	}
+}
+
+const char* gl_context_version_string()
+{
+	return gl_context_info.version_string;
+}
+
+int gl_context_version()
+{
+	return 100*gl_context_info.major + gl_context_info.minor;
+}
+
+int max_supported_glsl_version()
+{
+	switch (gl_context_info.profile)
+	{
+		case SDL_GL_CONTEXT_PROFILE_COMPATIBILITY:
+		case SDL_GL_CONTEXT_PROFILE_CORE:
+			switch (gl_context_version())
+			{
+				case 303: return 330;
+				case 302: return 150;
+				case 301: return 140;
+				case 300: return 130;
+				case 201: return 120;
+				case 200: return 110;
+				default: return 0;
+			}
+		case SDL_GL_CONTEXT_PROFILE_ES:
+			switch (10*gl_context_info.major + gl_context_info.minor)
+			{
+				case 31: return 10310;
+				case 30: return 10300;
+				case 20: return 10100;
+				default: return 0;
+			}
+		default:
+			return 0;
+	}
+}
