@@ -19,6 +19,7 @@
 #include "asc.h"
 #include "chat.h"
 #include "context_menu.h"
+#include "console.h"
 #include "counters.h"
 #include "errors.h"
 #include "font.h"
@@ -27,11 +28,15 @@
 #include "gamewin.h"
 #include "gl_init.h"
 #include "hud.h"
+#ifdef JSON_FILES
+#include "json_io.h"
+#endif
 #include "sound.h"
 #include "spells.h"
 #include "text.h"
 #include "translate.h"
 
+using namespace eternal_lands;
 
 namespace Indicators
 {
@@ -40,18 +45,20 @@ namespace Indicators
 	class Vars
 	{
 		public:
-			static const float zoom(void) { return scale; }
-			static const int space(void) { return (int)(0.5 + scale * 5); }
-			static const int border(void) { return (int)(0.5 + scale * 2); }
-			static const float font_x(void) { return DEFAULT_FONT_X_LEN; }
-			static const float font_y(void) { return DEFAULT_FONT_Y_LEN; }
-			static const int y_len(void) { return static_cast<int>(border() + zoom() * font_y() + 0.5); }
+			static float zoom(void) { return scale; }
+			static int space(void) { return (int)(0.5 + scale * 5); }
+			static int border(void) { return (int)(0.5 + scale * 2); }
+			static float font_x(void) { return FontManager::get_instance().max_width_spacing(font_cat); }
+			static float font_y(void) { return FontManager::get_instance().line_height(font_cat); }
+			static int y_len(void) { return static_cast<int>(border() + zoom() * font_y() + 0.5); }
 			static void set_scale(float new_scale) { scale = new_scale; }
+			static FontManager::Category const font_cat;
 		private:
 			static float scale;
 	};
 
 	float Vars::scale = 1.0;
+	FontManager::Category const Vars::font_cat = FontManager::Category::UI_FONT;
 
 	//	A class to hold the state for an individual, basic indicator.
 	//	Has a simple on / of state and no action.
@@ -68,7 +75,8 @@ namespace Indicators
 			virtual void set_active(bool new_active) { is_active = (new_active) ?1 :0; }
 			virtual bool not_active(void) const { return (is_active==0); }
 			virtual void set_over(void) { mouse_over = true; }
-			virtual int get_width(void) const { return indicator_text.size() * static_cast<int>(Vars::font_x() * Vars::zoom()); }
+			virtual int get_width(void) const { return indicator_width; }
+			virtual void ui_scale(void);
 			virtual ~Basic_Indicator(void) {}
 		protected:
 			std::string on_tooltip;
@@ -81,6 +89,7 @@ namespace Indicators
 			std::string context_menu_str;
 			int is_active;
 			bool mouse_over;
+			int indicator_width;
 	};
 
 
@@ -124,7 +133,7 @@ namespace Indicators
 				: indicators_win(-1), cm_menu_id(CM_INIT_VALUE),
 					cm_relocatable(0), x_len(0), y_len(0), default_location(true),
 					option_settings(0), position_settings(0), have_settings(false),
-					background_on(0), border_on(0) {}
+					background_on(0), border_on(0), no_indicators_str_width(0) {}
 			void init(void);
 			void destroy(void);
 			void show(void) { if (indicators_win >= 0) show_window (indicators_win); }
@@ -135,8 +144,12 @@ namespace Indicators
 			void click(int mx, Uint32 flags);
 			int cm_handler(window_info *win, int widget_id, int mx, int my, int option);
 			void set_settings(unsigned int opts, unsigned int pos) { option_settings = opts; position_settings = pos; have_settings = true;}
-			void get_settings(unsigned int *opts, unsigned int *pos);
-			void ui_scale_handler(window_info *win) { x_len = 0; y_len = 0; Vars::set_scale(win->current_scale); }
+			void get_settings(unsigned int *opts, unsigned int *pos) const;
+#ifdef JSON_FILES
+			void read_settings(const char *dict_name);
+			void write_settings(const char *dict_name) const;
+#endif
+			void ui_scale_handler(window_info *win);
 			int get_default_width(void);
 		private:
 			void set_win_flag(Uint32 flag, int state);
@@ -154,6 +167,7 @@ namespace Indicators
 			bool have_settings;
 			int background_on;
 			int border_on;
+			int no_indicators_str_width;
 			std::vector<Basic_Indicator *>::iterator get_over(int mx);
 			std::pair<int,int> get_default_location(void);
 			void change_width(int new_x_len);
@@ -166,7 +180,7 @@ namespace Indicators
 	//
 	Basic_Indicator::Basic_Indicator(const char *the_strings, int (*ctrl)(void), int (*unavailable)(void), int no)
 		: on_tooltip("Unset"), off_tooltip("Unset"), unavailable_tooltip("Unset"), cntr_func(ctrl), unavailable_func(unavailable),
-			indicator_text("*"), is_active(1), mouse_over(false)
+			indicator_text("*"), is_active(1), mouse_over(false), indicator_width(0)
 	{
 		if (the_strings)
 		{
@@ -204,11 +218,11 @@ namespace Indicators
 		if (mouse_over)
 			glColor3f(1.0f,1.0f,1.0f);
 		else if (unavailable_func && unavailable_func())
-			glColor3f(0.20f,0.15f,0.10f);
+			glColor3f(gui_dull_color[0]/2, gui_dull_color[1]/2, gui_dull_color[2]/2);
 		else if (cntr_func && cntr_func())
-			glColor3f(0.99f,0.87f,0.65f);
+			glColor3fv(gui_bright_color);
 		else
-			glColor3f(0.40f,0.30f,0.20f);
+			glColor3fv(gui_dull_color);
 		draw_string_zoomed(x_pos, Vars::border(), (const unsigned char*)indicator_text.c_str(), 1, Vars::zoom());
 		mouse_over = false;
 	}
@@ -221,6 +235,14 @@ namespace Indicators
 			tooltip = ((cntr_func && cntr_func()) ?on_tooltip : off_tooltip);
 	}
 
+	//	Respond to scale or font changes
+	//
+	void Basic_Indicator::ui_scale(void)
+	{
+		indicator_width = FontManager::get_instance().line_width(Vars::font_cat,
+			reinterpret_cast<const unsigned char*>(indicator_text.c_str()), indicator_text.size(), Vars::zoom());
+	}
+
 	//	If an action string is defined, execute using the standard command line parser.
 	//
 	void Parse_Action_Indicator::do_action(void) const
@@ -288,17 +310,22 @@ namespace Indicators
 	//
 	void Indicators_Container::init(void)
 	{
+		if (!FontManager::get_instance().is_initialized())
+			return;
+
 		std::pair<int,int> loc = get_default_location();
 
 		if (indicators.empty())
 		{
-			indicators.reserve(6);
+			indicators.reserve(7);
 			indicators.push_back(new Parse_Action_Indicator(day_indicator_str, today_is_special_day, 0, indicators.size(), "#day"));
 			indicators.push_back(new Basic_Indicator(harvest_indicator_str, now_harvesting, 0, indicators.size()));
 			indicators.push_back(new Basic_Indicator(poison_indicator_str, we_are_poisoned, 0, indicators.size()));
 			indicators.push_back(new Value_Indicator(messages_indicator_str, get_seen_pm_count, 0, indicators.size(), clear_seen_pm_count));
 			indicators.push_back(new Parse_Action_Indicator(ranginglock_indicator_str, ranging_lock_is_on, 0, indicators.size(), "#keypress #K_RANGINGLOCK"));
 			indicators.push_back(new Parse_Action_Indicator(glowperk_indicator_str, glow_perk_is_active, glow_perk_is_unavailable, indicators.size(), "#glow"));
+			std::string summon_attach_command = "#" +  std::string(cmd_summon_attack);
+			indicators.push_back(new Parse_Action_Indicator(summon_attack_indicator_str, summon_attack_is_active, summon_attack_is_unknown, indicators.size(), summon_attach_command.c_str()));
 		}
 
 		x_len = static_cast<int>(Vars::font_x() * indicators.size() * Vars::zoom() +
@@ -345,6 +372,7 @@ namespace Indicators
 			set_window_handler(indicators_win, ELW_HANDLER_MOUSEOVER, (int (*)())&mouseover_indicators_handler);
 			set_window_handler(indicators_win, ELW_HANDLER_CLICK, (int (*)())&click_indicators_handler);
 			set_window_handler(indicators_win, ELW_HANDLER_UI_SCALE, (int (*)())&ui_scale_indicators_handler);
+			set_window_handler(indicators_win, ELW_HANDLER_FONT_CHANGE, (int (*)())&ui_scale_indicators_handler);
 			ui_scale_indicators_handler(&windows_list.window[indicators_win]);
 
 			background_on = ((option_settings >> 25) & 1);
@@ -363,9 +391,7 @@ namespace Indicators
 			cm_menu << cm_indicators_str;
 			for (i=indicators.begin(); i<indicators.end(); ++i)
 				cm_menu << (*i)->get_context_menu_str() << std::endl;
-			cm_menu_id = cm_create(cm_title_menu_str, NULL);
-			cm_bool_line(cm_menu_id, 1, &windows_list.window[indicators_win].opaque, NULL);
-			cm_bool_line(cm_menu_id, 2, &windows_on_top, "windows_on_top");
+			cm_menu_id = cm_title_create_menu(&windows_list.window[indicators_win], cm_title_menu_str, NULL);
 			cm_add(cm_menu_id, cm_menu.str().c_str(), cm_indicators_handler);
 			cm_bool_line(cm_menu_id, CMHI_RELOC, &cm_relocatable, 0);
 			cm_bool_line(cm_menu_id, CMHI_BACKGROUND, &background_on, 0);
@@ -426,9 +452,9 @@ namespace Indicators
 		}
 		if (!have_active)
 		{
-			glColor3f(0.40f,0.30f,0.20f);
+			glColor3fv(gui_dull_color);
 			draw_string_zoomed(pos_x, Vars::border(), (const unsigned char *)no_indicators_str, 1, Vars::zoom());
-			pos_x += static_cast<int>(strlen(no_indicators_str) * Vars::zoom() * Vars::font_x() + 0.5);
+			pos_x += no_indicators_str_width;
 		}
 		change_width(pos_x + Vars::border());
 	}
@@ -481,9 +507,12 @@ namespace Indicators
 		std::vector<Basic_Indicator *>::iterator i = get_over(mx);
 		if (win && (i < indicators.end()))
 		{
+			eternal_lands::FontManager &fmgr = eternal_lands::FontManager::get_instance();
 			std::string tooltip("");
 			(*i)->get_tooltip(tooltip);
-			int x_offset = -static_cast<int>(Vars::border() + win->small_font_len_x * (1 + tooltip.size()) + 0.5);
+			int width = fmgr.line_width(win->font_category, (const unsigned char*)tooltip.c_str(),
+				tooltip.length(), win->current_scale_small);
+			int x_offset = -(Vars::border() + width);
 			if ((win->cur_x + x_offset) < 0)
 				x_offset = win->len_x;
 			show_help(tooltip.c_str(), x_offset, Vars::border(), win->current_scale);
@@ -569,6 +598,19 @@ namespace Indicators
 	}
 
 
+	// Respond to scale or font changes
+	//
+	void Indicators_Container::ui_scale_handler(window_info *win)
+	{
+		x_len = y_len = 0;
+		Vars::set_scale(win->current_scale);
+		no_indicators_str_width = FontManager::get_instance().line_width(Vars::font_cat,
+			reinterpret_cast<const unsigned char*>(no_indicators_str), strlen(no_indicators_str), Vars::zoom());
+		for (auto iter: indicators)
+			iter->ui_scale();
+	}
+
+
 	//	Get the width of the indicators window if enabled and in the default location, otherwise 0.
 	//
 	int Indicators_Container::get_default_width(void)
@@ -592,30 +634,82 @@ namespace Indicators
 
 	//	Called when saving client settings
 	//
-	void Indicators_Container::get_settings(unsigned int *opts, unsigned int *pos)
+	void Indicators_Container::get_settings(unsigned int *opts, unsigned int *pos) const
 	{
 		unsigned int flags = 0;
 		unsigned int shift = 0;
 		unsigned int x = 0;
 		unsigned int y = 0;
 
-		std::vector<Basic_Indicator *>::iterator i;
+		if (indicators_win < 0)
+		{
+			*opts = option_settings;
+			*pos = position_settings;
+			return;
+		}
+
+		std::vector<Basic_Indicator *>::const_iterator i;
 		for (i=indicators.begin(); i<indicators.end(); ++i, shift++)
 			flags |= (((*i)->not_active()) ?1 :0) << shift;
 
-		if (!default_location && (indicators_win >= 0))
-		{
+		if (!default_location)
 			flags |= 1 << 24;
-			x = static_cast<unsigned int>(windows_list.window[indicators_win].cur_x);
-			y = static_cast<unsigned int>(windows_list.window[indicators_win].cur_y);
-		}
 		flags |= background_on << 25;
 		flags |= border_on << 26;
-
 		*opts = flags;
+
+		x = static_cast<unsigned int>(windows_list.window[indicators_win].cur_x);
+		y = static_cast<unsigned int>(windows_list.window[indicators_win].cur_y);
 		*pos = x | (y<<16);
 	}
 
+
+#ifdef JSON_FILES
+	//	Read the options from the client state file
+	//
+	void Indicators_Container::read_settings(const char *dict_name)
+	{
+		int pos_x = json_cstate_get_int(dict_name, "pos_x", 0);
+		int pos_y = json_cstate_get_int(dict_name, "pos_y", 0);
+		position_settings = (pos_x & 0xFFFF) | ((pos_y & 0xFFFF) << 16);
+
+		option_settings = json_cstate_get_unsigned_int(dict_name, "disabled_flags", 0);
+		option_settings |= json_cstate_get_bool(dict_name, "relocated", 0) << 24;
+		option_settings |= json_cstate_get_bool(dict_name, "background_on", 0) << 25;
+		option_settings |= json_cstate_get_bool(dict_name, "border_on", 0) << 26;
+		have_settings = true;
+	}
+
+
+	//	Write the options from the client state file
+	//
+	void Indicators_Container::write_settings(const char *dict_name) const
+	{
+		if (indicators_win < 0)
+		{
+			json_cstate_set_int(dict_name, "pos_x", position_settings & 0xFFFF);
+			json_cstate_set_int(dict_name, "pos_y", (position_settings >> 16 ) & 0xFFFF);
+			json_cstate_set_unsigned_int(dict_name, "disabled_flags", option_settings & 0x00FFFFFF);
+			json_cstate_set_bool(dict_name, "relocated", (option_settings >> 24) & 1);
+			json_cstate_set_bool(dict_name, "background_on", (option_settings >> 25) & 1);
+			json_cstate_set_bool(dict_name, "border_on", (option_settings >> 26) & 1);
+			return;
+		}
+
+		unsigned int disabled_flags = 0, shift = 0;
+		std::vector<Basic_Indicator *>::const_iterator  i;
+		for (i=indicators.begin(); i<indicators.end(); ++i, shift++)
+			disabled_flags |= (((*i)->not_active()) ?1 :0) << shift;
+		json_cstate_set_unsigned_int(dict_name, "disabled_flags", disabled_flags);
+
+		json_cstate_set_unsigned_int(dict_name, "pos_x", static_cast<unsigned int>(windows_list.window[indicators_win].cur_x));
+		json_cstate_set_unsigned_int(dict_name, "pos_y", static_cast<unsigned int>(windows_list.window[indicators_win].cur_y));
+		json_cstate_set_bool(dict_name, "relocated", (default_location) ?0: 1);
+		json_cstate_set_bool(dict_name, "background_on", background_on);
+		json_cstate_set_bool(dict_name, "border_on", border_on);
+	}
+#endif
+
 } // end namespace
 
 
@@ -629,7 +723,11 @@ extern "C"
 	void show_hud_indicators_window(void) { if (show_hud_indicators) Indicators::container.show(); }
 	void hide_hud_indicators_window(void) { Indicators::container.hide(); }
 	void toggle_hud_indicators_window(int *show) { *show = !*show; Indicators::container.toggle(*show); }
-	void set_settings_hud_indicators(unsigned int opts, unsigned int pos) { return Indicators::container.set_settings(opts, pos); }
+	void set_settings_hud_indicators(unsigned int opts, unsigned int pos) { Indicators::container.set_settings(opts, pos); }
 	void get_settings_hud_indicators(unsigned int *opts, unsigned int *pos) { Indicators::container.get_settings(opts, pos); }
+#ifdef JSON_FILES
+	void read_settings_hud_indicators(const char *dict_name) { Indicators::container.read_settings(dict_name); }
+	void write_settings_hud_indicators(const char *dict_name) { Indicators::container.write_settings(dict_name); }
+#endif
 	int get_hud_indicators_default_width(void) { if (show_hud_indicators) return Indicators::container.get_default_width(); else return 0; }
 }
