@@ -53,11 +53,18 @@ static int mouse_over_clock = 0;					/* 1 if mouse is over digital or analogue c
 static int mouse_over_compass = 0;					/* 1 if mouse is over the compass */
 static int mouse_over_knowledge_bar = 0;			/* 1 if mouse is over the knowledge bar */
 static int knowledge_bar_height = 0;
+static float side_stats_bar_text_zoom = 0.0f;
+static int side_stats_bar_text_width = 0;
+static int side_stats_bar_text_height = 0;
+static int side_stats_bar_text_offset = 0;
 static int side_stats_bar_height = 0;
+static int quickbar_misc_sep = 5;
+static float digital_clock_zoom = 0.0f;
 static int digital_clock_height = 0;
 static int analog_clock_size = 0;
 static int compass_size = 0;
 static const int min_side_stats_bar = 5;
+static int hud_margin = 0; /* scaled to avoid the edge line in the hud texture */
 
 
 enum {	CMH_STATS=0, CMH_STATBARS, CMH_KNOWBAR, CMH_TIMER, CMH_DIGCLOCK, CMH_ANACLOCK,
@@ -65,10 +72,10 @@ enum {	CMH_STATS=0, CMH_STATBARS, CMH_KNOWBAR, CMH_TIMER, CMH_DIGCLOCK, CMH_ANAC
 		CMH_SEP2, CMH_SOUND, CMH_MUSIC, CMH_SEP3, CMH_LOCATION };
 
 
-static void draw_side_stats_bar(window_info *win, const int x, const int y, const int baselev, const int cur_exp, const int nl_exp, size_t colour)
+static void draw_side_stats_bar(window_info *win, const int x, const int y, const Sint16 baselev, const Uint32 cur_exp, const Uint32 nl_exp, size_t colour)
 {
 	const int max_len = win->len_x - x - 1;
-	const int bar_height = win->small_font_len_y - (int)(0.5 + win->current_scale * 2);
+	const int bar_height = side_stats_bar_text_height + 2;
 	int len = max_len - (float)max_len/(float)((float)(nl_exp-exp_lev[baselev])/(float)(nl_exp-cur_exp));
 
 	GLfloat colours[2][2][3] = { { {0.11f, 0.11f, 0.11f}, {0.3f, 0.5f, 0.2f} },
@@ -94,9 +101,9 @@ CHECK_GL_ERRORS();
 		glVertex3i(x+len, y+bar_height, 0);
 		glEnd();
 	}
-	
+
 	// draw the bar frame
-	glColor3f(0.77f, 0.57f, 0.39f);
+	glColor3fv(gui_color);
 	glBegin(GL_LINE_LOOP);
 	glVertex3i(x, y, 0);
 	glVertex3i(x+max_len, y, 0);
@@ -115,8 +122,8 @@ static int context_hud_handler(window_info *win, int widget_id, int mx, int my,
 	unsigned char protocol_name;
 	switch (option)
 	{
-		case CMH_MINIMAP: view_window(&minimap_win, 0); break;
-		case CMH_RANGSTATS: view_window(&range_win, 0); break;
+		case CMH_MINIMAP: view_window(MW_MINIMAP); break;
+		case CMH_RANGSTATS: view_window(MW_RANGING); break;
 #ifdef NEW_SOUND
 		case CMH_SOUND: toggle_sounds(&sound_on); set_var_unsaved("enable_sound", INI_FILE_VAR); break;
 		case CMH_MUSIC: toggle_music(&music_on); set_var_unsaved("enable_music", INI_FILE_VAR); break;
@@ -124,7 +131,7 @@ static int context_hud_handler(window_info *win, int widget_id, int mx, int my,
 		case CMH_LOCATION:
 			copy_next_LOCATE_ME = 1;
 			protocol_name= LOCATE_ME;
-			my_tcp_send(my_socket,&protocol_name,1);
+			my_tcp_send(&protocol_name,1);
 			break;
 		default:
 			init_misc_display();
@@ -139,18 +146,18 @@ static void context_hud_pre_show_handler(window_info *win, int widget_id, int mx
 	cm_sound_enabled = sound_on;
 	cm_music_enabled = music_on;
 #endif // NEW_SOUND
-	cm_minimap_shown = get_show_window(minimap_win);
-	cm_rangstats_shown = get_show_window(range_win);
+	cm_minimap_shown = get_show_window_MW(MW_MINIMAP);
+	cm_rangstats_shown = get_show_window_MW(MW_RANGING);
 }
 
 
 /*	Calculate num_disp_stat and first_disp_stat
 */
-static void calc_statbar_shown(int base_y_pos)
+static void calc_statbar_shown(int base_y_pos, float scale)
 {
 	int last_display = num_disp_stat;
 	int quickspell_base = get_quickspell_y_base();
-	int quickbar_base = get_quickbar_y_base();
+	int quickbar_base = get_quickbar_y_base() + (int)(0.5 + quickbar_misc_sep);
 	int highest_win_pos_y = base_y_pos - side_stats_bar_height*(NUM_WATCH_STAT-1);
 
 	/* calculate the overlap between the longest of the quickspell/bar and the statsbar */
@@ -179,16 +186,17 @@ static void calc_statbar_shown(int base_y_pos)
 static void reset_cm_regions(void)
 {
 	cm_remove_regions(game_root_win);
-	cm_remove_regions(map_root_win);
-	cm_remove_regions(console_root_win);
+	cm_remove_regions(get_id_MW(MW_TABMAP));
+	cm_remove_regions(get_id_MW(MW_CONSOLE));
 	cm_add_region(cm_hud_id, game_root_win, window_width-hud_x, 0, hud_x, window_height);
-	cm_add_region(cm_hud_id, map_root_win, window_width-hud_x, 0, hud_x, window_height);
-	cm_add_region(cm_hud_id, console_root_win, window_width-hud_x, 0, hud_x, window_height);
+	cm_add_region(cm_hud_id, get_id_MW(MW_TABMAP), window_width-hud_x, 0, hud_x, window_height);
+	cm_add_region(cm_hud_id, get_id_MW(MW_CONSOLE), window_width-hud_x, 0, hud_x, window_height);
 }
 
 
 static int display_misc_handler(window_info *win)
 {
+	const int tooltip_sep = (int)(0.5 + win->current_scale * 5);
 	const int scaled_5 = (int)(0.5 + win->current_scale * 5);
 	const int scaled_28 = (int)(0.5 + win->current_scale * 28);
 	int base_y_start = win->len_y;
@@ -211,7 +219,7 @@ CHECK_GL_ERRORS();
 	// allow for transparency
 	glEnable(GL_ALPHA_TEST);//enable alpha filtering, so we have some alpha key
 	glAlphaFunc(GL_GREATER, 0.09f);
-	
+
 	//draw the compass
 	base_y_start -= compass_size;
 	glBegin(GL_QUADS);
@@ -265,14 +273,34 @@ CHECK_GL_ERRORS();
 	//Digital Clock
 	if(view_digital_clock > 0)
 	{
-		const int scaled_6 = (int)(0.5 + win->current_scale * 6);
+		int centre_x = (0.5 + win->len_x / 2.0);
 		char str[10];
+
+		base_y_start -= digital_clock_height;
+		// When seconds are shown, we keep the single minutes character fixed in the window,
+		// and draw the rest of the string around it. If we would simply center the whole time
+		// string, the entire string will move horizontally every second, which is visually
+		// distracting. Now the horizontal alignment only changes once per minute.
+		// If seconds are not drawn, we simply center the time.
 		if (show_game_seconds)
+		{
+			int x;
 			safe_snprintf(str, sizeof(str), "%1d:%02d:%02d", real_game_minute/60, real_game_minute%60, real_game_second);
+			x = centre_x
+				- get_buf_width_zoom((const unsigned char*)str, 4, win->font_category, digital_clock_zoom)
+				+ get_char_width_zoom(str[3], win->font_category, digital_clock_zoom) / 2;
+			draw_text(max2i(x, hud_margin), base_y_start, (const unsigned char*)str, strlen(str),
+				win->font_category, TDO_SHADOW, 1, TDO_FOREGROUND, gui_color[0], gui_color[1], gui_color[2],
+				TDO_BACKGROUND, 0.0, 0.0, 0.0, TDO_ZOOM, digital_clock_zoom, TDO_END);
+		}
 		else
-			safe_snprintf(str, sizeof(str), " %1d:%02d ", real_game_minute/60, real_game_minute%60);
- 		base_y_start -= digital_clock_height;
- 		draw_string_shadowed_width(scaled_6/2, scaled_6/2 + base_y_start, (unsigned char*)str, win->len_x-scaled_6, 1,0.77f, 0.57f, 0.39f,0.0f,0.0f,0.0f);
+		{
+			safe_snprintf(str, sizeof(str), "%1d:%02d", real_game_minute/60, real_game_minute%60);
+			draw_text(centre_x, base_y_start, (const unsigned char*)str, strlen(str),
+				win->font_category, TDO_SHADOW, 1, TDO_FOREGROUND, gui_color[0], gui_color[1], gui_color[2],
+				TDO_BACKGROUND, 0.0, 0.0, 0.0, TDO_ALIGNMENT, CENTER, TDO_ZOOM, digital_clock_zoom,
+				TDO_END);
+		}
 	}
 
 	/* if mouse over the either of the clocks - display the time & date */
@@ -283,11 +311,14 @@ CHECK_GL_ERRORS();
 		int centre_y =  (view_analog_clock) ?win->len_y - compass_size - analog_clock_size/2 : base_y_start + digital_clock_height/2;
 
 		safe_snprintf(str, sizeof(str), "%1d:%02d:%02d", real_game_minute/60, real_game_minute%60, real_game_second);
-		draw_string_small_shadowed_zoomed(-win->small_font_len_x*(strlen(str)+0.5), centre_y-win->small_font_len_y, (unsigned char*)str, 1, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, win->current_scale);
+		draw_string_small_shadowed_zoomed_right(-tooltip_sep, centre_y-win->small_font_len_y,
+			(const unsigned char*)str, 1, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f,
+			win->current_scale);
 		if (the_date != NULL)
 		{
 			safe_snprintf(str, sizeof(str), "%s", the_date);
-			draw_string_small_shadowed_zoomed(-win->small_font_len_x*(strlen(str)+0.5), centre_y, (unsigned char*)str, 1, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, win->current_scale);
+			draw_string_small_shadowed_zoomed_right(-tooltip_sep, centre_y, (const unsigned char*)str,
+				1, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, win->current_scale);
 		}
 		mouse_over_clock = 0;
 	}
@@ -300,7 +331,9 @@ CHECK_GL_ERRORS();
 		if (me != NULL)
 		{
 			safe_snprintf(str, sizeof(str), "%d,%d", me->x_tile_pos, me->y_tile_pos);
-			draw_string_small_shadowed_zoomed(-win->small_font_len_x*(strlen(str)+0.5), win->len_y-compass_size, (unsigned char*)str, 1, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, win->current_scale);
+			draw_string_small_shadowed_zoomed_right(-tooltip_sep, win->len_y-compass_size,
+				(unsigned char*)str, 1, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f,
+				win->current_scale);
 		}
 		mouse_over_compass = 0;
 	}
@@ -311,24 +344,26 @@ CHECK_GL_ERRORS();
 		char str[20];
 		char *use_str = idle_str;
 		int percentage_done = 0;
-		int x = (int)(0.5 + win->current_scale * 3);
 		int y = base_y_start - side_stats_bar_height - ((knowledge_bar_height - side_stats_bar_height) / 2);
-		int off = 0;
-		
+
 		if (is_researching())
 		{
 			percentage_done = (int)(100 * get_research_fraction());
 			safe_snprintf(str, sizeof(str), "%d%%", percentage_done);
 			use_str = str;
 		}
-		off = ((win->len_x - x - 1) - (win->small_font_len_x * strlen(use_str))) / 2;
-		draw_side_stats_bar(win, x, y, 0, percentage_done, 100, 1);
-		draw_string_small_shadowed_zoomed(x+off+gx_adjust, y+gy_adjust, (unsigned char *)use_str, 1,1.0f,1.0f,1.0f,0.0f,0.0f,0.0f, win->current_scale);
+		draw_side_stats_bar(win, hud_margin, y, 0, percentage_done, 100, 1);
+		draw_string_shadowed_zoomed_centered(hud_margin + (win->len_x - hud_margin - 1) / 2, y,
+			(unsigned char *)use_str, 1,1.0f,1.0f,1.0f,0.0f,0.0f,0.0f,
+			side_stats_bar_text_zoom);
 
 		if (mouse_over_knowledge_bar)
 		{
 			use_str = (is_researching()) ?get_research_eta_str(str, sizeof(str)) : not_researching_str;
-			draw_string_small_shadowed_zoomed(-win->small_font_len_x* (strlen(use_str)+0.5), y+gy_adjust, (unsigned char*)use_str, 1,1.0f,1.0f,1.0f,0.0f,0.0f,0.0f, win->current_scale);
+			draw_text(-tooltip_sep, y + side_stats_bar_height / 2, (const unsigned char*)use_str,
+				strlen(use_str), win->font_category, TDO_SHADOW, 1, TDO_FOREGROUND, 1.0, 1.0, 1.0,
+				TDO_BACKGROUND, 0.0, 0.0, 0.0, TDO_ZOOM, win->current_scale_small, TDO_ALIGNMENT, RIGHT,
+				TDO_VERTICAL_ALIGNMENT, CENTER_LINE, TDO_END);
 			mouse_over_knowledge_bar = 0;
 		}
 
@@ -336,7 +371,7 @@ CHECK_GL_ERRORS();
 	}
 
 	/* if the timer is visible, draw it */
-	base_y_start -= display_timer(win, base_y_start);
+	base_y_start -= display_timer(win, base_y_start, tooltip_sep);
 
 	/*	Optionally display the stats bar.  If the current window size does not
 		provide enough room, display only some skills and allow scrolling to view
@@ -344,60 +379,81 @@ CHECK_GL_ERRORS();
 	if(show_stats_in_hud && have_stats)
 	{
 		char str[20];
-		int box_x = (int)(0.5 + win->current_scale * 3);
-		int text_x = box_x + ((win->len_x - box_x - 1) - (7 * win->small_font_len_x))/2;
+		int text_x_center = hud_margin + (win->len_x - hud_margin - 1) / 2;
+		int text_x_left, text_x_right;
 		int thestat;
 		int y = 0;
 		int skill_modifier;
 
 		// trade the number of quickbar slots if there is not enough space for the minimum stats
-		calc_statbar_shown(base_y_start + win->pos_y);
+		calc_statbar_shown(base_y_start + win->pos_y, win->current_scale);
+
+		text_x_left = text_x_center - side_stats_bar_text_width / 2;
+		text_x_right = text_x_left + side_stats_bar_text_width;
 
 #ifdef OPENGL_TRACE
 CHECK_GL_ERRORS();
 #endif //OPENGL_TRACE
-		
+
 		for (thestat=0; thestat<NUM_WATCH_STAT-1; thestat++)
 		{
+			static const GLfloat white_color[3] = { 1.0f, 1.0f, 1.0f };
+
 			int hover_offset = 0;
-			
+			const float *col;
+
 			/* skill skills until we have the skill displayed first */
 			if (thestat < first_disp_stat)
 				continue;
-				
+
 			/* end now if we have display all we can */
 			if (thestat > (first_disp_stat+num_disp_stat-1))
 				break;
-		
+
 			if (show_statbars_in_hud)
-				draw_side_stats_bar(win, box_x, y+1, statsinfo[thestat].skillattr->base,
+				draw_side_stats_bar(win, hud_margin, y, statsinfo[thestat].skillattr->base,
 					*statsinfo[thestat].exp, *statsinfo[thestat].next_lev, 0);
-		
-			safe_snprintf(str,sizeof(str),"%-3s %3i",
+
+			col = (statsinfo[thestat].is_selected == 1) ? gui_color : white_color;
+			draw_text(text_x_left, y + 1 + side_stats_bar_text_offset,
 				statsinfo[thestat].skillnames->shortname,
-				statsinfo[thestat].skillattr->base );
-			if (statsinfo[thestat].is_selected == 1)
-				draw_string_small_shadowed_zoomed(text_x+gx_adjust, y+gy_adjust, (unsigned char*)str, 1,0.77f, 0.57f, 0.39f,0.0f,0.0f,0.0f, win->current_scale);
-			else
-				draw_string_small_shadowed_zoomed(text_x+gx_adjust, y+gy_adjust, (unsigned char*)str, 1,1.0f,1.0f,1.0f,0.0f,0.0f,0.0f, win->current_scale);
-			
+				strlen((const char*)statsinfo[thestat].skillnames->shortname), win->font_category,
+				TDO_SHADOW, 1, TDO_FOREGROUND, col[0], col[1], col[2], TDO_BACKGROUND, 0.0, 0.0, 0.0,
+				TDO_ZOOM, side_stats_bar_text_zoom, TDO_END);
+			safe_snprintf(str, sizeof(str), "%3d", statsinfo[thestat].skillattr->base);
+			draw_text(text_x_right, y + 1 + side_stats_bar_text_offset, (const unsigned char*)str,
+				strlen(str), win->font_category, TDO_SHADOW, 1, TDO_FOREGROUND, col[0], col[1], col[2],
+				TDO_BACKGROUND, 0.0, 0.0, 0.0, TDO_ZOOM, side_stats_bar_text_zoom, TDO_ALIGNMENT, RIGHT,
+				TDO_END);
+
 			if((thestat!=NUM_WATCH_STAT-2) && floatingmessages_enabled &&
 				(skill_modifier = statsinfo[thestat].skillattr->cur -
-				 	statsinfo[thestat].skillattr->base) != 0){
+				 	statsinfo[thestat].skillattr->base) != 0)
+			{
+				int ytop = y + (side_stats_bar_height - win->small_font_len_y) / 2;
 				safe_snprintf(str,sizeof(str),"%+i",skill_modifier);
-				hover_offset = strlen(str)+1;
+				hover_offset = get_string_width_zoom((const unsigned char*)str,
+					win->font_category, win->current_scale_small);
 				if(skill_modifier > 0){
-					draw_string_small_shadowed_zoomed(-win->small_font_len_x * (strlen(str)+0.5), y+gy_adjust, (unsigned char*)str, 1,0.3f, 1.0f, 0.3f,0.0f,0.0f,0.0f, win->current_scale);
+					draw_string_small_shadowed_zoomed_right(-tooltip_sep, ytop,
+						(const unsigned char*)str, 1,0.3f, 1.0f, 0.3f,0.0f,0.0f,0.0f,
+						win->current_scale);
 				} else {
-					draw_string_small_shadowed_zoomed(-win->small_font_len_x * (strlen(str)+0.5), y+gy_adjust, (unsigned char*)str, 1,1.0f, 0.1f, 0.2f,0.0f,0.0f,0.0f, win->current_scale);
+					draw_string_small_shadowed_zoomed_right(-tooltip_sep, ytop,
+						(const unsigned char*)str, 1,1.0f, 0.1f, 0.2f,0.0f,0.0f,0.0f,
+						win->current_scale);
 				}
 			}
 
 			/* if the mouse is over the stat bar, draw the XP remaining */
 			if (stat_mouse_is_over == thestat)
 			{
-				safe_snprintf(str,sizeof(str),"%li",(*statsinfo[thestat].next_lev - *statsinfo[thestat].exp));
-				draw_string_small_shadowed_zoomed(-win->small_font_len_x*(strlen(str)+0.5+hover_offset), y+gy_adjust, (unsigned char*)str, 1,1.0f,1.0f,1.0f,0.0f,0.0f,0.0f, win->current_scale);
+				safe_snprintf(str, sizeof(str), "%i", (*statsinfo[thestat].next_lev - *statsinfo[thestat].exp));
+				draw_text(-hover_offset-tooltip_sep, y + side_stats_bar_height / 2,
+					(const unsigned char*)str, strlen(str), win->font_category, TDO_SHADOW, 1,
+					TDO_FOREGROUND, 1.0, 1.0, 1.0, TDO_BACKGROUND, 0.0, 0.0, 0.0,
+					TDO_ZOOM, win->current_scale_small, TDO_ALIGNMENT, RIGHT,
+					TDO_VERTICAL_ALIGNMENT, CENTER_LINE, TDO_END);
 				stat_mouse_is_over = -1;
 			}
 
@@ -423,7 +479,7 @@ CHECK_GL_ERRORS();
 			last_hud_x = hud_x;
 		}
 	}
-	
+
 #ifdef OPENGL_TRACE
 CHECK_GL_ERRORS();
 #endif //OPENGL_TRACE
@@ -490,7 +546,7 @@ static int click_misc_handler(window_info *win, int mx, int my, Uint32 flags)
 		unsigned char protocol_name;
 		do_click_sound();
 		protocol_name= GET_TIME;
-		my_tcp_send(my_socket,&protocol_name,1);
+		my_tcp_send(&protocol_name,1);
 		return 1;
 	}
 
@@ -512,7 +568,7 @@ static int click_misc_handler(window_info *win, int mx, int my, Uint32 flags)
 		{
 			copy_next_LOCATE_ME = 2;
 		}
-		my_tcp_send(my_socket,&protocol_name,1);
+		my_tcp_send(&protocol_name,1);
 		return 1;
 	}
 	//check to see if we clicked on the stats
@@ -531,8 +587,12 @@ static int mouseover_misc_handler(window_info *win, int mx, int my)
 	/* Optionally display scrolling help if statsbar is active and restricted in size */
 	if (show_help_text && show_stats_in_hud && (num_disp_stat < NUM_WATCH_STAT-1) &&
 		(my >= 0) && (my < num_disp_stat*side_stats_bar_height))
-		show_help(stats_scroll_help_str, -10-strlen(stats_scroll_help_str)*win->small_font_len_x,
+	{
+		int width = get_string_width_zoom((const unsigned char*)stats_scroll_help_str,
+			win->font_category, win->current_scale_small);
+		show_help(stats_scroll_help_str, - 10 - width,
 			win->len_y - HUD_MARGIN_Y - win->small_font_len_y, win->current_scale);
+	}
 
 	/* stat hover experience left */
 	if (show_stats_in_hud && have_stats && (my >= 0) && (my < num_disp_stat*side_stats_bar_height))
@@ -578,11 +638,68 @@ static int destroy_misc_handler(window_info *win)
 static int ui_scale_misc_handler(window_info *win)
 {
 	int y_len = 0;
+	int thestat, max_width, width, text_width, text_top, text_bottom, top, bottom;
+	unsigned char str[5];
+	int digital_clock_width;
+
+	// Set width, so we can use it in e.g. timer ui_scale handler
+	resize_window(win->window_id, HUD_MARGIN_X, win->len_y);
+
 	analog_clock_size = (int)(0.5 + win->current_scale * 64);
 	compass_size = (int)(0.5 + win->current_scale * 64);
 	knowledge_bar_height = win->small_font_len_y + 6;
-	side_stats_bar_height = win->small_font_len_y;
-	digital_clock_height = win->default_font_len_y;
+	hud_margin = (int)(0.5 + 3 * win->current_scale);
+
+	side_stats_bar_text_zoom = win->current_scale_small;
+	text_width = get_string_width_zoom((const unsigned char*)idle_str, win->font_category,
+		side_stats_bar_text_zoom);
+	width = 3 * get_max_digit_width_zoom(win->font_category, side_stats_bar_text_zoom)
+		+ get_char_width_zoom('%', win->font_category, side_stats_bar_text_zoom);
+	text_width = max2i(text_width, width);
+	for (thestat = 0; thestat < NUM_WATCH_STAT-1; ++thestat)
+	{
+		safe_snprintf((char*)str, sizeof(str), "%-3s ", statsinfo[thestat].skillnames->shortname);
+		width = get_string_width_zoom(str, win->font_category, side_stats_bar_text_zoom)
+			+ 3 * get_max_digit_width_zoom(win->font_category, side_stats_bar_text_zoom);
+		text_width = max2i(text_width, width);
+	}
+	max_width = win->len_x - hud_margin - 1;
+	if (text_width > max_width)
+	{
+		side_stats_bar_text_zoom *= (float)max_width / text_width;
+		text_width = win->len_x - hud_margin - 1;
+	}
+
+	get_top_bottom((const unsigned char*)"0123456789%", 11, win->font_category, side_stats_bar_text_zoom,
+		&text_top, &text_bottom);
+	for (thestat = 0; thestat < NUM_WATCH_STAT-1; ++thestat)
+	{
+		get_top_bottom(statsinfo[thestat].skillnames->shortname,
+			strlen((const char*)statsinfo[thestat].skillnames->shortname), win->font_category,
+			side_stats_bar_text_zoom, &top, &bottom);
+		text_top = min2i(text_top, top);
+		text_bottom = max2i(text_bottom, bottom);
+	}
+	get_top_bottom((const unsigned char*)idle_str, strlen(idle_str), win->font_category,
+		side_stats_bar_text_zoom, &top, &bottom);
+	text_top = min2i(text_top, top);
+	text_bottom = max2i(text_bottom, bottom);
+
+	side_stats_bar_text_width = text_width;
+	side_stats_bar_text_offset = -text_top + (int)(0.5 + win->current_scale);
+	side_stats_bar_text_height = text_bottom - text_top // text
+		+ 2 * (int)(0.5 + win->current_scale); // shadow, top and bottom
+	side_stats_bar_height = side_stats_bar_text_height
+		+ 2 // border around bar
+		+ (int)(win->current_scale * 2); // separation between bars
+
+	// always scale the digital clock as when seconds enabled
+	// allow 7px for max rouding error, margin to avoid the edges
+	digital_clock_width = 5 * get_max_digit_width_zoom(win->font_category, 1.0)
+		+ 2 * get_char_width_zoom(':', win->font_category, 1.0) + 7;
+	digital_clock_zoom = ((float)win->len_x - 2 * hud_margin) / (float)digital_clock_width;
+	digital_clock_height = get_line_height(win->font_category, digital_clock_zoom);
+
 	ui_scale_timer(win);
 	y_len = compass_size;
 	if (view_hud_timer)
@@ -596,8 +713,11 @@ static int ui_scale_misc_handler(window_info *win)
 	if (show_stats_in_hud && have_stats)
 		y_len += num_disp_stat * side_stats_bar_height;
 	resize_window(win->window_id, HUD_MARGIN_X, y_len);
-	move_window(win->window_id, -1, 0, window_width-HUD_MARGIN_X, window_height-y_len);
+	move_window(win->window_id, -1, 0, window_width-win->len_x, window_height-win->len_y);
 	reset_cm_regions();
+
+	quickbar_misc_sep = win->current_scale * 5;
+
 	return 1;
 }
 
@@ -612,6 +732,7 @@ int get_min_hud_misc_len_y(void)
 		min_len_y -= num_disp_stat * side_stats_bar_height;
 		min_len_y += min_side_stats_bar * side_stats_bar_height;
 	}
+	min_len_y += quickbar_misc_sep;
 	return min_len_y;
 }
 
@@ -628,6 +749,7 @@ void init_misc_display(void)
 			set_window_handler(misc_win, ELW_HANDLER_CLICK, &click_misc_handler);
 			set_window_handler(misc_win, ELW_HANDLER_MOUSEOVER, &mouseover_misc_handler );
 			set_window_handler(misc_win, ELW_HANDLER_UI_SCALE, &ui_scale_misc_handler );
+			set_window_handler(misc_win, ELW_HANDLER_FONT_CHANGE, &ui_scale_misc_handler);
 			set_window_handler(misc_win, ELW_HANDLER_DESTROY, &destroy_misc_handler );
 			cm_hud_id = cm_create(cm_hud_menu_str, context_hud_handler);
 			cm_bool_line(cm_hud_id, CMH_STATS, &show_stats_in_hud, "show_stats_in_hud");
