@@ -8,11 +8,12 @@
 #include "elconfig.h"
 #include "elwindows.h"
 #include "errors.h"
+#include "gamewin.h"
 #include "gl_init.h"
 #include "hud.h"
 #include "hud_quickspells_window.h"
 #include "hud_misc_window.h"
-#if defined JSON_FILES
+#ifdef JSON_FILES
 #include "json_io.h"
 #endif
 #include "loginwin.h"
@@ -20,15 +21,12 @@
 #include "spells.h"
 #include "sound.h"
 
-int quickspell_win = -1;
 int num_quickspell_slots = 6;
 int quickspells_relocatable = 0;
 mqbdata * mqb_data[MAX_QUICKSPELL_SLOTS+1]={NULL};//mqb_data will hold the magic quickspells name, image, pos.
 
 static int default_quickspells_x = -1;
 static int default_quickspells_y = -1;
-static int saved_quickspells_x = 0;
-static int saved_quickspells_y = 0;
 static int quickspell_y_space = -1;
 static int quickspells_loaded = 0;
 static int quickspell_size = -1;
@@ -71,6 +69,7 @@ static Uint32 get_flags(int win_id)
 // returns true if the window is not in the default place, false if it is, even if it can be relocated
 static int is_relocated(void)
 {
+	int quickspell_win = get_id_MW(MW_QUICKSPELLS);
 	window_info *win = NULL;
 	if (quickspell_win < 0 || quickspell_win >= windows_list.num_windows)
 		return 1;
@@ -86,6 +85,7 @@ static int is_relocated(void)
 // enable/disable window title bar and dragability
 static void toggle_quickspells_moveable(void)
 {
+	int quickspell_win = get_id_MW(MW_QUICKSPELLS);
 	Uint32 flags = get_flags(quickspell_win);
 	if (!quickspells_moveable)
 	{
@@ -94,7 +94,7 @@ static void toggle_quickspells_moveable(void)
 		change_flags (quickspell_win, flags);
 		quickspells_moveable = 1;
 	}
-	else 
+	else
 	{
 		flags |= ELW_SHOW_LAST;
 		flags &= ~(ELW_DRAGGABLE | ELW_TITLE_BAR);
@@ -105,8 +105,10 @@ static void toggle_quickspells_moveable(void)
 
 
 // return the window to it's default position
-static void reset_quickspells() 
+static void reset_quickspells()
 {
+	int quickspell_win = get_id_MW(MW_QUICKSPELLS);
+	limit_win_scale_to_default(get_scale_WM(MW_QUICKSPELLS));
 	quickspells_dir = VERTICAL;
 	quickspells_moveable = 0;
 	if (quickspells_relocatable)
@@ -251,8 +253,11 @@ CHECK_GL_ERRORS();
 
 	if(quickspell_over!=-1 && mqb_data[quickspell_over])
 	{
+		float zoom = win->current_scale_small;
 		int x = 0, y = 0;
-		int len_str = (strlen(mqb_data[quickspell_over]->spell_name) + 1) * win->small_font_len_x;
+		int len_str = get_string_width_zoom((const unsigned char*)mqb_data[quickspell_over]->spell_name, win->font_category, zoom)
+			+ get_char_width_zoom(' ', win->font_category, zoom);
+
 		// vertical place left (or right) and aligned with slot
 		if (quickspells_dir==VERTICAL)
 		{
@@ -378,7 +383,7 @@ static int ui_scale_quickspell_handler(window_info *win)
 {
 	quickspell_size = (int)(0.5 + win->current_scale * 20);
 	quickspell_x_len = (int)(0.5 + win->current_scale * 26);
-	default_quickspells_x = window_width - (int)(0.5 + win->current_scale * 60);
+	default_quickspells_x = window_width - HUD_MARGIN_X + (int)(0.5 + win->current_scale * 4);
 	default_quickspells_y = get_hud_logo_size();
 	quickspell_y_space = (int)(0.5 + win->current_scale * 30);
 	if (!quickspells_relocatable)
@@ -400,6 +405,7 @@ static int ui_scale_quickspell_handler(window_info *win)
 
 void init_quickspell(void)
 {
+	int quickspell_win = get_id_MW(MW_QUICKSPELLS);
 	Uint32 flags = ELW_USE_UISCALE | ELW_CLICK_TRANSPARENT;
 
 	if (!quickspells_relocatable)
@@ -408,10 +414,12 @@ void init_quickspell(void)
 		quickspells_moveable = 0;
 	}
 	if (quickspells_moveable)
-		flags |= ELW_TITLE_BAR | ELW_DRAGGABLE;	
+		flags |= ELW_TITLE_BAR | ELW_DRAGGABLE;
 
 	if (quickspell_win < 0){
-		quickspell_win = create_window ("Quickspell", -1, 0, saved_quickspells_x, saved_quickspells_y, 0, 0, flags);
+		quickspell_win = create_window ("Quickspell", -1, 0, get_pos_x_MW(MW_QUICKSPELLS), get_pos_y_MW(MW_QUICKSPELLS), 0, 0, flags);
+		set_id_MW(MW_QUICKSPELLS, quickspell_win);
+		set_window_custom_scale(quickspell_win, MW_QUICKSPELLS);
 		set_window_handler(quickspell_win, ELW_HANDLER_DISPLAY, &display_quickspell_handler);
 		set_window_handler(quickspell_win, ELW_HANDLER_CLICK, &click_quickspell_handler);
 		set_window_handler(quickspell_win, ELW_HANDLER_MOUSEOVER, &mouseover_quickspell_handler );
@@ -440,7 +448,7 @@ void load_quickspells (void)
 	Uint8 num_spells;
 	FILE *fp;
 	Uint32 i, index;
-#if defined JSON_FILES
+#ifdef JSON_FILES
 	int json_num_spells = 0;
 	int quickspell_ids[MAX_QUICKSPELL_SLOTS];
 #endif
@@ -451,17 +459,24 @@ void load_quickspells (void)
 	// succeeds)
 	quickspells_loaded = 1;
 
-#if defined JSON_FILES
-	safe_snprintf(fname, sizeof(fname), "%sspells_%s.json", get_path_config(), get_lowercase_username());
-	if ((json_num_spells = json_load_quickspells(fname, quickspell_ids, MAX_QUICKSPELL_SLOTS)) >= 0)
+#ifdef JSON_FILES
+	if (get_use_json_user_files())
 	{
-		size_t i;
-		memset(mqb_data, 0, sizeof (mqb_data));
-		for (i = 0, index = 1; i < json_num_spells; i++)
-			if (quickspell_ids[i] >= 0)
-				mqb_data[index++] = build_quickspell_data(quickspell_ids[i]);
-		return;
+		USE_JSON_DEBUG("Loading json file");
+		safe_snprintf(fname, sizeof(fname), "%sspells_%s.json", get_path_config(), get_lowercase_username());
+		if ((json_num_spells = json_load_quickspells(fname, quickspell_ids, MAX_QUICKSPELL_SLOTS)) >= 0)
+		{
+			size_t i;
+			memset(mqb_data, 0, sizeof (mqb_data));
+			for (i = 0, index = 1; i < json_num_spells; i++)
+				if (quickspell_ids[i] >= 0)
+					mqb_data[index++] = build_quickspell_data(quickspell_ids[i]);
+			return;
+		}
 	}
+
+	// if there is no json file, or json use disabled, try to load the old binary format
+	USE_JSON_DEBUG("Loading binary file");
 #endif
 
 	//open the data file
@@ -543,7 +558,7 @@ void save_quickspells(void)
 	FILE *fp;
 	Uint8 i;
 	size_t num_spells_to_write = 0;
-#if defined JSON_FILES
+#ifdef JSON_FILES
 	size_t index;
 	Uint16 *quickspell_ids = NULL;
 #endif
@@ -559,29 +574,25 @@ void save_quickspells(void)
 		num_spells_to_write++;
 	}
 
-#if defined JSON_FILES
-	// save the quickspell to the json file
-	quickspell_ids = malloc(num_spells_to_write * sizeof(Uint16));
-	for (index = 0; index < num_spells_to_write; index++)
-		quickspell_ids[index] = mqb_data[index+1]->spell_id;
-	safe_snprintf(fname, sizeof(fname), "%sspells_%s.json", get_path_config(), get_lowercase_username());
-	if (json_save_quickspells(fname, quickspell_ids, num_spells_to_write) < 0)
+#ifdef JSON_FILES
+	if (get_use_json_user_files())
 	{
-		LOG_ERROR("%s: %s \"%s\"\n", reg_error_str, cant_open_file, fname);
+		USE_JSON_DEBUG("Saving json file");
+		// save the quickspell to the json file
+		quickspell_ids = malloc(num_spells_to_write * sizeof(Uint16));
+		for (index = 0; index < num_spells_to_write; index++)
+			quickspell_ids[index] = mqb_data[index+1]->spell_id;
+		safe_snprintf(fname, sizeof(fname), "%sspells_%s.json", get_path_config(), get_lowercase_username());
+		if (json_save_quickspells(fname, quickspell_ids, num_spells_to_write) < 0)
+			LOG_ERROR("%s: %s \"%s\"\n", reg_error_str, cant_open_file, fname);
+		free(quickspell_ids);
 		return;
 	}
-	free(quickspell_ids);
-
-	// we have written the json file, only write the binary file if one already exists
-	// this is to maintain backwards comatibility until the next forced version release
-	safe_snprintf(fname, sizeof(fname), "spells_%s.dat",get_lowercase_username());
-	if (file_exists_config(fname)!=1)
-		return;
-#else
-	safe_snprintf(fname, sizeof(fname), "spells_%s.dat",get_lowercase_username());
+	USE_JSON_DEBUG("Saving binary file");
 #endif
 
 	//write to the data file, for historical reasons, we will write all the information
+	safe_snprintf(fname, sizeof(fname), "spells_%s.dat",get_lowercase_username());
 	fp=open_file_config(fname,"wb");
 	if(fp == NULL){
 		LOG_ERROR("%s: %s \"%s\": %s\n", reg_error_str, cant_open_file, fname, strerror(errno));
@@ -609,7 +620,7 @@ void save_quickspells(void)
 				mqb_data[i]->spell_name, fname);
 			break;
 		}
-		
+
 		LOG_DEBUG("Wrote spell '%s' to file '%s'",
 			mqb_data[i]->spell_name, fname);
 	}
@@ -695,27 +706,48 @@ void add_quickspell(void)
 }
 
 
-// if relocatable, save the position and options to the el.cfg file
+// if relocatable, save the position and options to the cfg file
 void get_quickspell_options(unsigned int *options, unsigned int *position)
 {
-	if (quickspells_relocatable && quickspell_win >= 0 && quickspell_win < windows_list.num_windows)
-	{
+	int quickspell_win = get_id_MW(MW_QUICKSPELLS);
+	if (quickspell_win >= 0 && quickspell_win < windows_list.num_windows)
 		*position = windows_list.window[quickspell_win].cur_x | (windows_list.window[quickspell_win].cur_y << 16);
-		*options = (quickspells_dir & 1) | ((quickspells_moveable & 1) << 1);
-	}
+	else
+		*position = get_pos_x_MW(MW_QUICKSPELLS) | (get_pos_y_MW(MW_QUICKSPELLS) << 16);
+	*options = (quickspells_dir & 1) | ((quickspells_moveable & 1) << 1);
 }
 
 
-// if relocatable, set position and options from the el.cfg file
+// if relocatable, set position and options from the cfg file
 void set_quickspell_options(unsigned int options, unsigned int position)
 {
 	if (quickspells_relocatable)
 	{
-		saved_quickspells_x = position & 0xFFFF;
-		saved_quickspells_y = position >> 16;
+		set_pos_MW(MW_QUICKSPELLS, position & 0xFFFF, position >> 16);
 		quickspells_dir = options & 1;
 		quickspells_moveable = (options & 2) >> 1;
 	}
 	else
 		reset_quickspells();
 }
+
+
+#ifdef JSON_FILES
+void read_quickspell_options(const char *dict_name)
+{
+	if (quickspells_relocatable)
+	{
+		quickspells_dir = (json_cstate_get_bool(dict_name, "vertical", 1) == VERTICAL) ?VERTICAL : HORIZONTAL;
+		quickspells_moveable = json_cstate_get_bool(dict_name, "movable", 0);
+	}
+	else
+		reset_quickspells();
+}
+
+
+void write_quickspell_options(const char *dict_name)
+{
+	json_cstate_set_bool(dict_name, "vertical", (quickspells_dir == VERTICAL) ?1 :0);
+	json_cstate_set_bool(dict_name, "movable", quickspells_moveable);
+}
+#endif
