@@ -48,11 +48,7 @@ struct quantities quantities = {
 };
 int edit_quantity=-1;
 
-int item_action_mode=ACTION_WALK;
-
-int items_win= -1;
-int items_menu_x=10;
-int items_menu_y=20;
+static int item_action_mode = ACTION_WALK;
 
 int item_dragged=-1;
 int item_quantity=1;
@@ -60,6 +56,7 @@ int use_item=-1;
 int item_uid_enabled = 0;
 const Uint16 unset_item_uid = (Uint16)-1;
 int items_text[MAX_ITEMS_TEXTURES];
+int independant_inventory_action_modes = 0;
 
 /* title menu options */
 int allow_equip_swap=0;
@@ -93,6 +90,7 @@ static char items_string[350] = {0};
 static size_t last_items_string_id = 0;
 static const char *item_help_str = NULL;
 static const char *item_desc_str = NULL;
+static float button_text_zoom = 0.0f;
 
 #define NUMBUT 5
 static size_t buttons_cm_id[NUMBUT] = {CM_INIT_VALUE, CM_INIT_VALUE, CM_INIT_VALUE, CM_INIT_VALUE, CM_INIT_VALUE};
@@ -104,6 +102,14 @@ static struct { int last_dest; int move_to; int move_from; size_t string_id; Uin
 static int show_items_handler(window_info * win);
 static void drop_all_handler(void);
 
+// Limit external setting of the action mode: Called due to an action keypress or action icon in the icon window.
+void set_items_action_mode(int new_mode)
+{
+	// Only change the action mode if is one used by the window.
+	if (!independant_inventory_action_modes && ((new_mode == ACTION_WALK) || (new_mode == ACTION_LOOK) || (new_mode == ACTION_USE) || (new_mode == ACTION_USE_WITEM)))
+		item_action_mode = new_mode;
+}
+
 void set_shown_string(char colour_code, const char *the_text)
 {
 	if (strlen(the_text) == 0)
@@ -141,9 +147,9 @@ void gray_out(int x_start, int y_start, int gridsize){
 		glVertex3i(x_start,y_start,0);
 		glVertex3i(x_start+gridsize,y_start,0);
 		glVertex3i(x_start+gridsize,y_start+gridsize,0);
-		glVertex3i(x_start,y_start+gridsize,0);				
+		glVertex3i(x_start,y_start+gridsize,0);
 	glEnd();
-	glDisable(GL_BLEND);	
+	glDisable(GL_BLEND);
 	glEnable(GL_TEXTURE_2D);
 	glColor3f(1.0f, 1.0f, 1.0f);
 }
@@ -194,7 +200,7 @@ int get_mouse_pos_in_grid(int mx, int my, int columns, int rows, int left, int t
 void reset_quantity (int pos)
 {
 	int val;
-					
+
 	switch(pos)
 	{
 		case 0:
@@ -243,13 +249,12 @@ void drag_item(int item, int storage, int mini)
 	int offset = (mini) ? my_items_grid_size/3 : my_items_grid_size/2;
 
 	int quantity=item_quantity;
-	char str[20];
-	
+
 	if(storage) {
 		if (item < 0 || item >= STORAGE_ITEMS_SIZE)
 			// oops
 			return;
-		
+
 		cur_item=storage_items[item].image_id;
 		if(!storage_items[item].quantity) {
 			use_item = storage_item_dragged=-1;
@@ -261,15 +266,29 @@ void drag_item(int item, int storage, int mini)
 		if (item < 0 || item >= ITEM_NUM_ITEMS)
 			// oops
 			return;
-		
+
 		cur_item=item_list[item].image_id;
 		if(!item_list[item].quantity) {
 			use_item = item_dragged=-1;
 			return;
 		}
-		if(item_list[item].is_stackable){
-			if(quantity>item_list[item].quantity)quantity=item_list[item].quantity;
-		} else quantity=-1;//The quantity for non-stackable items is misleading so don't show it...
+		if (item >= ITEM_WEAR_START) {
+			quantity = -1;
+		} else if (item_list[item].is_stackable) {
+			if(quantity > item_list[item].quantity)
+				quantity = item_list[item].quantity;
+		// The quantity for non-stackable items is misleading so don't show it unless we can reliably count how many we have.
+		} else if (item_list[item].id == unset_item_uid) {
+			quantity = -1;
+		} else {
+			size_t i;
+			int count = 0;
+			for (i = 0; i < ITEM_WEAR_START; i++)
+				if((item_list[i].quantity > 0) && (item_list[item].image_id == item_list[i].image_id) && (item_list[item].id == item_list[i].id))
+					count++;
+			if (quantity > count)
+				quantity = count;
+		}
 	}
 
 	cur_item_img=cur_item%25;
@@ -282,10 +301,14 @@ void drag_item(int item, int storage, int mini)
 	glBegin(GL_QUADS);
 	draw_2d_thing(u_start, v_start, u_end, v_end, mouse_x - offset, mouse_y - offset, mouse_x + offset, mouse_y + offset);
 	glEnd();
-	
-	if(!mini && quantity!=-1){
-		safe_snprintf(str,sizeof(str),"%i",quantity);
-		draw_string_small_zoomed(mouse_x-offset, mouse_y + offset - get_global_scale() * SMALL_FONT_Y_LEN, (unsigned char*)str, 1, get_global_scale());
+
+	if (!mini && quantity != -1)
+	{
+		int text_height = get_line_height(UI_FONT, get_global_scale() * DEFAULT_SMALL_RATIO);
+		unsigned char str[20];
+		safe_snprintf((char*)str, sizeof(str), "%d", quantity);
+		draw_string_small_zoomed_centered(mouse_x, mouse_y - offset - text_height,
+			str, 1, get_global_scale());
 	}
 }
 
@@ -305,16 +328,16 @@ void get_your_items (const Uint8 *data)
 	//data[7] -> pos
 	//data[8] -> flags
 	//data[9] -> id
-	
-	
+
+
 	total_items=data[0];
-	
+
 	//clear the items first
 	for(i=0;i<ITEM_NUM_ITEMS;i++){
 		item_list[i].quantity=0;
 		item_list_extra[i].slot_busy_start = 0;
 	}
-	
+
 	for(i=0;i<total_items;i++){
 		pos=data[i*len+1+6];
 		// try not to wipe out cooldown information if no real change
@@ -348,7 +371,7 @@ void get_your_items (const Uint8 *data)
 void check_for_item_sound(int pos)
 {
 	int i, snd = -1;
-	
+
 #ifdef _EXTRA_SOUND_DEBUG
 //	printf("Used item: %d, Image ID: %d, Action: %d\n", pos, item_list[pos].image_id, item_list[pos].action);
 #endif // _EXTRA_SOUND_DEBUG
@@ -396,7 +419,7 @@ void update_item_sound(int interval)
 				item_list[i].action_time = 0;
 			}
 		}
-	}	
+	}
 }
 #endif // NEW_SOUND
 
@@ -462,10 +485,27 @@ static void used_item_counter_check_confirmation(int pos, int new_quanity)
 	{
 		if ((item_list[pos].quantity > 0) && (new_quanity < item_list[pos].quantity))
 		{
+			static int sent_unique_message = 0;
+			static int sent_get_failed_message = 0;
+			int item_count = get_item_count(item_list[pos].id, item_list[pos].image_id);
 			used_item_counter_queue[pos]--;
 			if (!used_item_counter_queue[pos])
 				used_item_counter_queue_time[pos] = 0;
-			increment_used_item_counter(get_item_description(item_list[pos].id, item_list[pos].image_id), item_list[pos].quantity - new_quanity);
+			if (item_count == 1)
+				increment_used_item_counter(get_basic_item_description(item_list[pos].id, item_list[pos].image_id), item_list[pos].quantity - new_quanity);
+			else if ((item_count > 1) && !sent_unique_message)
+			{
+				LOG_TO_CONSOLE(c_red1, item_use_not_unique_str);
+				LOG_TO_CONSOLE(c_red1, item_uid_help_str);
+				sent_unique_message = 1;
+			}
+			else if ((item_count < 1) && !sent_get_failed_message)
+			{
+				char str[256];
+				safe_snprintf(str, sizeof(str), "%s id=%d image_id=%d", item_use_get_failed_str, item_list[pos].id, item_list[pos].image_id);
+				LOG_TO_CONSOLE(c_red1, str);
+				sent_get_failed_message = 1;
+			}
 			//printf("Used item counter confirmed pos %d - item removed [%s] quanity %d\n", pos, get_item_description(item_list[pos].id, item_list[pos].image_id), item_list[pos].quantity - new_quanity);
 		}
 		//else
@@ -482,11 +522,11 @@ void remove_item_from_inventory(int pos)
 
 	if (pos == swap_complete.move_from)
 		swap_complete.move_to = swap_complete.move_from = -1;
-	
+
 #ifdef NEW_SOUND
 	check_for_item_sound(pos);
 #endif // NEW_SOUND
-	
+
 	build_manufacture_list();
 	check_castability();
 }
@@ -527,11 +567,11 @@ void get_new_inventory_item (const Uint8 *data)
 	item_list[pos].is_reagent=((flags&ITEM_REAGENT)>0);
 	item_list[pos].use_with_inventory=((flags&ITEM_INVENTORY_USABLE)>0);
 	item_list[pos].is_stackable=((flags&ITEM_STACKABLE)>0);
-	
+
 #ifdef NEW_SOUND
 	check_for_item_sound(pos);
 #endif // NEW_SOUND
-	
+
 	build_manufacture_list();
 	check_castability();
 
@@ -561,7 +601,7 @@ void draw_item(int id, int x_start, int y_start, int gridsize){
 
 	//get the texture this item belongs to
 	this_texture=get_items_texture(id/25);
-		
+
 	bind_texture(this_texture);
 	glBegin(GL_QUADS);
 		draw_2d_thing(u_start,v_start,u_end,v_end,x_start,y_start,x_start+gridsize-1,y_start+gridsize-1);
@@ -574,7 +614,6 @@ static int display_items_handler(window_info *win)
 	char my_str[10];
 	int x,y,i;
 	int item_is_weared=0;
-	float equip_string_zoom;
 	Uint32 _cur_time = SDL_GetTicks(); /* grab a snapshot of current time */
 	char *but_labels[NUMBUT] = { sto_all_str, get_all_str, drp_all_str, NULL, itm_lst_str };
 
@@ -582,7 +621,7 @@ static int display_items_handler(window_info *win)
 
 	check_for_swap_completion();
 
-	/* 
+	/*
 	* Labrat: I never realised that a store all patch had been posted to Berlios by Awn in February '07
 	* Thanks to Awn for his earlier efforts (but this is not a derivative of his earlier work)
 	*
@@ -592,35 +631,38 @@ static int display_items_handler(window_info *win)
 	// draw the button labels
 	but_labels[BUT_MIX] = (items_mix_but_all) ?mix_all_str :mix_one_str;
 	for (i=0; i<NUMBUT; i++) {
-		int text_x_offset = (int)(0.5 + ((buttons_grid.width - (float)(3 * win->small_font_len_x)) / 2.0)) + gx_adjust;
-		int text_y_offset = (int)(0.5 + ((buttons_grid.height - (float)(2 * win->small_font_len_y)) / 2.0)) + gy_adjust;
 		strap_word(but_labels[i],my_str);
-		glColor3f(0.77f,0.57f,0.39f);
-		draw_string_small_zoomed(buttons_grid.pos_x + text_x_offset, buttons_grid.pos_y + buttons_grid.height * i + text_y_offset, (unsigned char*)my_str, 2, win->current_scale);
+		glColor3fv(gui_color);
+		draw_text(buttons_grid.pos_x + buttons_grid.width/2,
+			buttons_grid.pos_y + buttons_grid.height * i + buttons_grid.height/2,
+			(const unsigned char*)my_str, strlen(my_str), win->font_category,
+			TDO_ZOOM, button_text_zoom, TDO_ALIGNMENT, CENTER, TDO_VERTICAL_ALIGNMENT, CENTER_LINE,
+			TDO_END);
 	}
 
 	x = quantity_grid.pos_x + quantity_grid.width / 2;
-	y = quantity_grid.pos_y + (quantity_grid.height - win->small_font_len_y) / 2;
-	glColor3f(0.3f,0.5f,1.0f);
+	y = quantity_grid.pos_y + quantity_grid.height / 2;
 	for(i = 0; i < ITEM_EDIT_QUANT; x += quantity_grid.width, ++i){
 		if(i==edit_quantity){
 			glColor3f(1.0f, 0.0f, 0.3f);
-			draw_string_small_zoomed(1+gx_adjust+x-strlen(quantities.quantity[i].str)*win->small_font_len_x/2, y+gy_adjust, (unsigned char*)quantities.quantity[i].str, 1, win->current_scale);
-			glColor3f(0.3f, 0.5f, 1.0f);
 		} else if(i==quantities.selected){
 			glColor3f(0.0f, 1.0f, 0.3f);
-			draw_string_small_zoomed(1+gx_adjust+x-strlen(quantities.quantity[i].str)*win->small_font_len_x/2, y+gy_adjust, (unsigned char*)quantities.quantity[i].str, 1, win->current_scale);
-			glColor3f(0.3f, 0.5f, 1.0f);
-		} else draw_string_small_zoomed(1+gx_adjust+x-strlen(quantities.quantity[i].str)*win->small_font_len_x/2, y+gy_adjust, (unsigned char*)quantities.quantity[i].str, 1, win->current_scale);
+		} else {
+			glColor3f(0.3f,0.5f,1.0f);
+		}
+		draw_text(x, y, (const unsigned char*)quantities.quantity[i].str,
+			strlen(quantities.quantity[i].str), win->font_category, TDO_ZOOM, win->current_scale_small,
+			TDO_ALIGNMENT, CENTER, TDO_VERTICAL_ALIGNMENT, CENTER_DIGITS, TDO_END);
 	}
-	draw_string_small_zoomed(labels_box.pos_x + labels_box.len_x - strlen(quantity_str) * win->small_font_len_x, labels_box.pos_y, (unsigned char*)quantity_str, 1, win->current_scale);
+	glColor3f(0.3f,0.5f,1.0f);
+	draw_string_small_zoomed_right(labels_box.pos_x + labels_box.len_x, labels_box.pos_y,
+		(const unsigned char*)quantity_str, 1, win->current_scale);
 
 	glColor3f(0.57f,0.67f,0.49f);
-	equip_string_zoom = (float)equip_grid.len_x / (SMALL_FONT_X_LEN * strlen(equip_str));
-	if (equip_string_zoom > win->current_scale)
-		equip_string_zoom = win->current_scale;
-	draw_string_small_zoomed (gx_adjust + equip_grid.pos_x + equip_grid.len_x / 2 - (SMALL_FONT_X_LEN * equip_string_zoom * strlen(equip_str))/2,
-		gy_adjust + equip_grid.pos_y - 0.9 * SMALL_FONT_Y_LEN * win->current_scale + 1, (unsigned char*)equip_str, 1, equip_string_zoom);
+	draw_text(equip_grid.pos_x + equip_grid.len_x / 2, equip_grid.pos_y, (const unsigned char*)equip_str,
+		strlen(equip_str), win->font_category, TDO_MAX_WIDTH, equip_grid.len_x,
+		TDO_ZOOM, win->current_scale_small, TDO_SHRINK_TO_FIT, 1, TDO_ALIGNMENT, CENTER,
+		TDO_VERTICAL_ALIGNMENT, BOTTOM_LINE, TDO_END);
 
 	glColor3f(1.0f,1.0f,1.0f);
 	//ok, now let's draw the objects...
@@ -710,7 +752,7 @@ static int display_items_handler(window_info *win)
 				glDisable(GL_BLEND);
 				glEnable(GL_TEXTURE_2D);
 			}
-			
+
 			if(!item_is_weared){
 				int use_large = (items_grid.mouse_over == i) && enlarge_text();
 				safe_snprintf(str, sizeof(str), "%i", item_list[i].quantity);
@@ -735,7 +777,8 @@ static int display_items_handler(window_info *win)
 	{
 		if (last_items_string_id != inventory_item_string_id)
 		{
-			put_small_text_in_box_zoomed((unsigned char*)inventory_item_string, strlen(inventory_item_string), message_box.len_x, items_string, win->current_scale);
+			put_small_text_in_box_zoomed((const unsigned char*)inventory_item_string, strlen(inventory_item_string), message_box.len_x,
+			(unsigned char*)items_string, win->current_scale);
 			last_items_string_id = inventory_item_string_id;
 		}
 		draw_string_small_zoomed(message_box.pos_x, message_box.pos_y, (unsigned char*)items_string, message_box.rows, win->current_scale);
@@ -744,9 +787,9 @@ static int display_items_handler(window_info *win)
 	glDisable(GL_TEXTURE_2D);
 
 	if (text_arrow.mouse_over != -1)
-		glColor3f(0.99f,0.77f,0.55f);
+		glColor3fv(gui_bright_color);
 	else
-		glColor3f(0.77f,0.57f,0.39f);
+		glColor3fv(gui_color);
 	text_arrow.mouse_over = -1;
 	if (items_disable_text_block)
 	{
@@ -759,7 +802,7 @@ static int display_items_handler(window_info *win)
 		// if we have a coloured message, draw a small dot at the top of the arrow to indicate so, using the colour of the message
 		if ((strlen(inventory_item_string) > 0) && is_color(inventory_item_string[0]))
 		{
-			size_t colour = from_color_char (inventory_item_string[0]);
+			int colour = from_color_char (inventory_item_string[0]);
 			if ((colour >= c_lbound) && (colour <= c_ubound))
 			{
 				glColor4f((float) colors_list[colour].r1 / 255.0f, (float) colors_list[colour].g1 / 255.0f, (float) colors_list[colour].b1 / 255.0f, 1.0f);
@@ -786,9 +829,9 @@ static int display_items_handler(window_info *win)
 	}
 
 	if (unequip_arrow.mouse_over != -1)
-		glColor3f(0.99f,0.77f,0.55f);
+		glColor3fv(gui_bright_color);
 	else
-		glColor3f(0.77f,0.57f,0.39f);
+		glColor3fv(gui_color);
 	unequip_arrow.mouse_over = -1;
 	if (items_equip_grid_on_left)
 	{
@@ -826,18 +869,18 @@ static int display_items_handler(window_info *win)
 	}
 
 	// Render the grid *after* the images. It seems impossible to code
-	// it such that images are rendered exactly within the boxes on all 
+	// it such that images are rendered exactly within the boxes on all
 	// cards
-	glColor3f(0.77f,0.57f,0.39f);
+	glColor3fv(gui_color);
 
 	//draw the grids
 	rendergrid(items_grid.cols, items_grid.rows, items_grid.pos_x, items_grid.pos_y, items_grid.width, items_grid.height);
-	
+
 	glColor3f(0.57f,0.67f,0.49f);
 	rendergrid(equip_grid.cols, equip_grid.rows, equip_grid.pos_x, equip_grid.pos_y, equip_grid.width, equip_grid.height);
-	
+
 	// draw the button boxes
-	glColor3f(0.77f,0.57f,0.39f);
+	glColor3fv(gui_color);
 	for (i=0; i<NUMBUT; i++) {
 		glBegin(GL_LINE_LOOP);
 			glVertex3i(buttons_grid.pos_x + buttons_grid.width, buttons_grid.pos_y + buttons_grid.height * i, 0);
@@ -846,11 +889,11 @@ static int display_items_handler(window_info *win)
 			glVertex3i(buttons_grid.pos_x + buttons_grid.width, buttons_grid.pos_y + buttons_grid.height * i + buttons_grid.height, 0);
 		glEnd();
 	}
-	
+
 	// highlight a button with the mouse over
 	if (buttons_grid.mouse_over != -1)
 	{
-		glColor3f(0.99f,0.77f,0.55f);
+		glColor3fv(gui_bright_color);
 		glBegin(GL_LINE_LOOP);
 			glVertex3i(buttons_grid.pos_x + buttons_grid.width + 1, buttons_grid.pos_y + buttons_grid.height * buttons_grid.mouse_over - 1, 0);
 			glVertex3i(buttons_grid.pos_x - 1, buttons_grid.pos_y + buttons_grid.height * buttons_grid.mouse_over - 1, 0);
@@ -862,9 +905,9 @@ static int display_items_handler(window_info *win)
 	//now, draw the quantity boxes
 	glColor3f(0.3f,0.5f,1.0f);
 	rendergrid(quantity_grid.cols, quantity_grid.rows, quantity_grid.pos_x, quantity_grid.pos_y, quantity_grid.width, quantity_grid.height);
-	
+
 	glEnable(GL_TEXTURE_2D);
-	
+
 	// display help text for button if mouse over one
 	if ((buttons_grid.mouse_over != -1) && show_help_text) {
 		char *helpstr[NUMBUT] = { stoall_help_str, getall_help_str, ((disable_double_click) ?drpall_help_str :dcdrpall_help_str), mixoneall_help_str, itmlst_help_str };
@@ -929,7 +972,7 @@ int move_item(int item_pos_to_mov, int destination_pos, int avoid_pos)
 			else
 				set_shown_string(c_red2, items_stack_str);
 			/*  This still leaves one possibility for the dreaded server accusation.
-				If we have no free inventory slots, one or more stackable items 
+				If we have no free inventory slots, one or more stackable items
 				unequipped, and a single, different equipped item with the same id as
 				the aforementioned stack.  When we try to unequip the single item, the
 				client tries to place it on that stack. This may mean we have to
@@ -939,12 +982,8 @@ int move_item(int item_pos_to_mov, int destination_pos, int avoid_pos)
 	}
 	/* move item */
 	if(drop_on_stack || !item_list[destination_pos].quantity){
-		Uint8 str[20];
-		//send the drop info to the server
-		str[0]=MOVE_INVENTORY_ITEM;
-		str[1]=item_list[item_pos_to_mov].pos;
-		str[2]=destination_pos;
-		my_tcp_send(my_socket,str,3);
+		Uint8 str[3] = { MOVE_INVENTORY_ITEM, item_list[item_pos_to_mov].pos, destination_pos };
+		my_tcp_send(str,3);
 		swap_complete.last_dest = destination_pos;
 		return 1;
 	}
@@ -954,12 +993,8 @@ int move_item(int item_pos_to_mov, int destination_pos, int avoid_pos)
 
 static void equip_item(int item_pos_to_equip, int destination_pos)
 {
-	Uint8 str[20];
-	//send the drop info to the server
-	str[0]=MOVE_INVENTORY_ITEM;
-	str[1]=item_list[item_pos_to_equip].pos;
-	str[2]=destination_pos;
-	my_tcp_send(my_socket,str,3);
+	Uint8 str[3] = { MOVE_INVENTORY_ITEM, item_list[item_pos_to_equip].pos, destination_pos };
+	my_tcp_send(str,3);
 }
 
 static void prep_move_to_vacated_slot(int destination)
@@ -1088,7 +1123,7 @@ void try_auto_equip(int from_item)
 }
 
 static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
-{	
+{
 	Uint8 str[100];
 	int right_click = flags & ELW_RIGHT_MOUSE;
 	int ctrl_on = flags & KMOD_CTRL;
@@ -1171,7 +1206,7 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 			item_action_mode=ACTION_WALK;
 			return 1;
 		}
-		
+
 		if((mx >= equip_grid.pos_x) && (mx < equip_grid.pos_x + equip_grid.len_x) &&
 				(my >= equip_grid.pos_y) && (my < equip_grid.pos_y + equip_grid.len_y + 1)) {
 			switch(item_action_mode){
@@ -1206,8 +1241,9 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 			return 1;
 		}
 	}
-	
-	if(item_action_mode==ACTION_USE_WITEM)	action_mode=ACTION_USE_WITEM;
+
+	if (item_action_mode == ACTION_USE_WITEM)
+		set_gamewin_usewith_action();
 
 	//see if we changed the quantity
 	if((mx >= quantity_grid.pos_x) && (mx < quantity_grid.pos_x + quantity_grid.len_x) &&
@@ -1223,14 +1259,14 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 				}
 				edit_quantity=-1;
 			}
-			
+
 			item_quantity=quantities.quantity[pos].val;
 			quantities.selected=pos;
 		} else if(right_click){
 			//Edit the given quantity
 			edit_quantity=pos;
 		}
-		
+
 		return 1;
 	}
 
@@ -1240,12 +1276,12 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 		quantities.selected=edit_quantity;
 		edit_quantity=-1;
 	}
-	
+
 	//see if we clicked on any item in the main category
 	else if(mx>items_grid.pos_x && (mx < items_grid.pos_x + items_grid.len_x) &&
 				my>0 && my < items_grid.len_y) {
 		int pos=get_mouse_pos_in_grid(mx, my, items_grid.cols, items_grid.rows, items_grid.pos_x, items_grid.pos_y, items_grid.width, items_grid.height);
-		
+
 #ifdef NEW_SOUND
 		if(pos>-1) {
 			item_list[pos].action = ITEM_NO_ACTION;
@@ -1265,13 +1301,13 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 				}
 				item_dragged=-1;
 			}
-		
+
 		}
 		else if(storage_item_dragged!=-1){
 			str[0]=WITHDRAW_ITEM;
 			*((Uint16*)(str+1))=SDL_SwapLE16(storage_items[storage_item_dragged].pos);
 			*((Uint32*)(str+3))=SDL_SwapLE32(item_quantity);
-			my_tcp_send(my_socket, str, 6);
+			my_tcp_send(str, 7);
 			do_drop_item_sound();
 			if(storage_items[storage_item_dragged].quantity<=item_quantity) storage_item_dragged=-1;
 		}
@@ -1281,16 +1317,17 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 				str[1]=item_list[pos].pos;
 				if(item_list[pos].is_stackable)
 					*((Uint32 *)(str+2))=SDL_SwapLE32(item_list[pos].quantity);
-				else 
+				else
 					*((Uint32 *)(str+2))=SDL_SwapLE32(36);//Drop all
-				my_tcp_send(my_socket, str, 6);
+				my_tcp_send(str, 6);
 				do_drop_item_sound();
 			} else if (alt_on && (items_mod_click_any_cursor || (item_action_mode==ACTION_WALK))) {
-				if ((storage_win >= 0) && (get_show_window(storage_win)) && (view_only_storage == 0)) {
+				if ((get_id_MW(MW_STORAGE) >= 0) && (view_only_storage == 0) && storage_use_allowed) {
+					reset_storage_rate_limit();
 					str[0]=DEPOSITE_ITEM;
 					str[1]=item_list[pos].pos;
 					*((Uint32*)(str+2))=SDL_SwapLE32(INT_MAX);
-					my_tcp_send(my_socket, str, 6);
+					my_tcp_send(str, 6);
 					do_drop_item_sound();
 				} else {
 					drop_fail_time = SDL_GetTicks();
@@ -1300,12 +1337,12 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 				click_time=cur_time;
 				str[0]=LOOK_AT_INVENTORY_ITEM;
 				str[1]=item_list[pos].pos;
-				my_tcp_send(my_socket,str,2);
+				my_tcp_send(str,2);
 			} else if(item_action_mode==ACTION_USE) {
 				if(item_list[pos].use_with_inventory){
 					str[0]=USE_INVENTORY_ITEM;
 					str[1]=item_list[pos].pos;
-					my_tcp_send(my_socket,str,2);
+					my_tcp_send(str,2);
 					used_item_counter_action_use(pos);
 #ifdef NEW_SOUND
 					item_list[pos].action = USE_INVENTORY_ITEM;
@@ -1319,7 +1356,7 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 					str[0]=ITEM_ON_ITEM;
 					str[1]=item_list[use_item].pos;
 					str[2]=item_list[pos].pos;
-					my_tcp_send(my_socket,str,3);
+					my_tcp_send(str,3);
 					used_item_counter_action_use(use_item);
 #ifdef NEW_SOUND
 					item_list[use_item].action = ITEM_ON_ITEM;
@@ -1338,7 +1375,7 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 				do_drag_item_sound();
 			}
 		}
-	} 
+	}
 
 	// Get All button
 	else if(over_button(win, mx, my)==BUT_GET){
@@ -1351,24 +1388,32 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 	}
 
 	// Sto All button
-	else if(over_button(win, mx, my)==BUT_STORE && storage_win >= 0 && view_only_storage == 0 && get_show_window(storage_win) /*thanks alberich*/){
+	else if (over_button(win, mx, my) == BUT_STORE) {
+		if ((get_id_MW(MW_STORAGE) >= 0) && (view_only_storage == 0) && storage_use_allowed) {
+			reset_storage_rate_limit();
 #ifdef STORE_ALL
-		/*
-		* Future code to save server load by having one byte to represent the 36 slot inventory loop. Will need server support.
-		*/
-		str[0]=DEPOSITE_ITEM;
-		str[1]=STORE_ALL;
-		my_tcp_send(my_socket, str, 2);
+			/*
+			* Future code to save server load by having one byte to represent the 36 slot inventory loop. Will need server support.
+			*/
+			str[0]=DEPOSITE_ITEM;
+			str[1]=STORE_ALL;
+			my_tcp_send(str, 2);
 #else
-		for(pos=((items_stoall_nofirstrow)?6:0);pos<((items_stoall_nolastrow)?30:36);pos++){
-			if(item_list[pos].quantity>0){
-				str[0]=DEPOSITE_ITEM;
-				str[1]=item_list[pos].pos;
-				*((Uint32*)(str+2))=SDL_SwapLE32(item_list[pos].quantity);
-				my_tcp_send(my_socket, str, 6);
+			for(pos=((items_stoall_nofirstrow)?6:0);pos<((items_stoall_nolastrow)?30:36);pos++){
+				if(item_list[pos].quantity>0){
+					str[0]=DEPOSITE_ITEM;
+					str[1]=item_list[pos].pos;
+					*((Uint32*)(str+2))=SDL_SwapLE32(item_list[pos].quantity);
+					my_tcp_send(str, 6);
+				}
 			}
-		}
 #endif
+		}
+		else
+		{
+			drop_fail_time = SDL_GetTicks();
+			do_alert1_sound();
+		}
 	}
 
 	// Drop All button
@@ -1392,13 +1437,13 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 	else if((mx > equip_grid.pos_x) && (mx < equip_grid.pos_x + equip_grid.len_x) &&
 				(my > equip_grid.pos_y) && (my < equip_grid.pos_y + equip_grid.len_y)){
 		int pos = ITEM_WEAR_START + get_mouse_pos_in_grid(mx, my, equip_grid.cols, equip_grid.rows, equip_grid.pos_x, equip_grid.pos_y, equip_grid.width, equip_grid.height);
-		
+
 		if(pos < ITEM_WEAR_START) {
 		} else if(item_list[pos].quantity){
 			if(item_action_mode == ACTION_LOOK) {
 				str[0]=LOOK_AT_INVENTORY_ITEM;
 				str[1]=item_list[pos].pos;
-				my_tcp_send(my_socket, str, 2);
+				my_tcp_send(str, 2);
 			} else if(item_dragged==-1 && left_click) {
 				item_dragged=pos;
 				do_drag_item_sound();
@@ -1433,7 +1478,7 @@ static int click_items_handler(window_info *win, int mx, int my, Uint32 flags)
 			return 1;
 		}
 	}
-	
+
 	return 1;
 }
 
@@ -1447,7 +1492,7 @@ static void set_description_help(int pos)
 
 static int mouseover_items_handler(window_info *win, int mx, int my) {
 	int pos;
-	
+
 	// check and record if mouse if over a button
 	if ((buttons_grid.mouse_over = over_button(win, mx, my)) != -1)
 		return 0; // keep standard cursor
@@ -1517,7 +1562,7 @@ static int mouseover_items_handler(window_info *win, int mx, int my) {
 			(my > message_box.pos_y) && (my < message_box.pos_y + message_box.len_y)) {
 		item_help_str = (disable_double_click)?click_clear_str :double_click_clear_str;
 	}
-	
+
 	return 0;
 }
 
@@ -1585,7 +1630,7 @@ static void drop_all_handler (void)
 				str[0] = DROP_ITEM;
 				str[1] = item_list[i].pos;
 				*((Uint32 *)(str+2)) = SDL_SwapLE32(item_list[i].quantity);
-				my_tcp_send (my_socket, str, 6);
+				my_tcp_send(str, 6);
 				dropped_something = 1;
 			}
 		}
@@ -1594,6 +1639,32 @@ static void drop_all_handler (void)
 	}
 }
 
+static int get_max_but_label_len(window_info * win)
+{
+	char const *but_labels[] = { sto_all_str, get_all_str, drp_all_str, mix_all_str, mix_one_str, itm_lst_str };
+	int max_width = 0;
+	char *buf = NULL;
+	size_t i;
+	if (win == NULL)
+		return 0;
+	max_width = win->box_size;
+	for (i = 0; i < sizeof(but_labels) / sizeof(char *); i++)
+	{
+		char *saveptr = NULL;
+		char *next = NULL;
+		buf = realloc(buf, strlen(but_labels[i]) + 1);
+		strcpy((char *)buf, but_labels[i]);
+		next = strtok_r(buf, " ", &saveptr);
+		while (next != NULL)
+		{
+			max_width = max2i(max_width, get_buf_width_zoom((unsigned char *)next, strlen(next), win->font_category, button_text_zoom));
+			next = strtok_r(NULL, " ", &saveptr);
+		}
+	}
+	free(buf);
+	return max_width;
+}
+
 static int show_items_handler(window_info * win)
 {
 	int i, win_x_len, win_y_len;
@@ -1617,9 +1688,11 @@ static int show_items_handler(window_info * win)
 	equip_grid.len_y = equip_grid.height * equip_grid.rows;;
 
 	/* the buttons */
+	button_text_zoom = win->current_scale_small
+		* min2f(1.0, 0.5 * equip_grid.height / win->small_font_len_y);
 	buttons_grid.cols = 1;
 	buttons_grid.rows = NUMBUT;
-	buttons_grid.width = (int)(0.5 + 4 * win->small_font_len_x);
+	buttons_grid.width = get_max_but_label_len(win) + (int)(0.5 + 6 * button_text_zoom);
 	buttons_grid.height = equip_grid.height;
 	buttons_grid.len_x = buttons_grid.width * buttons_grid.cols;
 	buttons_grid.len_y = buttons_grid.height * buttons_grid.rows;
@@ -1715,9 +1788,9 @@ static int show_items_handler(window_info * win)
 
 	resize_window(win->window_id, win_x_len, win_y_len);
 
-	cm_remove_regions(items_win);
+	cm_remove_regions(win->window_id);
 	for (i=0; i<NUMBUT; i++)
-		cm_add_region(buttons_cm_id[i], items_win, buttons_grid.pos_x, buttons_grid.pos_y + buttons_grid.height * i, buttons_grid.width, buttons_grid.height);
+		cm_add_region(buttons_cm_id[i], win->window_id, buttons_grid.pos_x, buttons_grid.pos_y + buttons_grid.height * i, buttons_grid.width, buttons_grid.height);
 
 	/* make sure we redraw any string */
 	last_items_string_id = 0;
@@ -1740,21 +1813,21 @@ static int context_items_handler(window_info *win, int widget_id, int mx, int my
 
 void display_items_menu()
 {
+	int items_win = get_id_MW(MW_ITEMS);
 	if(items_win < 0){
-		int our_root_win = -1;
-		if (!windows_on_top) {
-			our_root_win = game_root_win;
-		}
-		items_win= create_window(win_inventory, our_root_win, 0, items_menu_x, items_menu_y, 0, 0, ELW_USE_UISCALE|ELW_WIN_DEFAULT);
+		items_win = create_window(win_inventory, (not_on_top_now(MW_ITEMS) ?game_root_win : -1), 0, get_pos_x_MW(MW_ITEMS), get_pos_y_MW(MW_ITEMS),
+			0, 0, ELW_USE_UISCALE|ELW_WIN_DEFAULT);
+		set_id_MW(MW_ITEMS, items_win);
 
-		set_window_custom_scale(items_win, &custom_scale_factors.items);
+		set_window_custom_scale(items_win, MW_ITEMS);
 		set_window_handler(items_win, ELW_HANDLER_DISPLAY, &display_items_handler );
 		set_window_handler(items_win, ELW_HANDLER_CLICK, &click_items_handler );
 		set_window_handler(items_win, ELW_HANDLER_MOUSEOVER, &mouseover_items_handler );
 		set_window_handler(items_win, ELW_HANDLER_KEYPRESS, (int (*)())&keypress_items_handler );
 		set_window_handler(items_win, ELW_HANDLER_SHOW, &show_items_handler );
 		set_window_handler(items_win, ELW_HANDLER_UI_SCALE, &show_items_handler );
-		
+		set_window_handler(items_win, ELW_HANDLER_FONT_CHANGE, &show_items_handler);
+
 		cm_add(windows_list.window[items_win].cm_id, cm_items_menu_str, context_items_handler);
 		cm_bool_line(windows_list.window[items_win].cm_id, ELW_CM_MENU_LEN+1, &use_small_items_window, NULL);
 		cm_bool_line(windows_list.window[items_win].cm_id, ELW_CM_MENU_LEN+2, &manual_size_items_window, NULL);
@@ -1767,11 +1840,11 @@ void display_items_menu()
 		buttons_cm_id[BUT_STORE] = cm_create(inv_keeprow_str, NULL);
 		cm_bool_line(buttons_cm_id[BUT_STORE], 0, &items_stoall_nofirstrow, NULL);
 		cm_bool_line(buttons_cm_id[BUT_STORE], 1, &items_stoall_nolastrow, NULL);
-		
+
 		buttons_cm_id[BUT_DROP] = cm_create(inv_keeprow_str, NULL);
 		cm_bool_line(buttons_cm_id[BUT_DROP], 0, &items_dropall_nofirstrow, NULL);
 		cm_bool_line(buttons_cm_id[BUT_DROP], 1, &items_dropall_nolastrow, NULL);
-		
+
 		buttons_cm_id[BUT_MIX] = cm_create(mix_all_str, NULL);
 		cm_bool_line(buttons_cm_id[BUT_MIX], 0, &items_mix_but_all, NULL);
 
@@ -1782,6 +1855,7 @@ void display_items_menu()
 		cm_bool_line(buttons_cm_id[BUT_ITEM_LIST], 0, &items_list_on_left, NULL);
 
 		show_items_handler(&windows_list.window[items_win]);
+		check_proportional_move(MW_ITEMS);
 
 	} else {
 		show_window(items_win);
@@ -1793,7 +1867,7 @@ void get_items_cooldown (const Uint8 *data, int len)
 {
 	int iitem, nitems, ibyte, pos;
 	Uint8 cooldown, max_cooldown;
-	
+
 	// reset old cooldown values
 	for (iitem = 0; iitem < ITEM_NUM_ITEMS; iitem++)
 	{
@@ -1803,7 +1877,7 @@ void get_items_cooldown (const Uint8 *data, int len)
 
 	nitems = len / 5;
 	if (nitems <= 0) return;
-	
+
 	ibyte = 0;
 	for (iitem = 0; iitem < nitems; iitem++)
 	{
@@ -1811,7 +1885,7 @@ void get_items_cooldown (const Uint8 *data, int len)
 		max_cooldown = SDL_SwapLE16 (*((Uint16*)(&data[ibyte+1])));
 		cooldown = SDL_SwapLE16 (*((Uint16*)(&data[ibyte+3])));
 		ibyte += 5;
-		
+
 		item_list[pos].cooldown_rate = 1000 * (Uint32)max_cooldown;
 		item_list[pos].cooldown_time = cur_time + 1000 * (Uint32)cooldown;
 	}
