@@ -14,7 +14,7 @@
 #include "init.h"
 #include "interface.h"
 #include "item_info.h"
-#if defined JSON_FILES
+#ifdef JSON_FILES
 #include "json_io.h"
 #endif
 #include "loginwin.h"
@@ -35,9 +35,6 @@
 int wanted_num_recipe_entries = 10;
 int disable_manuwin_keypress = 0;
 const int max_num_recipe_entries = 500;
-int manufacture_win= -1;
-int manufacture_menu_x=10;
-int manufacture_menu_y=20;
 static int pipeline_x = 0;
 static int last_changed_slot=-1;
 
@@ -61,14 +58,14 @@ static int recipes_loaded=0;
 static int mouse_over_recipe = -1;
 static int mouse_over_main_pos = -1;
 static int mouse_over_pipe_pos = -1;
-static char items_string[350]={0};
+static unsigned char items_string[350]={0};
 static size_t last_items_string_id = (size_t)-1;
 static item last_mix[NUM_MIX_SLOTS];
 static int recipe_names_changed = 0;
 static int initialised_recipe_names = 0;
 
 static int SLOT_SIZE = 0;
-static int PIPE_CONTROL_X = 0;
+static int pipe_control_width = 0;
 static int recipe_win_width = 0;
 static int manufacture_menu_x_len = 0;
 static int manufacture_menu_y_len = 0;
@@ -106,6 +103,7 @@ static void copy_recipe_items_to_items(item *items, size_t items_offset, recipe_
 	size_t j;
 	for (j=0; j<NUM_MIX_SLOTS; j++)
 	{
+		memset(&items[j+items_offset], 0, sizeof(item));
 		items[j+items_offset].id = recipe_items[j].id;
 		items[j+items_offset].image_id = recipe_items[j].image_id;
 		items[j+items_offset].quantity = recipe_items[j].quantity;
@@ -384,7 +382,7 @@ void load_recipes (){
 	off_t file_size;
 	const size_t recipe_size = sizeof(item)*NUM_MIX_SLOTS;
 	int num_recipes_in_file = -1;
-#if defined JSON_FILES
+#ifdef JSON_FILES
 	int have_json_file = 0;
 #endif
 
@@ -397,13 +395,19 @@ void load_recipes (){
 		return;
 	}
 
-#if defined JSON_FILES
-	/* try to use the json file first ... */
-	safe_snprintf(fname, sizeof(fname), "%srecipes_%s.json", get_path_config(), get_lowercase_username());
-	if ((num_recipes_in_file = json_open_recipes(fname)) >= 0)
-		have_json_file = 1;
-	/* ... then try the old binary file if we fail */
-	else
+#ifdef JSON_FILES
+	if (get_use_json_user_files())
+	{
+		USE_JSON_DEBUG("Loading json file");
+		/* try to use the json file first ... */
+		safe_snprintf(fname, sizeof(fname), "%srecipes_%s.json", get_path_config(), get_lowercase_username());
+		if ((num_recipes_in_file = json_open_recipes(fname)) >= 0)
+			have_json_file = 1;
+	}
+#endif
+
+#ifdef JSON_FILES
+	if (!have_json_file)
 	{
 #endif
 		safe_snprintf(fname, sizeof(fname), "recipes_%s.dat",get_lowercase_username());
@@ -411,7 +415,7 @@ void load_recipes (){
 		file_size = get_file_size_config(fname);
 		if ((file_size > 0) && (file_size % recipe_size == 0))
 			num_recipes_in_file = file_size / recipe_size - 1; // -1 as last is current in pipline
-#if defined JSON_FILES
+#ifdef JSON_FILES
 	}
 #endif
 
@@ -433,19 +437,17 @@ void load_recipes (){
 	recipes_loaded=1;
 	init_recipe_names();
 
-#if defined JSON_FILES
+#ifdef JSON_FILES
 	if (have_json_file)
 	{
-		cur_recipe = json_load_recipes(recipes_store, num_recipe_entries);
-		if ((cur_recipe >= 0) && (cur_recipe < num_recipe_entries))
-		{
-			memcpy(manu_recipe.items, recipes_store[cur_recipe].items, sizeof(recipe_item) * NUM_MIX_SLOTS);
-			fix_recipe_uids(manu_recipe.items);
-		}
-		else
+		cur_recipe = json_load_recipes(recipes_store, num_recipe_entries, manu_recipe.items);
+		if ((cur_recipe < 0) || (cur_recipe >= num_recipe_entries))
 			cur_recipe = 0;
 		return;
 	}
+
+	// if there is no json file, or json use disabled, try to load the old binary format
+	USE_JSON_DEBUG("Loading binary file");
 #endif
 
 	/* if the file exists but is not a valid size, don't use it */
@@ -510,26 +512,22 @@ void save_recipes(){
 	if (!recipes_loaded)
 		return;
 
-#if defined JSON_FILES
-	/* save in json format always */
-	safe_snprintf(fname, sizeof(fname), "%srecipes_%s.json",get_path_config(), get_lowercase_username());
-	if (json_save_recipes(fname, recipes_store, num_recipe_entries, cur_recipe) < 0)
+#ifdef JSON_FILES
+	if (get_use_json_user_files())
 	{
-		LOG_ERROR("%s: %s \"%s\"\n", reg_error_str, cant_open_file, fname);
+		USE_JSON_DEBUG("Saving json file");
+		/* save in json format always */
+		safe_snprintf(fname, sizeof(fname), "%srecipes_%s.json",get_path_config(), get_lowercase_username());
+		if (json_save_recipes(fname, recipes_store, num_recipe_entries, cur_recipe, manu_recipe.items) < 0)
+			LOG_ERROR("%s: %s \"%s\"\n", reg_error_str, cant_open_file, fname);
 		return;
 	}
-
-	/* Only save the old binary file if it already exist.  Doing both provides backward compatibility with older clients */
-	safe_snprintf(fname, sizeof(fname), "recipes_%s.dat",get_lowercase_username());
-	if (file_exists_config(fname)!=1)
-		return;
+	USE_JSON_DEBUG("Saving binary file");
 #endif
 
 	save_recipe_names();
 
-#if !defined JSON_FILES
 	safe_snprintf(fname, sizeof(fname), "recipes_%s.dat",get_lowercase_username());
-#endif
 	fp=open_file_config(fname,"wb");
 	if(fp == NULL){
 		LOG_ERROR("%s: %s \"%s\": %s\n", reg_error_str, cant_open_file, fname, strerror(errno));
@@ -642,30 +640,45 @@ static void use_recipe(int recipe_to_use)
 static void draw_recipe_controls(window_info *win){
 	int wpx=pipeline_x + SLOT_SIZE*NUM_MIX_SLOTS;
 	int wpy=manufacture_menu_y_len-recipe_y_offset;
-	int lpx=PIPE_CONTROL_X;
+	int lpx=pipe_control_width;
 	int lpy=SLOT_SIZE;
+	int half_plus_len = lpx/2-control_elem_size, half_plus_width = max2i(win->current_scale, 1);
+	int yp_ctr = wpy - control_elem_size + lpy/2;
+	int xp_ctr = wpx + control_elem_size + half_plus_len;
 
-	if (recipes_shown){
-	/* Up arrow */
-		glBegin(GL_QUADS);
+	if (recipes_shown)
+	{
+		/* Up arrow */
+		glBegin(GL_TRIANGLES);
 			glVertex3i(wpx+lpx/2, wpy+lpy-control_elem_size*2, 0); //top
 			glVertex3i(wpx+control_elem_size, wpy+lpy, 0); //left
 			glVertex3i(wpx+lpx-control_elem_size, wpy+lpy, 0); //right
-			glVertex3i(wpx+lpx/2, wpy+lpy-control_elem_size*2, 0); //top
 		glEnd();
-	} else {
+	}
+	else
+	{
 		/* Dn arrow */
-		glBegin(GL_QUADS);
-			glVertex3i(wpx+lpx/2, wpy+lpy, 0); //top
+		glBegin(GL_TRIANGLES);
+			glVertex3i(wpx+lpx/2, wpy+lpy, 0); //bottom
 			glVertex3i(wpx+control_elem_size, wpy+lpy-control_elem_size*2, 0); //left
 			glVertex3i(wpx+lpx-control_elem_size, wpy+lpy-control_elem_size*2, 0); //right
-			glVertex3i(wpx+lpx/2, wpy+lpy, 0); //top
 		glEnd();
 	}
 
 	/* Add btn */
-	glEnable(GL_TEXTURE_2D);
-	draw_string_zoomed(wpx+win->small_font_len_x/2, wpy-2, (unsigned char *)"+", 1, win->current_scale);
+	glBegin(GL_QUADS);
+
+	glVertex3i(xp_ctr-half_plus_len, yp_ctr-half_plus_width, 0);
+	glVertex3i(xp_ctr+half_plus_len, yp_ctr-half_plus_width, 0);
+	glVertex3i(xp_ctr+half_plus_len, yp_ctr+half_plus_width, 0);
+	glVertex3i(xp_ctr-half_plus_len, yp_ctr+half_plus_width, 0);
+
+	glVertex3i(xp_ctr-half_plus_width, yp_ctr-half_plus_len, 0);
+	glVertex3i(xp_ctr+half_plus_width, yp_ctr-half_plus_len, 0);
+	glVertex3i(xp_ctr+half_plus_width, yp_ctr+half_plus_len, 0);
+	glVertex3i(xp_ctr-half_plus_width, yp_ctr+half_plus_len, 0);
+
+	glEnd();
 }
 
 //draws a NUM_MIX_SLOTSx1 grid of items+grid
@@ -714,7 +727,7 @@ static int draw_production_pipe(window_info *win, int x, int y, int recipe_num){
 
 	//draw the grid, in red if selected
 	if (recipe_num==cur_recipe) glColor3f(1.0f,0.0f,0.0f);
-	else glColor3f(0.77f,0.57f,0.39f);
+	else glColor3fv(gui_color);
 	rendergrid(NUM_MIX_SLOTS,1,x, y, SLOT_SIZE, SLOT_SIZE);
 
 	glEnable(GL_TEXTURE_2D);
@@ -736,7 +749,7 @@ static int	display_manufacture_handler(window_info *win)
 			last_recipes_opaque = last_main_opaque = win->opaque = windows_list.window[recipe_win].opaque;
 	}
 
-	glColor3f(0.77f,0.57f,0.39f);
+	glColor3fv(gui_color);
 	glEnable(GL_TEXTURE_2D);
 
 	glColor3f(1.0f,1.0f,1.0f);
@@ -767,16 +780,19 @@ static int	display_manufacture_handler(window_info *win)
 	//now, draw the inventory text, if any.
 	if (last_items_string_id != inventory_item_string_id)
 	{
-		put_small_text_in_box_zoomed((unsigned char*)inventory_item_string, strlen(inventory_item_string), win->len_x-2*win->small_font_len_x, items_string, win->current_scale);
+		put_small_text_in_box_zoomed((unsigned char*)inventory_item_string,
+			strlen(inventory_item_string), win->len_x-2*win->small_font_max_len_x,
+			items_string, win->current_scale);
 		last_items_string_id = inventory_item_string_id;
 	}
-	draw_string_small_zoomed(win->small_font_len_x,manufacture_menu_y_len-text_y_offset,(unsigned char *)items_string,4, win->current_scale);
+	draw_string_small_zoomed(win->small_font_max_len_x, manufacture_menu_y_len-text_y_offset,
+		items_string, 4, win->current_scale);
 
 	// Render the grid *after* the images. It seems impossible to code
 	// it such that images are rendered exactly within the boxes on all
 	// cards
 	glDisable(GL_TEXTURE_2D);
-	glColor3f(0.77f,0.57f,0.39f);
+	glColor3fv(gui_color);
 
 	//draw the grid
 	rendergrid(GRID_COLS, GRID_ROWS, 0, 0, SLOT_SIZE, SLOT_SIZE);
@@ -1005,7 +1021,7 @@ static int recipe_controls_click_handler(window_info *win, int mx, int my, Uint3
 
 	int wpx=pipeline_x + SLOT_SIZE*NUM_MIX_SLOTS;
 	int wpy=manufacture_menu_y_len-recipe_y_offset;
-	int lpx=PIPE_CONTROL_X;
+	int lpx=pipe_control_width;
 	int lpy=SLOT_SIZE;
 
 	if (!recipes_loaded)
@@ -1027,8 +1043,9 @@ static int recipe_controls_click_handler(window_info *win, int mx, int my, Uint3
 
 		build_manufacture_list();
 		do_click_sound();
-	} else
-	if (mx>wpx+win->small_font_len_x/2 && mx<wpx+lpx-win->small_font_len_x/2 && my>wpy && my<wpy+win->small_font_len_y){
+	}
+	else if (mx>wpx && mx<wpx+lpx && my>wpy && my<wpy+lpy-control_elem_size*2)
+	{
 		//+ button
 		copy_items_to_recipe_items(recipes_store[cur_recipe].items, manufacture_list, MIX_SLOT_OFFSET);
 		clear_recipe_name(cur_recipe);
@@ -1043,7 +1060,6 @@ static int recipe_controls_click_handler(window_info *win, int mx, int my, Uint3
 static int click_manufacture_handler(window_info *win, int mx, int my, Uint32 flags)
 {
 	int pos;
-	Uint8 str[100];
 
 	int quantitytomove=1;
 
@@ -1051,18 +1067,20 @@ static int click_manufacture_handler(window_info *win, int mx, int my, Uint32 fl
 		quantitytomove = 10;
 
 	/* if the eye cursor is active and we right click, change to standard walk */
-	if(action_mode==ACTION_LOOK && (flags&ELW_RIGHT_MOUSE))
-		action_mode = ACTION_WALK;
+	if ((flags & ELW_RIGHT_MOUSE) && is_gamewin_look_action())
+	{
+		clear_gamewin_look_action();
+		return 0;
+	}
 
 	//see if we clicked on any item in the main category
 	pos=get_mouse_pos_in_grid(mx, my, GRID_COLS, GRID_ROWS, 0, 0, SLOT_SIZE, SLOT_SIZE);
 
 	if (pos >= 0 && manufacture_list[pos].quantity > 0)
 	{
-		if(action_mode==ACTION_LOOK || (flags&ELW_RIGHT_MOUSE)) {
-			str[0]=LOOK_AT_INVENTORY_ITEM;
-			str[1]=manufacture_list[pos].pos;
-			my_tcp_send(my_socket,str,2);
+		if ((flags & ELW_RIGHT_MOUSE) || is_gamewin_look_action()) {
+			Uint8 str[2] = { LOOK_AT_INVENTORY_ITEM, manufacture_list[pos].pos };
+			my_tcp_send(str, 2);
 			return 1;
 		} else	{
 			int j;
@@ -1115,10 +1133,9 @@ static int click_manufacture_handler(window_info *win, int mx, int my, Uint32 fl
 
 	if (pos >= 0 && manufacture_list[MIX_SLOT_OFFSET+pos].quantity > 0)
 	{
-		if(action_mode==ACTION_LOOK || (flags&ELW_RIGHT_MOUSE)){
-			str[0]=LOOK_AT_INVENTORY_ITEM;
-			str[1]=manufacture_list[MIX_SLOT_OFFSET+pos].pos;
-			my_tcp_send(my_socket,str,2);
+		if((flags & ELW_RIGHT_MOUSE) || is_gamewin_look_action()){
+			Uint8 str[2] = { LOOK_AT_INVENTORY_ITEM, manufacture_list[MIX_SLOT_OFFSET+pos].pos };
+			my_tcp_send(str, 2);
 			return 1;
 		} else {
 			int j;
@@ -1205,7 +1222,7 @@ int mix_handler(Uint8 quantity, const char* empty_error_str)
 	int i;
 	int cannot_manu = 1;
 
-	if(manufacture_win < 0)
+	if(get_id_MW(MW_MANU) < 0)
 		build_manufacture_list();
 
 	for(i=MIX_SLOT_OFFSET;i<MIX_SLOT_OFFSET+NUM_MIX_SLOTS;i++)
@@ -1234,7 +1251,7 @@ int mix_handler(Uint8 quantity, const char* empty_error_str)
 		//don't send an empty string
 		save_last_mix();
 		str[items_no*3+2]= quantity;
-		my_tcp_send(my_socket,str,items_no*3+3);
+		my_tcp_send(str,items_no*3+3);
 	}
 
 	return 1;
@@ -1277,14 +1294,15 @@ static int recipe_controls_mouseover_handler(window_info *win, int mx, int my, i
 {
 	int wpx=pipeline_x + SLOT_SIZE*NUM_MIX_SLOTS;
 	int wpy=win->len_y-recipe_y_offset;
-	int lpx=PIPE_CONTROL_X;
+	int lpx=pipe_control_width;
 	int lpy=SLOT_SIZE;
 
 	if (mx>wpx && mx<wpx+lpx && my>wpy+lpy-control_elem_size*2 && my<wpy+lpy){
 		//on arrow
 		show_help(recipe_show_hide_str, 0, win->len_y + 10 + win->small_font_len_y*(*help_line)++, win->current_scale);
-	} else
-	if (mx>wpx+win->small_font_len_x/2 && mx<wpx+lpx-win->small_font_len_x/2 && my>wpy && my<wpy+win->small_font_len_y){
+	}
+	else if (mx>wpx && mx<wpx+lpx && my>wpy && my<wpy+lpy-control_elem_size*2)
+	{
 		//on + button
 		show_help(recipe_save_str, 0, win->len_y + 10 + win->small_font_len_y*(*help_line)++, win->current_scale);
 	}
@@ -1348,11 +1366,10 @@ static int mouseover_manufacture_slot_handler(window_info *win, int mx, int my)
 		show_help(descp_str, 0, win->len_y + 10 + win->small_font_len_y*help_line++, win->current_scale);
 
 	/* if we're over an occupied slot and the eye cursor function is active, show the eye cursor */
-	if (check_for_eye){
-		if (action_mode == ACTION_LOOK){
-			elwin_mouse = CURSOR_EYE;
-			return 1;
-		}
+	if (check_for_eye && is_gamewin_look_action())
+	{
+		elwin_mouse = CURSOR_EYE;
+		return 1;
 	}
 
 	return 0;
@@ -1364,8 +1381,9 @@ static int mouseover_mixone_handler(widget_list *widget, int mx, int my)
 	if (show_help_text && widget)
 	{
 		window_info *win = &windows_list.window[widget->window_id];
-		int str_off = strlen(mix_str) * win->small_font_len_x / 2;
-		show_help(mix_str, (widget->pos_x + widget->len_x/2) - str_off, win->len_y+10, win->current_scale);
+		show_help_colored_scaled_centered((const unsigned char*)mix_str,
+			widget->pos_x + widget->len_x/2, win->len_y+10, 1.0f, 1.0f, 1.0f,
+			win->current_scale_small);
 	}
 	return 0;
 }
@@ -1376,8 +1394,9 @@ static int mouseover_mixall_handler(widget_list *widget, int mx, int my)
 	if (show_help_text && widget)
 	{
 		window_info *win = &windows_list.window[widget->window_id];
-		int str_off = strlen(mixall_str) * win->small_font_len_x / 2;
-		show_help(mixall_str, (widget->pos_x + widget->len_x/2) - str_off, win->len_y+10, win->current_scale);
+		show_help_colored_scaled_centered((const unsigned char*)mixall_str,
+			widget->pos_x + widget->len_x/2, win->len_y+10, 1.0f, 1.0f, 1.0f,
+			win->current_scale_small);
 	}
 	return 0;
 }
@@ -1491,13 +1510,16 @@ static void context_recipe_pre_show_handler(window_info *win, int widget_id, int
 //	Called when UI scaling changed
 static int ui_scale_manufacture_handler(window_info *win)
 {
-	int mbw = (int)(0.5 + 43 * win->current_scale);
-	int cbw = (int)(0.5 + 70 * win->current_scale);
+	int mbw = max2i((int)(0.5 + 43 * win->current_scale),
+		calc_button_width((const unsigned char*)">>", win->font_category, win->current_scale));
+	int cbw = max2i((int)(0.5 + 70 * win->current_scale),
+		calc_button_width((const unsigned char*)clear_str, win->font_category, win->current_scale));
 	int space = (int)(0.5 + 7 * win->current_scale);
-	int free_x = 0;
+	int free_x, pipe_width;
 
 	SLOT_SIZE = (int)(0.5 + 33 * win->current_scale);
-	PIPE_CONTROL_X = (int)(0.5 + 5 * win->current_scale) * 2 + win->small_font_len_x;
+	pipe_control_width = (int)(0.5 + 10 * win->current_scale) * 2;
+	pipe_width = SLOT_SIZE*NUM_MIX_SLOTS + pipe_control_width;
 	recipe_win_width = SLOT_SIZE * NUM_MIX_SLOTS + win->box_size + 2;
 	manufacture_menu_x_len = GRID_COLS * SLOT_SIZE + win->box_size;
 	manufacture_menu_y_len = (GRID_ROWS+1) * SLOT_SIZE + (4+1) * win->small_font_len_y;
@@ -1506,7 +1528,12 @@ static int ui_scale_manufacture_handler(window_info *win)
 	control_elem_size = (int)(0.5 + 5 * win->current_scale);
 
 	free_x = manufacture_menu_x_len - (cbw + 2*mbw + 3*space);
-	pipeline_x = space + cbw + free_x/2 - (SLOT_SIZE*NUM_MIX_SLOTS + PIPE_CONTROL_X)/2;
+	if (free_x < pipe_width + 2*space)
+	{
+		manufacture_menu_x_len += pipe_width +2*space - free_x;
+		free_x = pipe_width + 2*space;
+	}
+	pipeline_x = space + cbw + (free_x - pipe_width)/2;
 
 	resize_window(win->window_id, manufacture_menu_x_len, manufacture_menu_y_len);
 
@@ -1530,36 +1557,43 @@ static int ui_scale_manufacture_handler(window_info *win)
 	return 1;
 }
 
+static int change_manufacture_font_handler(window_info *win, font_cat cat)
+{
+	if (cat != win->font_category)
+		return 0;
+	ui_scale_manufacture_handler(win);
+	return 1;
+}
+
 void display_manufacture_menu()
 {
-	if(manufacture_win < 0){
-		int our_root_win = -1;
+	int manufacture_win = get_id_MW(MW_MANU);
 
-		if (!windows_on_top) {
-			our_root_win = game_root_win;
-		}
-		manufacture_win= create_window(win_manufacture, our_root_win, 0, manufacture_menu_x,
-			manufacture_menu_y, 0, 0, ELW_USE_UISCALE|ELW_WIN_DEFAULT);
-		set_window_custom_scale(manufacture_win, &custom_scale_factors.manufacture);
+	if(manufacture_win < 0){
+		manufacture_win = create_window(win_manufacture, (not_on_top_now(MW_MANU) ?game_root_win : -1), 0,
+			get_pos_x_MW(MW_MANU), get_pos_y_MW(MW_MANU), 0, 0, ELW_USE_UISCALE|ELW_WIN_DEFAULT);
+		set_id_MW(MW_MANU, manufacture_win);
+		set_window_custom_scale(manufacture_win, MW_MANU);
 
 		set_window_handler(manufacture_win, ELW_HANDLER_DISPLAY, &display_manufacture_handler );
 		set_window_handler(manufacture_win, ELW_HANDLER_CLICK, &click_manufacture_handler );
 		set_window_handler(manufacture_win, ELW_HANDLER_MOUSEOVER, &mouseover_manufacture_slot_handler );
 		set_window_handler(manufacture_win, ELW_HANDLER_KEYPRESS, (int (*)())&keypress_manufacture_handler );
 		set_window_handler(manufacture_win, ELW_HANDLER_UI_SCALE, &ui_scale_manufacture_handler );
+		set_window_handler(manufacture_win, ELW_HANDLER_FONT_CHANGE, &change_manufacture_font_handler);
 
 		mixone_button_id=button_add_extended(manufacture_win, mixone_button_id, NULL,
-			0, 0, 0, 0, 0, 1.0f, 0.77f, 0.57f, 0.39f, ">");
+			0, 0, 0, 0, 0, 1.0f, ">");
 		widget_set_OnClick(manufacture_win, mixone_button_id, mixone_handler);
 		widget_set_OnMouseover(manufacture_win, mixone_button_id, mouseover_mixone_handler);
 
 		mixall_button_id=button_add_extended(manufacture_win, mixall_button_id, NULL,
-			0, 0, 0, 0, 0, 1.0f, 0.77f, 0.57f, 0.39f, ">>");
+			0, 0, 0, 0, 0, 1.0f, ">>");
 		widget_set_OnClick(manufacture_win, mixall_button_id, mixall_handler);
 		widget_set_OnMouseover(manufacture_win, mixall_button_id, mouseover_mixall_handler);
 
 		clear_button_id=button_add_extended(manufacture_win, clear_button_id, NULL,
-			0, 0, 0, 0, 0, 1.0f, 0.77f, 0.57f, 0.39f, clear_str);
+			0, 0, 0, 0, 0, 1.0f, clear_str);
 		widget_set_OnClick(manufacture_win, clear_button_id, clear_handler);
 
 		if ((manufacture_win > -1) && (manufacture_win < windows_list.num_windows))
@@ -1571,7 +1605,7 @@ void display_manufacture_menu()
 		//Create a child window to show recipes in a dropdown panel
 		recipe_win= create_window("w_recipe", manufacture_win, 0, 0, 0, 0, 0,
 			ELW_USE_UISCALE|ELW_TITLE_NONE|ELW_SHOW|ELW_USE_BACKGROUND|ELW_ALPHA_BORDER|ELW_SWITCHABLE_OPAQUE|ELW_USE_BORDER|ELW_RESIZEABLE);
-		set_window_custom_scale(recipe_win, &custom_scale_factors.manufacture);
+		set_window_custom_scale(recipe_win, MW_MANU);
 		set_window_handler(recipe_win, ELW_HANDLER_DISPLAY, &recipe_dropdown_draw);
 		set_window_handler(recipe_win, ELW_HANDLER_CLICK, &recipe_dropdown_click_handler );
 		set_window_handler(recipe_win, ELW_HANDLER_MOUSEOVER, &mouseover_recipe_handler );
@@ -1579,10 +1613,11 @@ void display_manufacture_menu()
 		set_window_handler(recipe_win, ELW_HANDLER_KEYPRESS, (int (*)())&keypress_recipe_handler );
 
 		recipe_win_scroll_id = vscrollbar_add_extended(recipe_win, 1, NULL, 0,
-			0, 0, 0, 0, 1.0, 0.77f, 0.57f, 0.39f, 0, 1, num_recipe_entries-num_displayed_recipes);
+			0, 0, 0, 0, 1.0, 0, 1, num_recipe_entries-num_displayed_recipes);
 
 		if ((manufacture_win > -1) && (manufacture_win < windows_list.num_windows))
 			ui_scale_manufacture_handler(&windows_list.window[manufacture_win]);
+		check_proportional_move(MW_MANU);
 
 		// context menu
 		cm_recipewin = cm_create(cm_recipe_menu_str, context_recipe_handler);
