@@ -1,6 +1,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <math.h>
 #include "map.h"
 #include "2d_objects.h"
 #include "3d_objects.h"
@@ -43,13 +44,18 @@
 #include "mines.h"
 #include "highlight.h"
 
+#define MARK_CLIP_POS 20
+#define MARK_DIST 20
+
 int map_type=1;
 Uint32 map_flags=0;
 
 hash_table *server_marks=NULL;
 
+static int load_empty_map(void);
 
-void destroy_map()
+
+void destroy_map(void)
 {
 	int i;
 #ifdef EXTRA_DEBUG
@@ -121,7 +127,7 @@ int get_cur_map (const char * file_name)
 		}
 	}
 
-	return -1;	
+	return -1;
 }
 #endif
 
@@ -133,7 +139,7 @@ static void init_map_loading(const char *file_name)
 	 * Grum: the below is wrong: it never sets cur_map for the new map
 	 * when you're leaving an inside map. Perhaps it would be useful not
 	 * to set cur_map if you're *entering* an indide map, but we don't
-	 * know that at this point. 
+	 * know that at this point.
 	 *
 	 * I wonder why we souldn't want to set it anyway...
 	 */
@@ -143,12 +149,12 @@ static void init_map_loading(const char *file_name)
 	else cur_map=-1;
 	*/
 	cur_map = get_cur_map (file_name);
-	
+
 	create_loading_win(window_width, window_height, 1);
 	show_window(loading_win);
 }
 
-static __inline__ void build_path_map()
+static __inline__ void build_path_map(void)
 {
 	int i, x, y;
 
@@ -167,7 +173,7 @@ static __inline__ void build_path_map()
 	}
 }
 
-void updat_func(char *str, float percent)
+static void updat_func(char *str, float percent)
 {
 	update_loading_win(str, percent);
 }
@@ -179,7 +185,7 @@ static int el_load_map(const char * file_name)
 	init_map_loading(file_name);
 	ret = load_map(file_name, &updat_func);
 	if (!ret)
-		// don't try to build pathfinder maps etc. when loading 
+		// don't try to build pathfinder maps etc. when loading
 		// the map failed...
 		return ret;
 
@@ -201,7 +207,7 @@ static int el_load_map(const char * file_name)
 	}
 	build_path_map();
 	init_buffers();
-	
+
 	// reset light levels in case we enter or leave an inside map
 	new_minute();
 
@@ -241,18 +247,18 @@ void change_map (const char *mapname)
 		locked_to_console = 0;
 	}
 	load_map_marks();
-	
+
 #ifdef NEW_SOUND
 	get_map_playlist();
-	setup_map_sounds(get_cur_map(mapname));
+	setup_map_sounds(mapname);
 #endif // NEW_SOUND
 	have_a_map=1;
 	//also, stop the rain
 	weather_clear();
 
-	if ( get_show_window (map_root_win) )
+	if (get_show_window_MW(MW_TABMAP))
 	{
-		hide_window(map_root_win);
+		hide_window_MW(MW_TABMAP);
 		show_window(game_root_win);
 	}
 #else // !MAP_EDITOR2
@@ -271,7 +277,7 @@ void change_map (const char *mapname)
 
 #ifdef NEW_SOUND
 	get_map_playlist();
-	setup_map_sounds(get_cur_map(mapname));
+	setup_map_sounds(mapname);
 #endif // NEW_SOUND
 	have_a_map=1;
 #endif  //MAP_EDITOR2
@@ -282,24 +288,16 @@ void change_map (const char *mapname)
 #endif
 }
 
-int load_empty_map()
+static int load_empty_map(void)
 {
 	if (!el_load_map("./maps/nomap.elm"))
 	{
 #ifndef MAP_EDITOR2
 		locked_to_console = 1;
 		hide_window (game_root_win);
-		show_window (console_root_win);
-		LOG_TO_CONSOLE(c_red4, no_nomap_str);
+		show_window_MW(MW_CONSOLE);
 		LOG_ERROR(cant_change_map, "./maps/nomap.elm");
-		SDLNet_TCP_Close(my_socket);
-		disconnected = 1;
-#ifdef NEW_SOUND
-		stop_all_sounds();
-#endif // NEW_SOUND
-		disconnect_time = SDL_GetTicks();
-		SDLNet_Quit();
-		LOG_TO_CONSOLE(c_red3, disconnected_from_server);
+		force_server_disconnect(no_nomap_str);
 		//Fake a map to make sure we don't get any crashes.
 #endif
 		safe_snprintf(map_file_name, sizeof(map_file_name), "./maps/nomap.elm");
@@ -320,13 +318,15 @@ int load_empty_map()
 }
 
 
-void init_server_markers(){
+void init_server_markers(void)
+{
 	//init hash table
 	destroy_hash_table(server_marks);
 	server_marks= create_hash_table(50,hash_fn_int,cmp_fn_int,free);
 }
 
-void add_server_markers(){
+static void add_server_markers(void)
+{
 
 	hash_entry *he;
 	server_mark *sm;
@@ -337,7 +337,7 @@ void add_server_markers(){
 	for(i=0;i<max_mark;i++)
 		if(marks[i].server_side) break;
 	l=i;
-	
+
 	if(!server_marks) init_server_markers();
 	if(server_marks) {
 		hash_start_iterator(server_marks);
@@ -346,7 +346,7 @@ void add_server_markers(){
 			//is it in this map?
 			if(strcmp(mapname,sm->map_name)) continue;
 			//find the next slot. If not there, add 1
-			for(i=l;i<MAX_MARKINGS;i++) 
+			for(i=l;i<MAX_MARKINGS;i++)
 				if(marks[i].server_side||i>=max_mark) {l=i; if(l>=max_mark) max_mark=l+1; break;}
 			//add the marker
 			marks[l].x=sm->x;
@@ -363,7 +363,7 @@ void add_server_markers(){
 }
 
 void load_marks_to_buffer(char* mapname, marking* buffer, int* max)
-{ 
+{
 	FILE * fp = NULL;
 	char marks_file[256] = {0}, text[600] = {0};
 
@@ -381,7 +381,7 @@ void load_marks_to_buffer(char* mapname, marking* buffer, int* max)
 	//load user markers
 	while ( fgets(text, 600,fp) ) {
 		if (strlen (text) > 1) {
-			int r,g,b;			
+			int r,g,b;
 			sscanf (text, "%d %d", &buffer[*max].x, &buffer[*max].y);
 			//scanning mark color. It can be optional -> default=green
 			if(sscanf(text,"%*d %*d|%d,%d,%d|",&r,&g,&b)<3) { //NO SPACES in RGB format string!
@@ -405,15 +405,15 @@ void load_marks_to_buffer(char* mapname, marking* buffer, int* max)
 			}
 		}
 	}
-	
+
 	fclose(fp);
 
 	LOG_DEBUG("Read map markings from file '%s'", marks_file);
 
 }
 
-void load_map_marks()
-{ 
+void load_map_marks(void)
+{
 	//load user markers
 	load_marks_to_buffer(map_file_name, marks, &max_mark);
 
@@ -422,7 +422,7 @@ void load_map_marks()
 
 }
 
-void save_markings()
+void save_markings(void)
 {
 	FILE * fp;
 	char marks_file[256];
@@ -435,7 +435,7 @@ void save_markings()
 		LOG_ERROR("%s: %s \"%s\": %s\n", reg_error_str, cant_open_file, marks_file, strerror(errno));
 	} else {
 		for ( i = 0 ; i < max_mark ; i ++){
-			if ( marks[i].x > 0 && !marks[i].server_side){
+			if ( marks[i].x >= 0 && !marks[i].server_side){
 				fprintf(fp,"%d %d|%d,%d,%d| %s\n",marks[i].x,marks[i].y,marks[i].r,marks[i].g,marks[i].b,marks[i].text);
 			}
 		}
@@ -448,14 +448,15 @@ void save_markings()
 
 
 
-void load_server_markings(){
+void load_server_markings(void)
+{
 	char fname[128];
 	FILE *fp;
 	server_mark sm;
 	int rf;
 
 	init_server_markers();
-	
+
 	//open server markings file
 	safe_snprintf(fname, sizeof(fname), "servermarks_%s.dat",get_lowercase_username());
 
@@ -469,12 +470,12 @@ void load_server_markings(){
 		return;
 	}
 
-	while((rf=fscanf(fp,"%d %d %d %s %[^\n]s\n",&sm.id,&sm.x,&sm.y,sm.map_name,sm.text))==5){		
+	while((rf=fscanf(fp,"%d %d %d %s %[^\n]s\n",&sm.id,&sm.x,&sm.y,sm.map_name,sm.text))==5){
 		server_mark *nm = calloc(1,sizeof(server_mark));
 		memcpy(nm,&sm,sizeof(server_mark));
 		hash_add(server_marks,(void *)(uintptr_t)sm.id,(void*) nm);
 	}
-	
+
 	fclose (fp);
 
 	LOG_DEBUG("Read server markings from file '%s'", fname);
@@ -483,12 +484,13 @@ void load_server_markings(){
 }
 
 
-void save_server_markings(){
+void save_server_markings(void)
+{
 	char fname[128];
 	FILE *fp;
 	server_mark *sm;
 	hash_entry *he;
-	
+
 	if(!server_marks) return;
 
 	//open server markings file
@@ -500,24 +502,19 @@ void save_server_markings(){
 	}
 
 	hash_start_iterator(server_marks);
-	
-	while((he=hash_get_next(server_marks))){		
+
+	while((he=hash_get_next(server_marks))){
 		sm = (server_mark *) he->item;
 		fprintf(fp,"%d %d %d %s %s\n",sm->id, sm->x, sm->y, sm->map_name, sm->text);
 	}
-	
-	fclose (fp);	
+
+	fclose (fp);
 
 	LOG_DEBUG("Wrote server markings to file '%s'", fname);
 }
 
-//called in elconfig.c when turning markers on/off
-void change_3d_marks(int *rel){
-	*rel= !*rel;
-}
 
-
-void init_buffers()
+void init_buffers(void)
 {
 	int terrain_buffer_size;
 	int water_buffer_size;
@@ -533,11 +530,11 @@ void init_buffers()
 			cur_tile = tile_map[i*tile_map_size_x+j];
 			if (cur_tile != 255)
 			{
-				if (IS_WATER_TILE(cur_tile)) 
+				if (IS_WATER_TILE(cur_tile))
 				{
 					water_buffer_size++;
 				}
-				else 
+				else
 				{
 					terrain_buffer_size++;
 				}
@@ -549,7 +546,7 @@ void init_buffers()
 	init_reflection_portals(water_buffer_size);
 }
 
-void free_buffers()
+void free_buffers(void)
 {
 	if (water_tile_buffer)
 		free(water_tile_buffer);
@@ -569,13 +566,13 @@ int get_3d_objects_from_server (int nr_objs, const Uint8 *data, int len)
 	int name_len, max_name_len;
 	int id = -1;
 	int all_ok = 1;
-	
+
 	offset = 0;
 	nb_left = len;
 	for (iobj = 0; iobj < nr_objs; iobj++)
 	{
 		int obj_err = 0;
-		
+
 		if (nb_left < 14)
 		{
 			// Warn about this error!
@@ -583,7 +580,7 @@ int get_3d_objects_from_server (int nr_objs, const Uint8 *data, int len)
 			all_ok = 0;
                         break;
 		}
-		
+
 		obj_x = SDL_SwapLE16 (*((Uint16 *)(&data[offset])));
 		offset += 2;
 		obj_y = SDL_SwapLE16 (*((Uint16 *)(&data[offset])));
@@ -610,7 +607,7 @@ int get_3d_objects_from_server (int nr_objs, const Uint8 *data, int len)
 			y = 0.5f * obj_y + 0.25f;
 			z = get_tile_height(obj_x, obj_y);
 		}
-		
+
 		nb_left -= 12;
 		max_name_len = nb_left > sizeof (obj_name) ? sizeof (obj_name) : nb_left;
 		name_len = safe_snprintf (obj_name, max_name_len, "%s", &data[offset]);
@@ -621,21 +618,21 @@ int get_3d_objects_from_server (int nr_objs, const Uint8 *data, int len)
 			all_ok = 0;
                         break;
 		}
-		
+
 		offset += name_len + 1;
 		nb_left -= name_len + 1;
-		
+
 		if (!obj_err)
 			add_e3d_at_id (id, obj_name, x, y, z, rx, ry, rz, 0, 0, 1.0f, 1.0f, 1.0f, 1);
 		else
 			all_ok = 0;
 	}
-	
+
 	return all_ok;
 }
-	
+
 void remove_3d_object_from_server (int id)
-{	
+{
 	if (id < 0 || id > MAX_OBJ_3D)
 	{
 		LOG_ERROR ("Trying to remove object with invalid id %d", id);
@@ -656,21 +653,29 @@ void remove_3d_object_from_server (int id)
 #define ABS(a) ( ((a)<0)?(-(a)):(a)  )
 #define DST(xa,ya,xb,yb) ( MAX(ABS(xa-xb),ABS(ya-yb))  )
 int marks_3d=1;
-float mark_z_rot=0;
-
-void animate_map_markers(){
+int filter_marks_3d = 0;
+static float mark_z_rot=0;
 
+static void animate_map_markers(void)
+{
 	int dt;
 	static int last_rot=0;
 
 	dt=cur_time-last_rot;
 	last_rot+=dt;
 	mark_z_rot+=0.1*dt;
-	if(mark_z_rot>360) mark_z_rot-=360;
-
+	if (mark_z_rot > 360.0f)
+		mark_z_rot = fmodf(mark_z_rot, 360.0f);
 }
 
-void display_map_marks(){
+static __inline__ int filter_out(const char *mark_text)
+{
+	return (mark_filter_active && filter_marks_3d &&
+		!safe_strcasestr(mark_text, strlen(mark_text), mark_filter_text, strlen(mark_filter_text)));
+}
+
+static void display_map_marks(void)
+{
 	actor *me;
 	float x,y,z;
 	int i,ax,ay;
@@ -683,19 +688,21 @@ void display_map_marks(){
 	if(!me) return;
 	ax = me->x_pos;
 	ay = me->y_pos;
-	
+
 	glDisable(GL_TEXTURE_2D);
 	glDisable(GL_LIGHTING);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
-	glEnable(GL_ALPHA_TEST);	
+	glEnable(GL_ALPHA_TEST);
 
 	for(i=0;i<max_mark;i++){
+		if (filter_out(marks[i].text))
+			continue;
 		x=marks[i].x/2.0;
 		y=marks[i].y/2.0;
 		x += (TILESIZE_X / 2);
 		y += (TILESIZE_Y / 2);
-		if(DST(ax,ay,x,y)>MARK_DIST||marks[i].x<0||!marks_3d) continue;
+		if(DST(ax,ay,x,y)>MARK_DIST||marks[i].x<0) continue;
 		z = get_tile_height(marks[i].x, marks[i].y);
 		for(j=z-fr/5,ff=1;j<z+2;j+=0.1,ff=(2-(j-z))/2) {
 			if(marks[i].server_side) glColor4f(0.0f, 0.0f, 1.0f, 0.9f-(j-z)/3);
@@ -707,17 +714,17 @@ void display_map_marks(){
 				glVertex3f(x+dx*ff,y-dy*ff,j);
 			glEnd();
 		}
-		
+
 	}
-	
+
 	glDisable(GL_ALPHA_TEST);
 	//glEnable(GL_LIGHTING);
 	glDisable(GL_BLEND);
 	glEnable(GL_TEXTURE_2D);
-	
 }
 
-void display_map_markers() {
+static void display_map_markers(void)
+{
 	int ax, ay;
 	float z,x,y;
 	int i;
@@ -727,7 +734,7 @@ void display_map_markers() {
 	float banner_width;
 	float font_scale = 1.0f/ALT_INGAME_FONT_X_LEN;
 	float font_size_x=font_scale*SMALL_INGAME_FONT_X_LEN;
-	float font_size_y=font_scale*SMALL_INGAME_FONT_Y_LEN;
+	float font_size_y=font_scale*SMALL_INGAME_FONT_X_LEN;
 	char tmpb[4];
 	actor *me;
 
@@ -735,7 +742,7 @@ void display_map_markers() {
 	if(!me) return;
 	ax = me->x_pos;
 	ay = me->y_pos;
-	
+
 	glGetDoublev(GL_MODELVIEW_MATRIX, model);
 	glGetDoublev(GL_PROJECTION_MATRIX, proj);
 	glGetIntegerv(GL_VIEWPORT, view);
@@ -751,24 +758,27 @@ void display_map_markers() {
 	glDisable(GL_LIGHTING);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
-	
+
 	for(i=0;i<max_mark;i++){
+		if (filter_out(marks[i].text))
+			continue;
 		x=marks[i].x/2.0;
 		y=marks[i].y/2.0;
 		x += (TILESIZE_X / 2);
 		y += (TILESIZE_Y / 2);
-		if(DST(ax,ay,x,y)>MARK_DIST||marks[i].x<0||!marks_3d) continue;
+		if(DST(ax,ay,x,y)>MARK_DIST||marks[i].x<0) continue;
 		z = get_tile_height(marks[i].x, marks[i].y)+2.3;
 		gluProject(x, y, z, model, proj, view, &hx, &hy, &hz);
 		//shorten text
 		memcpy(tmpb,marks[i].text+MARK_CLIP_POS,4);
 		marks[i].text[MARK_CLIP_POS]=marks[i].text[MARK_CLIP_POS+1]=marks[i].text[MARK_CLIP_POS+2]='.';
 		marks[i].text[MARK_CLIP_POS+3]=0;
-		banner_width = ((float)get_string_width((unsigned char*)marks[i].text)*(font_size_x*name_zoom))/2.0;
-		draw_ortho_ingame_string(hx-banner_width, hy, hz, (unsigned char*)marks[i].text, 4, font_size_x, font_size_y);
+		banner_width = 0.5 * (float)get_string_width_zoom((unsigned char*)marks[i].text, NAME_FONT, font_size_x);
+		draw_ortho_ingame_string(hx-banner_width, hy, hz, (unsigned char*)marks[i].text,
+			4, NAME_FONT, font_size_x, font_size_y);
 		//restore text
 		memcpy(marks[i].text+MARK_CLIP_POS,tmpb,4);
-			
+
 	}
 	glDisable(GL_BLEND);
 	glDisable(GL_TEXTURE_2D);
@@ -778,9 +788,12 @@ void display_map_markers() {
 	glPopMatrix();
 	//glEnable(GL_LIGHTING);
 	glDepthFunc(GL_LESS);
-	
-	
 }
 
 
-
+void draw_3d_marks(void)
+{
+	animate_map_markers();
+	display_map_markers(); //draw text
+	display_map_marks(); //draw cross
+}
