@@ -458,7 +458,7 @@ int	draw_window_title(window_info *win)
 	if((win->flags&ELW_TITLE_BAR) == ELW_TITLE_NONE)	return 0;
 
 	glColor3f(1.0f,1.0f,1.0f);
-	//ok, now draw that shit...
+	//ok, now draw it ...
 
 	bind_texture(icons_text);
 	glEnable(GL_ALPHA_TEST);
@@ -543,6 +543,7 @@ int	draw_window_border(window_info *win)
 			glVertex3i(0, win->len_y, 0);
 			glVertex3i(0, win->len_y, 0);
 			glVertex3i(0, 0, 0);
+			glEnd();
 		}
 
 	if(win->flags&ELW_CLOSE_BOX)
