@@ -52,6 +52,7 @@ int begin_lost_missiles = -1;
 int end_lost_missiles = -1;
 
 int range_total_shots = 0;
+static int range_exp_at_shot = 0;
 int range_success_hits = 0;
 int range_critical_hits = 0;
 
@@ -124,7 +125,7 @@ int missiles_add(int type,
 	missile_type *mis_type = &missiles_defs[type];
 	float direction[3];
 	float dist;
-        
+
 	if (missiles_count >= MAX_MISSILES) {
 		LOG_ERROR("too many missiles, can't add the last one!");
 		return MAX_MISSILES;
@@ -132,7 +133,7 @@ int missiles_add(int type,
 
 	missiles_log_message("missiles_add: origin=(%.2f,%.2f,%.2f), target=(%.2f,%.2f,%.2f) type %u",
 						 origin[0], origin[1], origin[2], target[0], target[1], target[2], shot_type);
-        
+
 	direction[0] = target[0] - origin[0];
 	direction[1] = target[1] - origin[1];
 	direction[2] = target[2] - origin[2];
@@ -149,7 +150,7 @@ int missiles_add(int type,
     else {
         missiles_log_message("missiles_add: distance of the shot: %f", dist);
     }
-        
+
 	mis = &missiles_list[missiles_count++];
 
 	mis->type = type;
@@ -164,7 +165,7 @@ int missiles_add(int type,
 	mis->trace_length = mis_type->trace_length;
 	mis->covered_distance = 0;
 	mis->remaining_distance += shift;
-        
+
 	if (use_eye_candy == 1)
 	{
 		ec_create_missile_effect(missiles_count-1, (poor_man ? 6 : 10), shot_type);
@@ -213,19 +214,19 @@ void missiles_remove(int missile_id)
 		//mis->position[1] -= mis->direction[1] * dist;
 		//mis->position[2] -= mis->direction[2] * dist;
 		missiles_log_message("adding a lost missile at (%f,%f,%f) with rotation (%f,%f,%f)",
-							 mis->position[0] - mis->direction[0] * dist, 
-							 mis->position[1] - mis->direction[1] * dist, 
+							 mis->position[0] - mis->direction[0] * dist,
+							 mis->position[1] - mis->direction[1] * dist,
 							 mis->position[2] - mis->direction[2] * dist,
                              x_rot, y_rot, z_rot);
 		obj_3d_id = add_e3d(missiles_defs[mis->type].lost_mesh,
-							mis->position[0] - mis->direction[0] * dist, 
-							mis->position[1] - mis->direction[1] * dist, 
+							mis->position[0] - mis->direction[0] * dist,
+							mis->position[1] - mis->direction[1] * dist,
 							mis->position[2] - mis->direction[2] * dist,
 							x_rot, y_rot, z_rot, 0, 0, 1.0, 1.0, 1.0, 1);
 		if (obj_3d_id >= 0)
 			missiles_add_lost(obj_3d_id);
 	}
-        
+
 	ec_remove_missile(missile_id);
 
 	--missiles_count;
@@ -242,7 +243,7 @@ void missiles_update()
 	int i;
 	static int last_update = 0;
 	float time_diff = (cur_time - last_update) / 1000.0;
-        
+
 	for (i = 0; i < missiles_count; ) {
 		missile *mis = &missiles_list[i];
 		float dist = mis->speed * time_diff;
@@ -449,7 +450,7 @@ int missiles_fire_arrow(actor *a, float target[3], MissileShotType shot_type)
 	float shift[3] = {0.0, get_actor_scale(a), 0.0};
 	missile_type *mis_type;
 	int mis_type_id;
-        
+
 	mis_type_id = actors_defs[a->actor_type].shield[a->cur_shield].missile_type;
 
 	if (mis_type_id < 0 || mis_type_id >= MAX_MISSILES_DEFS) {
@@ -462,13 +463,14 @@ int missiles_fire_arrow(actor *a, float target[3], MissileShotType shot_type)
 	shift[1] *= mis_type->length;
 
 	cal_get_actor_bone_absolute_position(a, get_actor_bone_id(a, arrow_bone), shift, origin);
-        
+
 /* 	if (shot_type != MISSED_SHOT) */
 	mis_id = missiles_add(mis_type_id, origin, target, 0.0, shot_type);
-	
+
 	if(a->actor_id == yourself)
 	{
 		range_total_shots++;
+		range_exp_at_shot=get_session_exp_ranging();
 		if (shot_type == MISSED_SHOT)
 		{
 			add_floating_message(a->actor_id, "miss", FLOATINGMESSAGE_NORTH, 1.0, 0.55, 0.0, 1250);
@@ -485,7 +487,7 @@ int missiles_fire_arrow(actor *a, float target[3], MissileShotType shot_type)
 	}
 	/* 	else */
 /* 		mis_id = missiles_add(a->cur_shield, origin, target, arrow_speed*2.0/3.0, arrow_trace_length*2.0/3.0, 0.0, shot_type); */
-        
+
 	return mis_id;
 }
 
@@ -503,7 +505,7 @@ void missiles_rotate_actor_bones(actor *a)
 		return;
 
 	skel = CalModel_GetSkeleton(a->calmodel);
-        
+
 	if (a->cal_rotation_blend < 1.0) {
 		a->cal_rotation_blend += a->cal_rotation_speed*(cur_time-a->cal_last_rotation_time);
 
@@ -663,14 +665,14 @@ void missiles_aim_at_b(int actor1_id, int actor2_id)
 		UNLOCK_ACTORS_LISTS();
 		return;
 	}
-	
+
 	missiles_log_message("%s (%d): cleaning the queue from missiles_aim_at_b",
 						 act1->actor_name, actor1_id);
 	missiles_clean_range_actions_queue(act1);
 
 	if (act1->range_actions_count < MAX_RANGE_ACTION_QUEUE)	{
 		range_action *action = &act1->range_actions[act1->range_actions_count];
-	
+
 		missiles_log_message("%s (%d): will aim at actor %d (time=%d)", act1->actor_name, actor1_id, actor2_id, cur_time);
 
 		cal_get_actor_bone_absolute_position(act2, get_actor_bone_id(act2, body_top_bone), NULL, action->aim_position);
@@ -742,7 +744,7 @@ void missiles_fire_a_to_b(int actor1_id, int actor2_id)
 	LOCK_ACTORS_LISTS();
 	act1 = get_actor_ptr_from_id(actor1_id);
 	act2 = get_actor_ptr_from_id(actor2_id);
-        
+
 	if (!act1) {
 		LOG_ERROR("missiles_fire_a_to_b: the actor %d does not exists!", actor1_id);
 		UNLOCK_ACTORS_LISTS();
@@ -801,7 +803,7 @@ void missiles_fire_a_to_xyz(int actor_id, float *target)
 		range_action *action = &act->range_actions[act->range_actions_count-1];
 
 		missiles_log_message("%s (%d): will fire to target %f,%f,%f", act->actor_name, actor_id, target[0], target[1], target[2]);
-		
+
 		memcpy(action->fire_position, target, sizeof(float) * 3);
 		missiles_test_target_validity(action->fire_position, "missiles_fire_a_to_xyz");
 		action->fire_actor = -1;
@@ -971,84 +973,89 @@ void missiles_init_defs()
 /*! Ranging win */
 /**********************************************/
 
-int range_win = -1;
-int range_win_x_len = 10;
-int range_win_y_len = 20;
-int ranging_win_x = 10;
-int ranging_win_y = 20;
+static int range_win_x_len = 10;
+static int range_win_y_len = 20;
 static int print_to_console = 0;
+static int result_x = 0;
 
 int display_range_handler(window_info *win)
 {
 	char str[50];
 	const int margin = 5 * win->current_scale;
-	const int step_y = win->small_font_len_y;
+	const int step_y = get_line_height(win->font_category, win->current_scale_small);
 	const int pos_x = margin;
 	int pos_y = margin;
-	int max_width = 0;
-	int len_x = 0;
-	int len_y = 0;
-	int resize = 0;
 
-	if (print_to_console) LOG_TO_CONSOLE(c_green2, ranging_win_title_str);
+	if (print_to_console)
+	{
+		int len, max_len = strlen(ranging_total_shots_str);
+		if ((len = strlen(ranging_sucessful_shots_str)) > max_len) max_len = len;
+		if ((len = strlen(ranging_missed_shots_str)) > max_len) max_len = len;
+		if ((len = strlen(ranging_success_rate_str)) > max_len) max_len = len;
+		if ((len = strlen(ranging_critical_rate_str)) > max_len) max_len = len;
+		if ((len = strlen(ranging_exp_per_arrow_str)) > max_len) max_len = len;
 
-	safe_snprintf(str, sizeof(str), ranging_total_shots_str, range_total_shots);
-	if (print_to_console) LOG_TO_CONSOLE(c_green1, str);
-	if (max_width < strlen(str)) max_width = strlen(str);
-	draw_string_small_zoomed(pos_x, pos_y, (unsigned char*)str,2, win->current_scale);
+		LOG_TO_CONSOLE(c_green2, ranging_win_title_str);
+		safe_snprintf(str, sizeof(str), "%-*s %d", max_len, ranging_total_shots_str,
+			range_total_shots);
+		LOG_TO_CONSOLE(c_green1, str);
+		safe_snprintf(str, sizeof(str), "%-*s %d", max_len, ranging_sucessful_shots_str,
+			range_success_hits);
+		LOG_TO_CONSOLE(c_green1, str);
+		safe_snprintf(str, sizeof(str), "%-*s %d", max_len, ranging_missed_shots_str,
+			range_total_shots - range_success_hits);
+		LOG_TO_CONSOLE(c_green1, str);
+		safe_snprintf(str, sizeof(str), "%-*s %.2f %%", max_len, ranging_success_rate_str,
+			range_success_hits > 0 ? (float)range_success_hits/range_total_shots*100 : 0.0f);
+		LOG_TO_CONSOLE(c_green1, str);
+		safe_snprintf(str, sizeof(str), "%-*s %.2f %%", max_len, ranging_critical_rate_str,
+			range_critical_hits > 0 ? (float)range_critical_hits/range_success_hits*100 : 0.0f);
+		LOG_TO_CONSOLE(c_green1, str);
+		safe_snprintf(str, sizeof(str), "%-*s %.2f exp", max_len, ranging_exp_per_arrow_str,
+			range_total_shots > 0 ? (float)range_exp_at_shot/range_total_shots : 0.0f);
+		LOG_TO_CONSOLE(c_green1, str);
+
+		print_to_console = 0;
+	}
+
+	draw_string_small_zoomed(pos_x, pos_y, (const unsigned char*)ranging_total_shots_str,
+		1, win->current_scale);
+	safe_snprintf(str, sizeof(str), "%d", range_total_shots);
+	draw_string_small_zoomed(result_x, pos_y, (const unsigned char*)str, 1, win->current_scale);
 	pos_y += step_y;
 
-	safe_snprintf(str, sizeof(str), ranging_sucessful_shots_str, range_success_hits);
-	if (print_to_console) LOG_TO_CONSOLE(c_green1, str);
-	if (max_width < strlen(str)) max_width = strlen(str);
-	draw_string_small_zoomed(pos_x, pos_y, (unsigned char*)str,2, win->current_scale);
+	draw_string_small_zoomed(pos_x, pos_y, (const unsigned char*)ranging_sucessful_shots_str,
+		1, win->current_scale);
+	safe_snprintf(str, sizeof(str), "%d", range_success_hits);
+	draw_string_small_zoomed(result_x, pos_y, (const unsigned char*)str, 1, win->current_scale);
 	pos_y += step_y;
 
-	safe_snprintf(str, sizeof(str), ranging_missed_shots_str, range_total_shots - range_success_hits);
-	if (print_to_console) LOG_TO_CONSOLE(c_green1, str);
-	if (max_width < strlen(str)) max_width = strlen(str);
-	draw_string_small_zoomed(pos_x, pos_y, (unsigned char*)str,2, win->current_scale);
+	draw_string_small_zoomed(pos_x, pos_y, (const unsigned char*)ranging_missed_shots_str,
+		1, win->current_scale);
+	safe_snprintf(str, sizeof(str), "%d", range_total_shots - range_success_hits);
+	draw_string_small_zoomed(result_x, pos_y, (const unsigned char*)str, 1, win->current_scale);
 	pos_y += 2*step_y;
 
-	if(range_success_hits > 0)
-		safe_snprintf(str, sizeof(str), ranging_success_rate_str, ((double)range_success_hits/(double)range_total_shots)*100.0);
-	else
-		safe_snprintf(str, sizeof(str), ranging_success_rate_str, 0.0);
-	if (print_to_console) LOG_TO_CONSOLE(c_green1, str);
-	if (max_width < strlen(str)) max_width = strlen(str);
-	draw_string_small_zoomed(pos_x, pos_y, (unsigned char*)str,2, win->current_scale);
+	draw_string_small_zoomed(pos_x, pos_y, (const unsigned char*)ranging_success_rate_str,
+		1, win->current_scale);
+	safe_snprintf(str, sizeof(str), "%.2f %%",
+		range_success_hits > 0 ? (float)range_success_hits/range_total_shots*100 : 0.0f);
+	draw_string_small_zoomed(result_x, pos_y, (const unsigned char*)str, 1, win->current_scale);
 	pos_y += step_y;
 
-	if(range_critical_hits > 0)
-		safe_snprintf(str, sizeof(str), ranging_critical_rate_str, ((double)range_critical_hits/(double)range_success_hits)*100.0);
-	else
-		safe_snprintf(str, sizeof(str), ranging_critical_rate_str, 0.0);
-	if (print_to_console) LOG_TO_CONSOLE(c_green1, str);
-	if (max_width < strlen(str)) max_width = strlen(str);
-	draw_string_small_zoomed(pos_x, pos_y, (unsigned char*)str,2, win->current_scale);
+	draw_string_small_zoomed(pos_x, pos_y, (const unsigned char*)ranging_critical_rate_str,
+		1, win->current_scale);
+	safe_snprintf(str, sizeof(str), "%.2f %%",
+		range_critical_hits > 0 ? (float)range_critical_hits/range_success_hits*100 : 0.0f);
+	draw_string_small_zoomed(result_x, pos_y, (const unsigned char*)str, 1, win->current_scale);
 	pos_y += 2*step_y;
 
-	if(range_total_shots > 0)
-		safe_snprintf(str, sizeof(str), ranging_exp_per_arrow_str, get_session_exp_ranging()/(double)range_total_shots);
-	else
-		safe_snprintf(str, sizeof(str), ranging_exp_per_arrow_str, 0.0);
-	if (print_to_console) LOG_TO_CONSOLE(c_green1, str);
-	if (max_width < strlen(str)) max_width = strlen(str);
-	draw_string_small_zoomed(pos_x, pos_y, (unsigned char*)str,2, win->current_scale);
-	pos_y += step_y;
-
-	len_x = (max_width * win->small_font_len_x) + 2*margin + win->box_size;
-	if (len_x != win->len_x)
-		resize = 1;
-
-	len_y = pos_y + margin;
-	if (len_y != win->len_y)
-		resize = 1;
-
-	if (resize)
-		resize_window(win->window_id, len_x, len_y);
-
-	print_to_console = 0;
+	// % .2f exp
+	draw_string_small_zoomed(pos_x, pos_y, (const unsigned char*)ranging_exp_per_arrow_str,
+		2, win->current_scale);
+	safe_snprintf(str, sizeof(str), "%.2f exp",
+		range_total_shots > 0 ? (float)range_exp_at_shot/range_total_shots : 0.0f);
+	draw_string_small_zoomed(result_x, pos_y, (const unsigned char*)str, 1, win->current_scale);
 
 	return 1;
 }
@@ -1062,19 +1069,69 @@ static int cm_ranging_handler(window_info *win, int widget_id, int mx, int my, i
 	return 1;
 }
 
+static int set_range_window_size(window_info *win)
+{
+	static const char* labels[6] = {
+		ranging_total_shots_str,
+		ranging_sucessful_shots_str,
+		ranging_missed_shots_str,
+		ranging_success_rate_str,
+		ranging_critical_rate_str,
+		ranging_exp_per_arrow_str,
+	};
+
+	float zoom = win->current_scale_small;
+	int lbl_max_width = 0;
+	int exp_max_width = get_string_width_zoom((const unsigned char*)"100.00 exp",
+		win->font_category, zoom);
+	int sep_width = 2 * get_char_width_zoom('M', win->font_category, zoom);
+	int margin = 5 * win->current_scale;
+	int line_height = get_line_height(win->font_category, zoom);
+	int len_x, len_y;
+	int i;
+
+	for (i = 0; i < 6; ++i)
+	{
+		int lbl_width = get_string_width_zoom((const unsigned char*)labels[i],
+			win->font_category, zoom);
+		if (lbl_width > lbl_max_width)
+			lbl_max_width = lbl_width;
+	}
+
+	result_x = margin + lbl_max_width + sep_width;
+
+	len_x = lbl_max_width + sep_width + exp_max_width + 2 * margin + win->box_size;
+	len_y = 8 * line_height + 2 * margin;
+	resize_window(win->window_id, len_x, len_y);
+
+	return 1;
+}
+
+int change_range_font_handler(window_info* win, font_cat cat)
+{
+	if (cat != UI_FONT)
+		return 0;
+	set_range_window_size(win);
+	return 1;
+}
+
 void display_range_win(void)
 {
+	int range_win = get_id_MW(MW_RANGING);
 	if(range_win < 0){
-		int our_root_win = -1;
-			
-		if (!windows_on_top) {
-			our_root_win = game_root_win;
-		}
-		range_win = create_window(ranging_win_title_str, our_root_win, 0, ranging_win_x, ranging_win_y, range_win_x_len, range_win_y_len, ELW_USE_UISCALE|ELW_WIN_DEFAULT);
-		set_window_custom_scale(range_win, &custom_scale_factors.ranging);
-		set_window_handler(range_win, ELW_HANDLER_DISPLAY, &display_range_handler );
+		range_win = create_window(ranging_win_title_str, (not_on_top_now(MW_RANGING) ?game_root_win : -1), 0, get_pos_x_MW(MW_RANGING), get_pos_y_MW(MW_RANGING),
+			range_win_x_len, range_win_y_len, ELW_USE_UISCALE|ELW_WIN_DEFAULT);
+		set_id_MW(MW_RANGING, range_win);
+		set_window_custom_scale(range_win, MW_RANGING);
+		set_window_handler(range_win, ELW_HANDLER_DISPLAY, &display_range_handler);
+		set_window_handler(range_win, ELW_HANDLER_UI_SCALE, &set_range_window_size);
+		set_window_handler(range_win, ELW_HANDLER_FONT_CHANGE, &change_range_font_handler);
 		cm_add(windows_list.window[range_win].cm_id, cm_ranging_menu_str, cm_ranging_handler);
-	} else {
+		set_range_window_size(&windows_list.window[range_win]);
+		check_proportional_move(MW_RANGING);
+	}
+	else
+	{
 		show_window(range_win);
 		select_window(range_win);
 	}
