@@ -3,6 +3,7 @@
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
+#include <limits.h>
 #include "new_actors.h"
 #include "actor_scripts.h"
 #include "asc.h"
@@ -58,7 +59,7 @@ void build_glow_color_table()
 }
 
 //return the ID (number in the actors_list[]) of the new allocated actor
-int add_enhanced_actor(enhanced_actor *this_actor, float x_pos, float y_pos,
+static int add_enhanced_actor(enhanced_actor *this_actor, float x_pos, float y_pos,
 	float z_pos, float z_rot, float scale, int actor_id, const char* name)
 {
 	int texture_id;
@@ -78,7 +79,7 @@ int add_enhanced_actor(enhanced_actor *this_actor, float x_pos, float y_pos,
 
 	our_actor = calloc(1, sizeof(actor));
 
-	memset(our_actor->current_displayed_text, 0, MAX_CURRENT_DISPLAYED_TEXT_LEN);
+	memset(our_actor->current_displayed_text, 0, sizeof(our_actor->current_displayed_text));
 	our_actor->current_displayed_text_time_left =  0;
 
 	our_actor->texture_id=texture_id;
@@ -198,7 +199,7 @@ Uint32 delay_texture_item_change(actor* a, const int which_part, const int which
 			a->delayed_item_changes[a->delayed_item_changes_count] = which_id;
 			a->delayed_item_type_changes[a->delayed_item_changes_count] = which_part;
 			a->delayed_item_changes_count++;
-	
+
 			return 1;
 		}
 	}
@@ -220,7 +221,9 @@ void unwear_item_from_actor(int actor_id,Uint8 which_part)
 							{
 								ec_remove_weapon(actors_list[i]);
 								if(actors_list[i]->cur_weapon == GLOVE_FUR || actors_list[i]->cur_weapon == GLOVE_LEATHER){
-									my_strcp(actors_list[i]->body_parts->hands_tex, actors_list[i]->body_parts->hands_tex_save);
+									safe_strncpy(actors_list[i]->body_parts->hands_tex,
+										actors_list[i]->body_parts->hands_tex_save,
+										sizeof(actors_list[i]->body_parts->hands_tex));
 								}
 								if (delay_texture_item_change(actors_list[i], which_part, -1))
 								{
@@ -302,14 +305,14 @@ void custom_path(char * path, char * custom1, char * custom2) {
 	/* Check if custom1 has path readable */
 	safe_snprintf(buffer, sizeof(buffer), "%s%s", custom1, path);
 	if (el_custom_file_exists(buffer)) {
-		my_strcp(path, buffer);
+		strcpy(path, buffer);
 		return;
 	}
 
 	/* Check if custom2 has path readable */
 	safe_snprintf(buffer, sizeof(buffer), "%s%s", custom2, path);
 	if (el_custom_file_exists(buffer)) {
-		my_strcp(path, buffer);
+		strcpy(path, buffer);
 		return;
 	}
 
@@ -318,43 +321,59 @@ void custom_path(char * path, char * custom1, char * custom2) {
 }
 #endif  //CUSTOM_LOOK
 
+/*
+ * Common function to return only the player name in the specified buffer, stripped of possible leading colour
+ * codes and terminated at the end of the name, before any guild tag.
+ * The function returns index in name_buf of the character immediately after the name.
+ */ 
+static size_t get_onlyname(const char *name_buf, size_t name_buf_size, char *onlyname_buf, size_t onlyname_buf_size)
+{
+	size_t i, j;
+
+	/* step past any leading colour codes */
+	for (i = 0; (name_buf[i] != '\0') && (i < name_buf_size) && is_color(name_buf[i]); i++) /* do nothing */;
+
+	/* copy characters, terminate at the end of the name or the colour code before the guild tag */
+	for (j = 0; (name_buf[i] != '\0') && (i < name_buf_size) && is_printable(name_buf[i]) && (j < (onlyname_buf_size - 1)); i++, j++)
+		onlyname_buf[j] = name_buf[i];
+
+	/* if the name index is now on the colour code after the space bewtween name and guild tag, move back to the space */
+	if ((name_buf[i] != '\0') && !is_printable(name_buf[i]) && (i > 0) && (j > 0) && (name_buf[i-1] == ' '))
+	{
+		i--;
+		j--;
+	}
+	onlyname_buf[j] = '\0';
+	my_tolower(onlyname_buf);
+
+	return i;
+}
+
 void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 {
-	int i;
-#ifdef CUSTOM_LOOK
-	char playerpath[256], guildpath[256], onlyname[32]={0};
-	int j;
-#endif
+	size_t i;
 
-	
 	for(i=0;i<max_actors;i++)
 		{
 			if(actors_list[i])
 				if(actors_list[i]->actor_id==actor_id)
 					{
 #ifdef CUSTOM_LOOK
+						char playerpath[256], guildpath[256], onlyname[32]={0};
+						get_onlyname(actors_list[i]->actor_name, sizeof(actors_list[i]->actor_name), onlyname, sizeof(onlyname));
 						safe_snprintf(guildpath, sizeof(guildpath), "custom/guild/%d/", actors_list[i]->body_parts->guild_id);
-						for(j=0;j<30;j++){
-                            if(actors_list[i]->actor_name[j]==' ' || actors_list[i]->actor_name[j]>125){
-								j=31;
-							}
-							else if(actors_list[i]->actor_name[0]>'z'){
-								onlyname[j]=actors_list[i]->actor_name[j+1];
-							}
-							else
-							{
-								onlyname[j]=actors_list[i]->actor_name[j];
-							}
-						}
-						my_tolower(onlyname);
 						safe_snprintf(playerpath, sizeof(playerpath), "custom/player/%s/", onlyname);
 #endif
 						if (which_part==KIND_OF_WEAPON)
 							{
 								if (which_id == GLOVE_FUR || which_id == GLOVE_LEATHER)
 								{
-									my_strcp(actors_list[i]->body_parts->hands_tex, actors_defs[actors_list[i]->actor_type].weapon[which_id].skin_name);
-									my_strcp(actors_list[i]->body_parts->hands_mask, actors_defs[actors_list[i]->actor_type].weapon[which_id].skin_mask);
+									safe_strncpy(actors_list[i]->body_parts->hands_tex,
+										actors_defs[actors_list[i]->actor_type].weapon[which_id].skin_name,
+										sizeof(actors_list[i]->body_parts->hands_tex));
+									safe_strncpy(actors_list[i]->body_parts->hands_mask,
+										actors_defs[actors_list[i]->actor_type].weapon[which_id].skin_mask,
+										sizeof(actors_list[i]->body_parts->hands_mask));
 #ifdef CUSTOM_LOOK
 									custom_path(actors_list[i]->body_parts->hands_tex, playerpath, guildpath);
 									custom_path(actors_list[i]->body_parts->hands_mask, playerpath, guildpath);
@@ -362,7 +381,9 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 								}
 								else
 								{
-									my_strcp(actors_list[i]->body_parts->weapon_tex,actors_defs[actors_list[i]->actor_type].weapon[which_id].skin_name);
+									safe_strncpy(actors_list[i]->body_parts->weapon_tex,
+										actors_defs[actors_list[i]->actor_type].weapon[which_id].skin_name,
+										sizeof(actors_list[i]->body_parts->weapon_tex));
 #ifdef CUSTOM_LOOK
 									custom_path(actors_list[i]->body_parts->weapon_tex, playerpath, guildpath);
 #endif
@@ -434,7 +455,9 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 
 						else if (which_part==KIND_OF_SHIELD)
 							{
-								my_strcp(actors_list[i]->body_parts->shield_tex,actors_defs[actors_list[i]->actor_type].shield[which_id].skin_name);
+								safe_strncpy(actors_list[i]->body_parts->shield_tex,
+									actors_defs[actors_list[i]->actor_type].shield[which_id].skin_name,
+									sizeof(actors_list[i]->body_parts->shield_tex));
 #ifdef CUSTOM_LOOK
 								custom_path(actors_list[i]->body_parts->shield_tex, playerpath, guildpath);
 #endif
@@ -450,7 +473,9 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 
 						else if (which_part==KIND_OF_CAPE)
 							{
-								my_strcp(actors_list[i]->body_parts->cape_tex,actors_defs[actors_list[i]->actor_type].cape[which_id].skin_name);
+								safe_strncpy(actors_list[i]->body_parts->cape_tex,
+									actors_defs[actors_list[i]->actor_type].cape[which_id].skin_name,
+									sizeof(actors_list[i]->body_parts->cape_tex));
 #ifdef CUSTOM_LOOK
 								custom_path(actors_list[i]->body_parts->cape_tex, playerpath, guildpath);
 #endif
@@ -464,7 +489,9 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 
 						else if (which_part==KIND_OF_HELMET)
 							{
-								my_strcp(actors_list[i]->body_parts->helmet_tex,actors_defs[actors_list[i]->actor_type].helmet[which_id].skin_name);
+								safe_strncpy(actors_list[i]->body_parts->helmet_tex,
+									actors_defs[actors_list[i]->actor_type].helmet[which_id].skin_name,
+									sizeof(actors_list[i]->body_parts->helmet_tex));
 #ifdef CUSTOM_LOOK
 								custom_path(actors_list[i]->body_parts->helmet_tex, playerpath, guildpath);
 #endif
@@ -478,7 +505,9 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 						else if (which_part==KIND_OF_NECK)
 							{
 								assert(!"Using old client data" || actors_defs[actors_list[i]->actor_type].neck != NULL);
-								my_strcp(actors_list[i]->body_parts->neck_tex,actors_defs[actors_list[i]->actor_type].neck[which_id].skin_name);
+								safe_strncpy(actors_list[i]->body_parts->neck_tex,
+									actors_defs[actors_list[i]->actor_type].neck[which_id].skin_name,
+									sizeof(actors_list[i]->body_parts->neck_tex));
 #ifdef CUSTOM_LOOK
 								custom_path(actors_list[i]->body_parts->neck_tex, playerpath, guildpath);
 #endif
@@ -492,10 +521,18 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 
 						else if (which_part==KIND_OF_BODY_ARMOR)
 							{
-								my_strcp(actors_list[i]->body_parts->arms_tex,actors_defs[actors_list[i]->actor_type].shirt[which_id].arms_name);
-								my_strcp(actors_list[i]->body_parts->torso_tex,actors_defs[actors_list[i]->actor_type].shirt[which_id].torso_name);
-								my_strcp(actors_list[i]->body_parts->arms_mask,actors_defs[actors_list[i]->actor_type].shirt[which_id].arms_mask);
-								my_strcp(actors_list[i]->body_parts->torso_mask,actors_defs[actors_list[i]->actor_type].shirt[which_id].torso_mask);
+								safe_strncpy(actors_list[i]->body_parts->arms_tex,
+									actors_defs[actors_list[i]->actor_type].shirt[which_id].arms_name,
+									sizeof(actors_list[i]->body_parts->arms_tex));
+								safe_strncpy(actors_list[i]->body_parts->torso_tex,
+									actors_defs[actors_list[i]->actor_type].shirt[which_id].torso_name,
+									sizeof(actors_list[i]->body_parts->torso_tex));
+								safe_strncpy(actors_list[i]->body_parts->arms_mask,
+									actors_defs[actors_list[i]->actor_type].shirt[which_id].arms_mask,
+									sizeof(actors_list[i]->body_parts->arms_mask));
+								safe_strncpy(actors_list[i]->body_parts->torso_mask,
+									actors_defs[actors_list[i]->actor_type].shirt[which_id].torso_mask,
+									sizeof(actors_list[i]->body_parts->torso_mask));
 #ifdef CUSTOM_LOOK
 								custom_path(actors_list[i]->body_parts->arms_tex, playerpath, guildpath);
 								custom_path(actors_list[i]->body_parts->torso_tex, playerpath, guildpath);
@@ -515,8 +552,12 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 							}
 						else if (which_part==KIND_OF_LEG_ARMOR)
 							{
-								my_strcp(actors_list[i]->body_parts->pants_tex,actors_defs[actors_list[i]->actor_type].legs[which_id].legs_name);
-								my_strcp(actors_list[i]->body_parts->pants_mask,actors_defs[actors_list[i]->actor_type].legs[which_id].legs_mask);
+								safe_strncpy(actors_list[i]->body_parts->pants_tex,
+									actors_defs[actors_list[i]->actor_type].legs[which_id].legs_name,
+									sizeof(actors_list[i]->body_parts->pants_tex));
+								safe_strncpy(actors_list[i]->body_parts->pants_mask,
+									actors_defs[actors_list[i]->actor_type].legs[which_id].legs_mask,
+									sizeof(actors_list[i]->body_parts->pants_mask));
 #ifdef CUSTOM_LOOK
 								custom_path(actors_list[i]->body_parts->pants_tex, playerpath, guildpath);
 								custom_path(actors_list[i]->body_parts->pants_mask, playerpath, guildpath);
@@ -535,8 +576,12 @@ void actor_wear_item(int actor_id,Uint8 which_part, Uint8 which_id)
 
 						else if (which_part==KIND_OF_BOOT_ARMOR)
 							{
-								my_strcp(actors_list[i]->body_parts->boots_tex,actors_defs[actors_list[i]->actor_type].boots[which_id].boots_name);
-								my_strcp(actors_list[i]->body_parts->boots_mask,actors_defs[actors_list[i]->actor_type].boots[which_id].boots_mask);
+								safe_strncpy(actors_list[i]->body_parts->boots_tex,
+									actors_defs[actors_list[i]->actor_type].boots[which_id].boots_name,
+									sizeof(actors_list[i]->body_parts->boots_tex));
+								safe_strncpy(actors_list[i]->body_parts->boots_mask,
+									actors_defs[actors_list[i]->actor_type].boots[which_id].boots_mask,
+									sizeof(actors_list[i]->body_parts->boots_mask));
 #ifdef CUSTOM_LOOK
 								custom_path(actors_list[i]->body_parts->boots_tex, playerpath, guildpath);
 								custom_path(actors_list[i]->body_parts->boots_mask, playerpath, guildpath);
@@ -588,10 +633,11 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 	int dead=0;
 	int kind_of_actor;
 	enhanced_actor *this_actor;
+#ifdef CUSTOM_LOOK
 	char playerpath[256], guildpath[256];
+#endif
 	char onlyname[32]={0};
-	Uint32 j;
-	Uint32 uniq_id; // - Post ported.... We'll come up with something later...
+	Uint32 uniq_id = 0; // - Post ported.... We'll come up with something later...
 	Uint32 guild_id;
 	double f_x_pos,f_y_pos,f_z_rot;
 	float   scale=1.0f;
@@ -624,10 +670,7 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 #endif
 
 	if(actor_type >= MAX_ACTOR_DEFS || (actor_type > 0 && actors_defs[actor_type].actor_type != actor_type) ){
-		char    str[256];
-
-		safe_snprintf(str, sizeof(str), "Illegal/missing enhanced actor definition %d", actor_type);
-		LOG_ERROR(str);
+		LOG_ERROR("Illegal/missing enhanced actor definition %d", actor_type);
 		return;		// We cannot load an actor without a def (seg fault) so bail here.
 	}
 
@@ -652,12 +695,14 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 #endif
 	//the last bytes of the packet even if scale+attachment is not sent
 #ifdef NEW_EYES
-	if((*(in_data+len-2) > EYES_GOLD)|| (*(in_data+len-2) < 0))
-	{
-		eyes=0;
-	} else {
-		eyes=*(in_data+len-2);
-	}
+	if ((in_data[len-2] > EYES_GOLD)
+#if (EYES_BROWN > 0) || (CHAR_MIN == SCHAR_MIN)
+		|| (in_data[len-2] < EYES_BROWN)
+#endif
+	)
+		eyes = EYES_BROWN;
+	else
+		eyes = in_data[len-2];
 #endif
 	neck=*(in_data+len-1);
 
@@ -748,9 +793,13 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 							i--;// last actor was put here, he needs to be checked too
 						}
 #ifdef UID
-					else if(kind_of_actor==COMPUTER_CONTROLLED_HUMAN && (actors_list[i]->kind_of_actor==COMPUTER_CONTROLLED_HUMAN || actors_list[i]->kind_of_actor==PKABLE_COMPUTER_CONTROLLED) && !my_strcompare(&in_data[32], actors_list[i]->actor_name))
+					else if (kind_of_actor == COMPUTER_CONTROLLED_HUMAN
+						&& (actors_list[i]->kind_of_actor == COMPUTER_CONTROLLED_HUMAN || actors_list[i]->kind_of_actor == PKABLE_COMPUTER_CONTROLLED)
+						&& !strcasecmp(&in_data[32], actors_list[i]->actor_name))
 #else
-					else if(kind_of_actor==COMPUTER_CONTROLLED_HUMAN && (actors_list[i]->kind_of_actor==COMPUTER_CONTROLLED_HUMAN || actors_list[i]->kind_of_actor==PKABLE_COMPUTER_CONTROLLED) && !my_strcompare(&in_data[28], actors_list[i]->actor_name))
+					else if (kind_of_actor == COMPUTER_CONTROLLED_HUMAN
+						&& (actors_list[i]->kind_of_actor == COMPUTER_CONTROLLED_HUMAN || actors_list[i]->kind_of_actor == PKABLE_COMPUTER_CONTROLLED)
+						&& !strcasecmp(&in_data[28], actors_list[i]->actor_name))
 #endif
 						{
 #ifdef UID
@@ -772,54 +821,34 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 
 	/* build a clean player name and a guild id */
 	{
-		/* get the name string into a working buffer */
-		char buffer[256], *name, *guild;
 #ifdef UID
-		my_strncp(buffer,&in_data[32],sizeof(buffer));
+		const size_t name_index = 32;
 #else
-		my_strncp(buffer,&in_data[28],sizeof(buffer));
-		uniq_id = 0;
+		const size_t name_index = 28;
 #endif
+#ifdef NEW_EYES
+		const size_t max_name_len = len - name_index - 5;
+#else
+		const size_t max_name_len = len - name_index - 4;
+#endif
+		size_t guild_name_index = get_onlyname(&in_data[name_index], max_name_len, onlyname, sizeof(onlyname));
 
-		/* skip leading color codes */
-		for (name=buffer; *name && is_color (*name); name++) /* nothing */ ;
-		/* trim off any guild tag, leaving solely the name (onlyname)*/
-		for(j=0; name[j] && name[j]>32;j++){
-			onlyname[j]=name[j];
-		}
-
-		/* search for string end or color mark */
+		/* the guild tag is space + colour code + name (max 4 characters) */
+		guild_id = 0;
 		this_actor->guild_tag_color = 0;
-		for (guild = name; *guild && is_printable (*guild); guild++);
-		if (*guild) {
-			/* separate the two strings */
-			this_actor->guild_tag_color = from_color_char (*guild);
-			*guild = 0;
-			guild++;
-		}
-
-		/* perform case insensitive comparison/hashing */
-		my_tolower(name);
-		my_tolower(onlyname);
-
-		//perfect hashing of guildtag
- 		switch(strlen(guild))
- 		{
-		case 0:
-			guild_id = 0;
-			break;
-		case 1:
-			guild_id = guild[0];
-			break;
- 		case 2:
- 			guild_id = guild[0] + (guild[1] << 8);
- 			break;
- 		case 3:
- 			guild_id = guild[0] + (guild[1] << 8) + (guild[2] << 16);
- 			break;
- 		default:
- 			guild_id = guild[0] + (guild[1] << 8) + (guild[2] << 16) + (guild[3] << 24);
- 			break;
+		if (guild_name_index < max_name_len)
+		{
+			const char *guild = &in_data[name_index + guild_name_index];
+			size_t guild_tag_len = strnlen(guild, max_name_len - guild_name_index);
+			/* get the colour code and hash of guildtag */
+			if ((guild_tag_len > 2) && (guild_tag_len < 7) && (guild[0] == ' ') && is_color(guild[1]))
+			{
+				unsigned int shift = 0;
+				size_t j;
+				this_actor->guild_tag_color = from_color_char(guild[1]);
+				for (j = 0; (j < (guild_tag_len - 2)) && (j < 4); j++, shift += 8)
+					guild_id += guild[2+j] << shift;
+			}
 		}
 	}
 
@@ -834,32 +863,32 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 	this_actor->guild_id = guild_id;
 
 	//get the torso
-	my_strncp(this_actor->arms_tex,actors_defs[actor_type].shirt[shirt].arms_name,sizeof(this_actor->arms_tex));
-	my_strncp(this_actor->torso_tex,actors_defs[actor_type].shirt[shirt].torso_name,sizeof(this_actor->torso_tex));
-	my_strncp(this_actor->arms_mask,actors_defs[actor_type].shirt[shirt].arms_mask,sizeof(this_actor->arms_mask));
-	my_strncp(this_actor->torso_mask,actors_defs[actor_type].shirt[shirt].torso_mask,sizeof(this_actor->torso_mask));
+	safe_strncpy(this_actor->arms_tex,actors_defs[actor_type].shirt[shirt].arms_name,sizeof(this_actor->arms_tex));
+	safe_strncpy(this_actor->torso_tex,actors_defs[actor_type].shirt[shirt].torso_name,sizeof(this_actor->torso_tex));
+	safe_strncpy(this_actor->arms_mask,actors_defs[actor_type].shirt[shirt].arms_mask,sizeof(this_actor->arms_mask));
+	safe_strncpy(this_actor->torso_mask,actors_defs[actor_type].shirt[shirt].torso_mask,sizeof(this_actor->torso_mask));
 	//skin
-	my_strncp(this_actor->hands_tex,actors_defs[actor_type].skin[skin].hands_name,sizeof(this_actor->hands_tex));
-	my_strncp(this_actor->hands_tex_save,actors_defs[actor_type].skin[skin].hands_name,sizeof(this_actor->hands_tex_save));
-	my_strncp(this_actor->hands_mask,"",sizeof(this_actor->hands_mask));	// by default, nothing
-	my_strncp(this_actor->head_tex,actors_defs[actor_type].skin[skin].head_name,sizeof(this_actor->head_tex));
-	my_strncp(this_actor->head_base,actors_defs[actor_type].skin[skin].head_name,sizeof(this_actor->head_base));
-	my_strncp(this_actor->head_mask,"",sizeof(this_actor->head_mask));	// by default, nothing
+	safe_strncpy(this_actor->hands_tex,actors_defs[actor_type].skin[skin].hands_name,sizeof(this_actor->hands_tex));
+	safe_strncpy(this_actor->hands_tex_save,actors_defs[actor_type].skin[skin].hands_name,sizeof(this_actor->hands_tex_save));
+	safe_strncpy(this_actor->hands_mask,"",sizeof(this_actor->hands_mask));	// by default, nothing
+	safe_strncpy(this_actor->head_tex,actors_defs[actor_type].skin[skin].head_name,sizeof(this_actor->head_tex));
+	safe_strncpy(this_actor->head_base,actors_defs[actor_type].skin[skin].head_name,sizeof(this_actor->head_base));
+	safe_strncpy(this_actor->head_mask,"",sizeof(this_actor->head_mask));	// by default, nothing
 	if(*actors_defs[actor_type].head[head].skin_name)
-		my_strncp(this_actor->head_tex,actors_defs[actor_type].head[head].skin_name,sizeof(this_actor->head_tex));
+		safe_strncpy(this_actor->head_tex,actors_defs[actor_type].head[head].skin_name,sizeof(this_actor->head_tex));
 	if(*actors_defs[actor_type].head[head].skin_mask)
-		my_strncp(this_actor->head_mask,actors_defs[actor_type].head[head].skin_mask,sizeof(this_actor->head_mask));
-	my_strncp(this_actor->body_base,actors_defs[actor_type].skin[skin].body_name,sizeof(this_actor->body_base));
-	my_strncp(this_actor->arms_base,actors_defs[actor_type].skin[skin].arms_name,sizeof(this_actor->arms_base));
-	my_strncp(this_actor->legs_base,actors_defs[actor_type].skin[skin].legs_name,sizeof(this_actor->legs_base));
-	my_strncp(this_actor->boots_base,actors_defs[actor_type].skin[skin].feet_name,sizeof(this_actor->boots_base));
+		safe_strncpy(this_actor->head_mask,actors_defs[actor_type].head[head].skin_mask,sizeof(this_actor->head_mask));
+	safe_strncpy(this_actor->body_base,actors_defs[actor_type].skin[skin].body_name,sizeof(this_actor->body_base));
+	safe_strncpy(this_actor->arms_base,actors_defs[actor_type].skin[skin].arms_name,sizeof(this_actor->arms_base));
+	safe_strncpy(this_actor->legs_base,actors_defs[actor_type].skin[skin].legs_name,sizeof(this_actor->legs_base));
+	safe_strncpy(this_actor->boots_base,actors_defs[actor_type].skin[skin].feet_name,sizeof(this_actor->boots_base));
 	//hair
-	my_strncp(this_actor->hair_tex,actors_defs[actor_type].hair[hair].hair_name,sizeof(this_actor->hair_tex));
+	safe_strncpy(this_actor->hair_tex,actors_defs[actor_type].hair[hair].hair_name,sizeof(this_actor->hair_tex));
 #ifdef NEW_EYES
 	//eyes
 	if (actors_defs[actor_type].eyes && *actors_defs[actor_type].eyes[eyes].eyes_name != '\0')
 	{
-		my_strncp(this_actor->eyes_tex,actors_defs[actor_type].eyes[eyes].eyes_name,sizeof(this_actor->eyes_tex));
+		safe_strncpy(this_actor->eyes_tex,actors_defs[actor_type].eyes[eyes].eyes_name,sizeof(this_actor->eyes_tex));
 	}
 	else
 	{
@@ -867,18 +896,18 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 		if (!already_said)
 		{
 			char *message = "Looks like we compiled with NEW_EYES but do not have the textures";
-			LOG_ERROR(message);
+			LOG_ERROR("%s", message);
 			LOG_TO_CONSOLE(c_red2,message);
 			already_said = 1;
 		}
 	}
 #endif
 	//boots
-	my_strncp(this_actor->boots_tex,actors_defs[actor_type].boots[boots].boots_name,sizeof(this_actor->boots_tex));
-	my_strncp(this_actor->boots_mask,actors_defs[actor_type].boots[boots].boots_mask,sizeof(this_actor->boots_mask));
+	safe_strncpy(this_actor->boots_tex,actors_defs[actor_type].boots[boots].boots_name,sizeof(this_actor->boots_tex));
+	safe_strncpy(this_actor->boots_mask,actors_defs[actor_type].boots[boots].boots_mask,sizeof(this_actor->boots_mask));
 	//legs
-	my_strncp(this_actor->pants_tex,actors_defs[actor_type].legs[pants].legs_name,sizeof(this_actor->pants_tex));
-	my_strncp(this_actor->pants_mask,actors_defs[actor_type].legs[pants].legs_mask,sizeof(this_actor->pants_mask));
+	safe_strncpy(this_actor->pants_tex,actors_defs[actor_type].legs[pants].legs_name,sizeof(this_actor->pants_tex));
+	safe_strncpy(this_actor->pants_mask,actors_defs[actor_type].legs[pants].legs_mask,sizeof(this_actor->pants_mask));
 
 #ifdef CUSTOM_LOOK
 	if(kind_of_actor != NPC)
@@ -913,7 +942,7 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 	//cape
 	if(cape!=CAPE_NONE)
 		{
-			my_strncp(this_actor->cape_tex,actors_defs[actor_type].cape[cape].skin_name,sizeof(this_actor->cape_tex));
+			safe_strncpy(this_actor->cape_tex,actors_defs[actor_type].cape[cape].skin_name,sizeof(this_actor->cape_tex));
 #ifdef CUSTOM_LOOK
 			if(kind_of_actor != NPC)
 				custom_path(this_actor->cape_tex, playerpath, guildpath);
@@ -921,12 +950,12 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 		}
 	else
 		{
-			my_strncp(this_actor->cape_tex,"",sizeof(this_actor->cape_tex));
+			safe_strncpy(this_actor->cape_tex,"",sizeof(this_actor->cape_tex));
 		}
 	//shield
 	if(shield!=SHIELD_NONE)
 		{
-			my_strncp(this_actor->shield_tex,actors_defs[actor_type].shield[shield].skin_name,sizeof(this_actor->shield_tex));
+			safe_strncpy(this_actor->shield_tex,actors_defs[actor_type].shield[shield].skin_name,sizeof(this_actor->shield_tex));
 #ifdef CUSTOM_LOOK
 			if(kind_of_actor != NPC)
 				custom_path(this_actor->shield_tex, playerpath, guildpath);
@@ -934,12 +963,12 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 		}
 	else
 		{
-			my_strncp(this_actor->shield_tex,"",sizeof(this_actor->shield_tex));
+			safe_strncpy(this_actor->shield_tex,"",sizeof(this_actor->shield_tex));
 		}
 
 	if (weapon == GLOVE_FUR || weapon == GLOVE_LEATHER)
 	{
-		my_strncp(this_actor->hands_tex, actors_defs[actor_type].weapon[weapon].skin_name,sizeof(this_actor->hands_tex));
+		safe_strncpy(this_actor->hands_tex, actors_defs[actor_type].weapon[weapon].skin_name,sizeof(this_actor->hands_tex));
 #ifdef CUSTOM_LOOK
 		if(kind_of_actor != NPC)
 			custom_path(this_actor->hands_tex, playerpath, guildpath);
@@ -947,7 +976,7 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 	}
 	else
 	{
-		my_strncp(this_actor->weapon_tex,actors_defs[actor_type].weapon[weapon].skin_name,sizeof(this_actor->weapon_tex));
+		safe_strncpy(this_actor->weapon_tex,actors_defs[actor_type].weapon[weapon].skin_name,sizeof(this_actor->weapon_tex));
 #ifdef CUSTOM_LOOK
 		if(kind_of_actor != NPC)
 			custom_path(this_actor->weapon_tex, playerpath, guildpath);
@@ -959,7 +988,7 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 	//helmet
 	if(helmet!=HELMET_NONE)
 		{
-			my_strncp(this_actor->helmet_tex,actors_defs[actor_type].helmet[helmet].skin_name,sizeof(this_actor->helmet_tex));
+			safe_strncpy(this_actor->helmet_tex,actors_defs[actor_type].helmet[helmet].skin_name,sizeof(this_actor->helmet_tex));
 
 #ifdef CUSTOM_LOOK
 			if(kind_of_actor != NPC)
@@ -968,14 +997,14 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 		}
 	else
 		{
-			my_strncp(this_actor->helmet_tex,"",sizeof(this_actor->helmet_tex));
+			safe_strncpy(this_actor->helmet_tex,"",sizeof(this_actor->helmet_tex));
 		}
 
 	//neck
 	if(neck!=NECK_NONE)
 		{
 			assert(!"Using old client data" || actors_defs[actor_type].neck != NULL);
-			my_strncp(this_actor->neck_tex,actors_defs[actor_type].neck[neck].skin_name,sizeof(this_actor->neck_tex));
+			safe_strncpy(this_actor->neck_tex,actors_defs[actor_type].neck[neck].skin_name,sizeof(this_actor->neck_tex));
 #ifdef CUSTOM_LOOK
 			if(kind_of_actor != NPC)
 				custom_path(this_actor->neck_tex, playerpath, guildpath);
@@ -983,7 +1012,7 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 		}
 	else
 		{
-			my_strncp(this_actor->neck_tex,"",sizeof(this_actor->neck_tex));
+			safe_strncpy(this_actor->neck_tex,"",sizeof(this_actor->neck_tex));
 		}
 
 	i=add_enhanced_actor(this_actor,f_x_pos,f_y_pos,0.0,f_z_rot,scale,actor_id, onlyname);
@@ -1072,7 +1101,7 @@ void add_enhanced_actor_from_server (const char *in_data, int len)
 			*ptr = '\0';
 			add_name_to_tablist(buffer);
 		}
-		my_strncp(actors_list[i]->actor_name,&in_data[28],sizeof(actors_list[i]->actor_name));
+		safe_strncpy(actors_list[i]->actor_name,&in_data[28],sizeof(actors_list[i]->actor_name));
 		if(caps_filter && my_isupper(actors_list[i]->actor_name, -1)) {
 			my_tolower(actors_list[i]->actor_name);
 		}
@@ -1230,20 +1259,20 @@ actor * add_actor_interface(float x, float y, float z_rot, float scale, int acto
 	actor * a;
 
 	//get the torso
-	my_strncp(this_actor->arms_tex,actors_defs[actor_type].shirt[shirt].arms_name,sizeof(this_actor->arms_tex));
-	my_strncp(this_actor->arms_mask,actors_defs[actor_type].shirt[shirt].arms_mask,sizeof(this_actor->arms_mask));
-	my_strncp(this_actor->torso_tex,actors_defs[actor_type].shirt[shirt].torso_name,sizeof(this_actor->torso_tex));
-	my_strncp(this_actor->torso_mask,actors_defs[actor_type].shirt[shirt].torso_mask,sizeof(this_actor->torso_mask));
-	my_strncp(this_actor->hands_tex,actors_defs[actor_type].skin[skin].hands_name,sizeof(this_actor->hands_tex));
-	my_strncp(this_actor->head_tex,actors_defs[actor_type].skin[skin].head_name,sizeof(this_actor->head_tex));
-	my_strncp(this_actor->hair_tex,actors_defs[actor_type].hair[hair].hair_name,sizeof(this_actor->hair_tex));
+	safe_strncpy(this_actor->arms_tex,actors_defs[actor_type].shirt[shirt].arms_name,sizeof(this_actor->arms_tex));
+	safe_strncpy(this_actor->arms_mask,actors_defs[actor_type].shirt[shirt].arms_mask,sizeof(this_actor->arms_mask));
+	safe_strncpy(this_actor->torso_tex,actors_defs[actor_type].shirt[shirt].torso_name,sizeof(this_actor->torso_tex));
+	safe_strncpy(this_actor->torso_mask,actors_defs[actor_type].shirt[shirt].torso_mask,sizeof(this_actor->torso_mask));
+	safe_strncpy(this_actor->hands_tex,actors_defs[actor_type].skin[skin].hands_name,sizeof(this_actor->hands_tex));
+	safe_strncpy(this_actor->head_tex,actors_defs[actor_type].skin[skin].head_name,sizeof(this_actor->head_tex));
+	safe_strncpy(this_actor->hair_tex,actors_defs[actor_type].hair[hair].hair_name,sizeof(this_actor->hair_tex));
 #ifdef NEW_EYES
-	my_strncp(this_actor->eyes_tex,actors_defs[actor_type].eyes[eyes].eyes_name,sizeof(this_actor->eyes_tex));
+	safe_strncpy(this_actor->eyes_tex,actors_defs[actor_type].eyes[eyes].eyes_name,sizeof(this_actor->eyes_tex));
 #endif
-	my_strncp(this_actor->boots_tex,actors_defs[actor_type].boots[boots].boots_name,sizeof(this_actor->boots_tex));
-	my_strncp(this_actor->boots_mask,actors_defs[actor_type].boots[boots].boots_mask,sizeof(this_actor->boots_mask));
-	my_strncp(this_actor->pants_tex,actors_defs[actor_type].legs[pants].legs_name,sizeof(this_actor->pants_tex));
-	my_strncp(this_actor->pants_mask,actors_defs[actor_type].legs[pants].legs_mask,sizeof(this_actor->pants_mask));
+	safe_strncpy(this_actor->boots_tex,actors_defs[actor_type].boots[boots].boots_name,sizeof(this_actor->boots_tex));
+	safe_strncpy(this_actor->boots_mask,actors_defs[actor_type].boots[boots].boots_mask,sizeof(this_actor->boots_mask));
+	safe_strncpy(this_actor->pants_tex,actors_defs[actor_type].legs[pants].legs_name,sizeof(this_actor->pants_tex));
+	safe_strncpy(this_actor->pants_mask,actors_defs[actor_type].legs[pants].legs_mask,sizeof(this_actor->pants_mask));
 
 	a=actors_list[add_enhanced_actor(this_actor, x*0.5f, y*0.5f, 0.00000001f, z_rot, scale, 0, 0)];
 
@@ -1257,7 +1286,7 @@ actor * add_actor_interface(float x, float y, float z_rot, float scale, int acto
 	a->stop_animation=1;//helps when the actor is dead...
 	a->kind_of_actor=HUMAN;
 
-	safe_snprintf(a->actor_name, sizeof(a->actor_name), playername);
+	safe_snprintf(a->actor_name, sizeof(a->actor_name), "%s", playername);
 
 	if (actors_defs[actor_type].coremodel!=NULL) {
 		a->calmodel=model_new(actors_defs[actor_type].coremodel);
