@@ -32,7 +32,8 @@
 float proj_on_ground[16];
 float ground_plane[4]={0,0,1,0};
 float sun_position[4]={400.0, 400.0, 500.0, 0.0};
-double light_view_mat[16],light_proj_mat[16],shadow_texgen_mat[16];
+static float light_view_mat[16],light_proj_mat[16];
+float shadow_texgen_mat[16];
 int shadows_on=0;
 int is_day=0;
 int use_shadow_mapping=1;
@@ -139,18 +140,18 @@ void calc_shadow_matrix()
 			xrot*=180.0f/(float)M_PI;
 			glOrtho(-light_view_hscale,light_view_hscale,
 				light_view_bottom,light_view_top,light_view_near,light_view_far);
-			glGetDoublev(GL_MODELVIEW_MATRIX,light_proj_mat);
+			glGetFloatv(GL_MODELVIEW_MATRIX,light_proj_mat);
 			glLoadIdentity();
 			glRotatef(xrot,1.0f,0.0f,0.0f);
 			glRotatef(zrot,0.0f,0.0f,1.0f);
-			glGetDoublev(GL_MODELVIEW_MATRIX,light_view_mat);
+			glGetFloatv(GL_MODELVIEW_MATRIX,light_view_mat);
 			glLoadIdentity();
 			if(depth_texture_target!=GL_TEXTURE_2D)glScalef(shadow_map_size,shadow_map_size,0);
 			glTranslatef(0.5,0.5,0.5);   // This...
 			glScalef(0.5,0.5,0.5);       // ...and this == S
-			glMultMatrixd(light_proj_mat);     // Plight
-			glMultMatrixd(light_view_mat);     // L^-1
-			glGetDoublev(GL_MODELVIEW_MATRIX,shadow_texgen_mat);
+			glMultMatrixf(light_proj_mat);     // Plight
+			glMultMatrixf(light_view_mat);     // L^-1
+			glGetFloatv(GL_MODELVIEW_MATRIX,shadow_texgen_mat);
 			glPopMatrix();
 		}
 	else
@@ -431,10 +432,10 @@ void render_light_view()
 				CHECK_GL_ERRORS();
 				CHECK_FBO_ERRORS();
 				ELglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, shadow_fbo);
-			        ELglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_TEXTURE_2D, depth_map_id, 0);
+				ELglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_TEXTURE_2D, depth_map_id, 0);
 				glClear(GL_DEPTH_BUFFER_BIT);
-        			glDrawBuffer(GL_NONE);
-	        		glReadBuffer(GL_NONE);
+				glDrawBuffer(GL_NONE);
+				glReadBuffer(GL_NONE);
 				CHECK_GL_ERRORS();
 				CHECK_FBO_ERRORS();
 			}
@@ -457,10 +458,10 @@ void render_light_view()
 			CHECK_GL_ERRORS();
 			glMatrixMode(GL_PROJECTION);
 			glPushMatrix();
-			glLoadMatrixd(light_proj_mat);
+			glLoadMatrixf(light_proj_mat);
 			glMatrixMode(GL_MODELVIEW);
 			glPushMatrix();
-			glLoadMatrixd(light_view_mat);
+			glLoadMatrixf(light_view_mat);
 			glTranslatef((int)camera_x,(int)camera_y,(int)camera_z);
 			cur_intersect_type = get_cur_intersect_type(main_bbox_tree);
 			set_cur_intersect_type(main_bbox_tree, INTERSECTION_TYPE_SHADOW);
@@ -554,12 +555,12 @@ void setup_shadow_mapping()
 	glPushMatrix();
 	glLoadIdentity();
 	if (!first_person)
-	glTranslatef(0.0f, 0.0f, -zoom_level*camera_distance);
+		glTranslatef(0.0f, 0.0f, -zoom_level*camera_distance);
 	glRotatef(rx, 1.0f, 0.0f, 0.0f);
 	if (first_person)
 	{
 		float head_pos[3];
-        cal_get_actor_bone_local_position(get_our_actor(), get_actor_bone_id(get_our_actor(), head_bone), NULL, head_pos);
+		cal_get_actor_bone_local_position(get_our_actor(), get_actor_bone_id(get_our_actor(), head_bone), NULL, head_pos);
 		glTranslatef(head_pos[0], head_pos[1], 0.0);
 	}
 	glRotatef(rz, 0.0f, 0.0f, 1.0f);
@@ -601,11 +602,34 @@ CHECK_GL_ERRORS();
 #endif //OPENGL_TRACE
 }
 
+void setup_cloud_texturing(void)
+{
+	ELglActiveTextureARB(detail_unit);
+	if (!dungeon && clouds_shadows)
+	{
+		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
+		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE);
+		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PREVIOUS);
+		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);
+		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_TEXTURE);
+		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR);
+		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE);
+		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_PREVIOUS);
+		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA);
+	}
+	else
+	{
+		glDisable(GL_TEXTURE_2D);
+		glBindTexture(GL_TEXTURE_2D, 0);
+	}
+	ELglActiveTextureARB(base_unit);
+}
+
 void draw_sun_shadowed_scene(int any_reflection)
 {
 	if(use_shadow_mapping)
 		{
-            reset_material();
+			reset_material();
 
 			shadow_unit=GL_TEXTURE0_ARB;
 			base_unit=GL_TEXTURE1_ARB;
@@ -618,10 +642,7 @@ void draw_sun_shadowed_scene(int any_reflection)
 			glEnable(depth_texture_target);
 			setup_shadow_mapping();
 
-			ELglActiveTextureARB(detail_unit);
-			glDisable(GL_TEXTURE_2D);
-			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-			glBindTexture(GL_TEXTURE_2D,0);
+			setup_cloud_texturing();
 
 			ELglClientActiveTextureARB(base_unit);
 			ELglActiveTextureARB(base_unit);
@@ -685,10 +706,11 @@ void draw_sun_shadowed_scene(int any_reflection)
 		}
 	else
 		{
-
 			glNormal3f(0.0f,0.0f,1.0f);
 			if(any_reflection)draw_lake_tiles();
 
+			setup_cloud_texturing();
+
 			draw_tile_map();
 #ifdef MAP_EDITOR2
 			get_world_x_y ();
