@@ -6,6 +6,7 @@
 #include "asc.h"
 #include "context_menu.h"
 #include "dialogues.h"
+#include "elloggingwrapper.h"
 #include "elwindows.h"
 #include "filter.h"
 #include "gamewin.h"
@@ -26,7 +27,6 @@
 #endif
 
 #define STORAGE_CATEGORIES_SIZE 50
-#define STORAGE_CATEGORIES_DISPLAY 13
 #define STORAGE_SCROLLBAR_CATEGORIES 1200
 #define STORAGE_SCROLLBAR_ITEMS 1201
 
@@ -39,19 +39,21 @@ struct storage_category {
 int no_storage_categories=0;
 int selected_category=-1;
 int view_only_storage=0;
+int storage_use_allowed = 0;
 Uint32 drop_fail_time = 0;
 int sort_storage_categories = 0;
-int max_cat_len = 0;
+int sort_storage_items = 0;
 
 int active_storage_item=-1;
 
-ground_item storage_items[STORAGE_ITEMS_SIZE]={{0,0,0}};
+ground_item storage_items[STORAGE_ITEMS_SIZE]={{0,0,0,0}};
 int no_storage;
 
 #define MAX_DESCR_LEN 202
+static const int item_grid_size = 6;
 char storage_text[MAX_DESCR_LEN]={0};
 static char last_storage_text[MAX_DESCR_LEN]={0};
-static char wrapped_storage_text[MAX_DESCR_LEN+10]={0};
+static unsigned char wrapped_storage_text[MAX_DESCR_LEN+10]={0};
 
 static struct { int quantity; int id; int image_id; } print_info[STORAGE_ITEMS_SIZE];
 static int number_to_print = 0;
@@ -71,7 +73,7 @@ static void select_item(int image_id, Uint16 item_id)
 {
 	int i;
 	int found_at = -1;
-	
+
 	for (i=0; i<no_storage; i++)
 	{
 		if ((item_id != unset_item_uid) && (storage_items[i].id != unset_item_uid) && (storage_items[i].quantity > 0))
@@ -116,7 +118,7 @@ static Uint16 wanted_item_id = -1;
 static int wanted_image_id = -1;
 static void move_to_category(int cat);
 static int find_category(int id);
-
+static int storage_categories_display = 0;
 
 //	Called when the category is changed.
 //	- Update the store of items/category.
@@ -141,7 +143,7 @@ static void category_updated(void)
 //
 void pickup_storage_item(int image_id, Uint16 item_id, int cat_id)
 {
-	if ((storage_win<0) || (find_category(cat_id) == -1))
+	if ((get_id_MW(MW_STORAGE) < 0) || (find_category(cat_id) == -1))
 	{
 		do_alert1_sound();
 		item_lists_reset_pickup_fail_time();
@@ -149,7 +151,7 @@ void pickup_storage_item(int image_id, Uint16 item_id, int cat_id)
 	}
 	wanted_category = find_category(cat_id);
 	wanted_image_id = image_id;
-	wanted_item_id = item_id;	
+	wanted_item_id = item_id;
 	if (selected_category == wanted_category)
 	{
 		select_item(wanted_image_id, wanted_item_id);
@@ -186,34 +188,18 @@ static int category_cmp(const void *a, const void *b)
 				((const struct storage_category *)b)->name);
 }
 
-void get_storage_categories (const char *in_data, int len)
+void get_storage_categories(const char *in_data, int len)
 {
-	int i;
-	int idx, idxp;
+	int i, idx;
 
 	idx = 1;
 	for (i = 0; i < in_data[0] && i < STORAGE_CATEGORIES_SIZE && idx < len; i++)
 	{
 		storage_categories[i].id = (Uint8)in_data[idx++];
-		storage_categories[i].name[0] = to_color_char (c_orange1);
-		idxp = 1;
-		while (idx < len && idxp < sizeof (storage_categories[i].name) - 1 && in_data[idx] != '\0')
-		{
-			storage_categories[i].name[idxp++] = in_data[idx++];
-		}
-		// always make sure the string is terminated
-		storage_categories[i].name[idxp] = '\0';
-
-		// was the string too long?
-		if (idxp >= sizeof (storage_categories[i].name) - 1)
-		{
-			// skip rest of string
-			while (idx < len && in_data[idx] != '\0') idx++;
-		}
-		// save the maximum length category name
-		if ((idxp-1) > max_cat_len)
-			max_cat_len = idxp-1;
-		idx++;
+		storage_categories[i].name[0] = to_color_char(c_orange1);
+		safe_strncpy(storage_categories[i].name+1, in_data+idx,
+			sizeof(storage_categories[i].name)-1);
+		idx += strlen(in_data+idx) + 1;
 	}
 	if (sort_storage_categories)
 		qsort(storage_categories, i, sizeof(*storage_categories), category_cmp);
@@ -224,7 +210,7 @@ void get_storage_categories (const char *in_data, int len)
 	}
 
 	no_storage_categories = in_data[0];
-	if (storage_win > 0) vscrollbar_set_bar_len(storage_win, STORAGE_SCROLLBAR_CATEGORIES, ( no_storage_categories - STORAGE_CATEGORIES_DISPLAY ) > 1 ? (no_storage_categories - STORAGE_CATEGORIES_DISPLAY) : 1);
+	if (get_id_MW(MW_STORAGE) > 0) vscrollbar_set_bar_len(get_id_MW(MW_STORAGE), STORAGE_SCROLLBAR_CATEGORIES, ( no_storage_categories - storage_categories_display ) > 1 ? (no_storage_categories - storage_categories_display) : 1);
 
 	selected_category=-1;
 	active_storage_item=-1;
@@ -247,29 +233,65 @@ int find_category(int id)
 
 void set_window_name(char *extra_sep, char *extra_name)
 {
-	safe_snprintf(windows_list.window[storage_win].window_name,
-		sizeof(windows_list.window[storage_win].window_name),
+	safe_snprintf(windows_list.window[get_id_MW(MW_STORAGE)].window_name,
+		sizeof(windows_list.window[get_id_MW(MW_STORAGE)].window_name),
 		"%s%s%s%s", win_storage, extra_sep, extra_name, ((view_only_storage) ?win_storage_vo:""));
 }
 
 void move_to_category(int cat)
 {
 	Uint8 str[4];
-	
+
 	if(cat<0||cat>=no_storage_categories) return;
 	storage_categories[cat].name[0] = to_color_char (c_red3);
-	if (selected_category!=-1 && cat!=selected_category) 
+	if (selected_category!=-1 && cat!=selected_category)
 		storage_categories[selected_category].name[0] = to_color_char (c_orange1);
 	set_window_name(" - ", storage_categories[cat].name+1);
 
 	str[0]=GET_STORAGE_CATEGORY;
 	*((Uint8 *)(str+1))=storage_categories[cat].id;
 
-	my_tcp_send(my_socket, str, 2);
+	my_tcp_send(str, 2);
 }
 
-static void update_item_filter(void)
+static int item_cmp(const void *a, const void *b)
 {
+	const ground_item *item_a = (const ground_item *)a;
+	const ground_item *item_b = (const ground_item *)b;
+	int a_count, b_count;
+
+	if ((item_a->quantity <= 0) && (item_b->quantity <= 0))
+		return 0;
+	if (item_a->quantity <= 0)
+		return 1;
+	if (item_b->quantity <= 0)
+		return -1;
+
+	a_count = get_item_count(item_a->id, item_a->image_id);
+	b_count = get_item_count(item_b->id, item_b->image_id);
+
+	if ((a_count != 1) && (b_count != 1))
+		return 0;
+	if (a_count != 1)
+		return 1;
+	if (b_count != 1)
+		return -1;
+
+	return strcmp(get_basic_item_description(item_a->id, item_a->image_id),
+		get_basic_item_description(item_b->id, item_b->image_id));
+}
+
+static void update_items_in_category(void)
+{
+	// Make sure dragged storage items with zero quantity are no longer dragged
+	if ((storage_item_dragged >=0) && (storage_item_dragged < STORAGE_ITEMS_SIZE) && (storage_items[storage_item_dragged].quantity <= 0))
+		storage_item_dragged = -1;
+
+	// if enabled, sort the items alphabetically by description (if we have one)
+	if ((no_storage > 0) && sort_storage_items)
+		qsort(storage_items, STORAGE_ITEMS_SIZE, sizeof(ground_item), item_cmp);
+
+	// Update the filter
 	if (!disable_storage_filter && (no_storage > 0) && (filter_item_text_size > 0))
 		filter_items_by_description(storage_items_filter, storage_items, filter_item_text, no_storage);
 	else
@@ -278,6 +300,22 @@ static void update_item_filter(void)
 		for (i=0; i<STORAGE_ITEMS_SIZE; i++)
 			storage_items_filter[i] = 0;
 	}
+
+	// Calculate the last row containing items and set the scrollbar to scroll just enough.
+	// If the last row is full, we set the bar to scroll to the next row so we can see empty space.
+	{
+		size_t i;
+		int last_occupied_row = 0;
+		for (i = 0; i < STORAGE_ITEMS_SIZE; i++)
+			if (storage_items[i].quantity > 0)
+			{
+				last_occupied_row = i / item_grid_size;
+				if ((i % item_grid_size) == (item_grid_size - 1))
+					last_occupied_row += 1;
+			}
+		vscrollbar_set_bar_len(get_id_MW(MW_STORAGE), STORAGE_SCROLLBAR_ITEMS,
+			(last_occupied_row > (item_grid_size - 1)) ?last_occupied_row - (item_grid_size - 1) : 0);
+	}
 }
 
 void get_storage_items (const Uint8 *in_data, int len)
@@ -286,6 +324,13 @@ void get_storage_items (const Uint8 *in_data, int len)
 	int cat, pos;
 	int idx;
 	int plen;
+	int min_len = 2;
+
+	if (len < min_len)
+	{
+		LOG_WARNING("Got incomplete storage items update from the server");
+		return;
+	}
 
 	if (item_uid_enabled)
 		plen=10;
@@ -294,10 +339,18 @@ void get_storage_items (const Uint8 *in_data, int len)
 
 	if (in_data[0] == 255)
 	{
+		// We expect a single full update packet
+		min_len += plen;
+		if (len < min_len)
+		{
+			LOG_WARNING("Got incomplete storage items update from the server");
+			return;
+		}
+
 		// It's just an update - make sure we're in the right category
 		idx = 2;
 		active_storage_item = SDL_SwapLE16(*((Uint16*)(&in_data[idx+6])));
-		
+
 		for (i = 0; i < STORAGE_ITEMS_SIZE; i++)
 		{
 			if ((storage_items[i].pos == SDL_SwapLE16(*((Uint16*)(&in_data[idx+6])))) && (storage_items[i].quantity > 0))
@@ -308,7 +361,7 @@ void get_storage_items (const Uint8 *in_data, int len)
 					storage_items[i].id = SDL_SwapLE16(*((Uint16*)(&in_data[idx+8])));
 				else
 					storage_items[i].id = unset_item_uid;
-				update_item_filter();
+				update_items_in_category();
 				return;
 			}
 		}
@@ -325,7 +378,7 @@ void get_storage_items (const Uint8 *in_data, int len)
 				storage_items[i].image_id = SDL_SwapLE16(*((Uint16*)(&in_data[idx])));
 				storage_items[i].quantity = SDL_SwapLE32(*((Uint32*)(&in_data[idx+2])));
 				no_storage++;
-				update_item_filter();
+				update_items_in_category();
 				return;
 			}
 		}
@@ -354,29 +407,27 @@ void get_storage_items (const Uint8 *in_data, int len)
 		else
 			storage_items[i].id = unset_item_uid;
 	}
-	
+
 	for ( ; i < STORAGE_ITEMS_SIZE; i++)
 	{
 		storage_items[i].quantity=0;
 	}
-	
-	vscrollbar_set_pos(storage_win, STORAGE_SCROLLBAR_ITEMS, 0);
-	pos = vscrollbar_get_pos(storage_win, STORAGE_SCROLLBAR_CATEGORIES);
+
+	vscrollbar_set_pos(get_id_MW(MW_STORAGE), STORAGE_SCROLLBAR_ITEMS, 0);
+	pos = vscrollbar_get_pos(get_id_MW(MW_STORAGE), STORAGE_SCROLLBAR_CATEGORIES);
 	if (cat < pos) {
-		vscrollbar_set_pos(storage_win, STORAGE_SCROLLBAR_CATEGORIES, cat);
-	} else	if (cat >= pos + STORAGE_CATEGORIES_DISPLAY) {
-		vscrollbar_set_pos(storage_win, STORAGE_SCROLLBAR_CATEGORIES, cat - STORAGE_CATEGORIES_DISPLAY + 1);
+		vscrollbar_set_pos(get_id_MW(MW_STORAGE), STORAGE_SCROLLBAR_CATEGORIES, cat);
+	} else	if (cat >= pos + storage_categories_display) {
+		vscrollbar_set_pos(get_id_MW(MW_STORAGE), STORAGE_SCROLLBAR_CATEGORIES, cat - storage_categories_display + 1);
 	}
 
+	update_items_in_category();
+
 	if (selected_category != -1)
 		category_updated();
 
-	update_item_filter();
 }
 
-int storage_win=-1;
-int storage_win_x=100;
-int storage_win_y=100;
 static int cat_scrollbar_id = 0;
 static int items_scrollbar_id = 0;
 
@@ -390,24 +441,36 @@ static int item_grid_left_offset = 0;
 static int item_right_offset = 0;
 static int border_size = 0;
 static int bottom_offset = 0;
-static int item_grid_size = 0;
 static int item_box_size = 0;
 static int cat_string_left_offset = 0;
 static int cat_string_top_offset = 0;
-static int cat_name_seperation = 0;
+static int cat_name_separation = 0;
 static int desc_string_left_offset = 0;
 static int desc_string_top_offset = 0;
 static int desc_box_top_offset = 0;
 static int desc_box_right_offset = 0;
-static int desc_box_botton_offset = 0;
+static int desc_box_bottom_offset = 0;
 static int scrollbar_width = 0;
 static int scrollbar_len = 0;
 
+static int get_max_cat_width(font_cat cat, float zoom)
+{
+	int i, max_width = 0;
+	for (i = 0; i < no_storage_categories; ++i)
+	{
+		int width = get_string_width_zoom((const unsigned char*)storage_categories[i].name,
+			cat, zoom);
+		max_width = max2i(max_width, width);
+	}
+	return max_width;
+}
+
 /* called when the UI scale changes */
 static int ui_scale_storage_handler(window_info *win)
 {
+	int max_cat_width = get_max_cat_width(win->font_category, win->current_scale_small);
+
 	// set the basic sizes
-	item_grid_size = 6;
 	border_size = (int)(0.5 + 10 * win->current_scale);
 	item_box_size = (int)(0.5 + 32 * win->current_scale);
 
@@ -418,20 +481,21 @@ static int ui_scale_storage_handler(window_info *win)
 
 	cat_string_left_offset = (int)(0.5 + 2 * border_size);
 	cat_string_top_offset = (int)(0.5 + 2 * border_size);
-	cat_name_seperation = (int)(0.5 + (scrollbar_len - 2 * border_size) / (float)STORAGE_CATEGORIES_DISPLAY);
-	cat_right_offset = (int)(0.5 + 3 * border_size + max_cat_len * win->small_font_len_x);
+	storage_categories_display = (scrollbar_len - 2*border_size) / win->small_font_len_y;
+	cat_name_separation = (int)(0.5 + (scrollbar_len - 2*border_size) / storage_categories_display);
+	cat_right_offset = (int)(0.5 + 3 * border_size + max_cat_width);
 
 	item_left_offset = (int)(0.5 + cat_right_offset + scrollbar_width);
 	item_grid_left_offset = (int)(0.5 + item_left_offset + border_size);
 	item_right_offset = (int)(0.5 + item_grid_left_offset + item_grid_size * item_box_size);
 
 	desc_box_top_offset = (int)(0.5 + 2 * border_size + item_grid_size * item_box_size);
-	desc_box_botton_offset = (int)(0.5 + desc_box_top_offset + 2 * border_size + 2 * win->small_font_len_y);
+	desc_box_bottom_offset = (int)(0.5 + desc_box_top_offset + 2 * border_size + 2 * win->small_font_len_y);
 	desc_string_left_offset = (int)(0.5 + 2 * border_size);
 	desc_string_top_offset = (int)(0.5 + desc_box_top_offset + border_size);
 
 	storage_win_x_len = (int)(0.5 + item_right_offset + scrollbar_width + border_size + win->box_size);
-	storage_win_y_len = (int)(0.5 + desc_box_botton_offset + border_size);
+	storage_win_y_len = (int)(0.5 + desc_box_bottom_offset + border_size);
 	desc_box_right_offset = (int)(0.5 + storage_win_x_len - border_size);
 
 	// resize the window and scrollbars and move the scrollbars to correct position
@@ -440,32 +504,40 @@ static int ui_scale_storage_handler(window_info *win)
 	widget_resize(win->window_id, cat_scrollbar_id, scrollbar_width, scrollbar_len);
 	widget_move(win->window_id, items_scrollbar_id, item_right_offset, border_size);
 	widget_resize(win->window_id, items_scrollbar_id, scrollbar_width, scrollbar_len);
+	vscrollbar_set_bar_len(win->window_id, STORAGE_SCROLLBAR_CATEGORIES,
+		(no_storage_categories - storage_categories_display) > 1 ? (no_storage_categories - storage_categories_display) : 1);
 
 	return 0;
 }
 
+static int change_storage_font_handler(window_info *win, font_cat cat)
+{
+	if (cat != UI_FONT)
+		return 0;
+	ui_scale_storage_handler(win);
+	return 1;
+}
+
 int cur_item_over=-1;
 int storage_item_dragged=-1;
 
 static int post_display_storage_handler(window_info * win)
 {
-	if(cur_item_over!=-1 && mouse_in_window(win->window_id, mouse_x, mouse_y) == 1){
-		char str[20];
-		if (active_storage_item!=storage_items[cur_item_over].pos) {
-			safe_snprintf(str, sizeof(str), "%d",storage_items[cur_item_over].quantity);
-			if (enlarge_text())
-				show_help_big(str, mouse_x - win->pos_x - strlen(str) * win->default_font_len_x / 2,
-					mouse_y - win->pos_y - win->default_font_len_y, win->current_scale);
-			else
-				show_help(str, mouse_x - win->pos_x - strlen(str) * win->small_font_len_x / 2,
-					mouse_y - win->pos_y - win->small_font_len_y, win->current_scale);
-		}
+	if ((cur_item_over !=- 1) && (mouse_in_window(win->window_id, mouse_x, mouse_y) == 1) &&
+		(active_storage_item != storage_items[cur_item_over].pos)  && (storage_items[cur_item_over].quantity > 0))
+	{
+		float zoom = enlarge_text() ? win->current_scale : win->current_scale_small;
+		float line_height = enlarge_text() ? win->default_font_len_y : win->small_font_len_y;
+		unsigned char str[20];
+		safe_snprintf((char*)str, sizeof(str), "%d",storage_items[cur_item_over].quantity);
+		show_help_colored_scaled_centered(str, mouse_x - win->pos_x,
+			mouse_y - win->pos_y - line_height, 1.0f, 1.0f, 1.0f, zoom);
 	}
 
 	if(active_storage_item >= 0) {
 		int i, pos;
 		/* Draw the active item's quantity on top of everything else. */
-		for(i = pos = item_grid_size * vscrollbar_get_pos(storage_win, STORAGE_SCROLLBAR_ITEMS); i < pos + item_grid_size*item_grid_size && i < no_storage; i++) {
+		for(i = pos = item_grid_size * vscrollbar_get_pos(win->window_id, STORAGE_SCROLLBAR_ITEMS); i < pos + item_grid_size*item_grid_size && i < no_storage; i++) {
 			if(storage_items[i].pos == active_storage_item) {
 				if (storage_items[i].quantity) {
 					char str[20];
@@ -507,31 +579,35 @@ int display_storage_handler(window_info * win)
 
 	have_storage_list = 0;	//We visited storage, so we may have changed something
 
-	glColor3f(0.77f, 0.57f, 0.39f);
+	glColor3fv(gui_color);
 	glEnable(GL_TEXTURE_2D);
-	
-	for(i=pos=vscrollbar_get_pos(storage_win,STORAGE_SCROLLBAR_CATEGORIES); i<no_storage_categories && storage_categories[i].id!=-1 && i<pos+STORAGE_CATEGORIES_DISPLAY; i++,n++){
+
+	for(i=pos=vscrollbar_get_pos(win->window_id,STORAGE_SCROLLBAR_CATEGORIES); i<no_storage_categories && storage_categories[i].id!=-1 && i<pos+storage_categories_display; i++,n++){
 		int the_colour = from_color_char(storage_categories[i].name[0]);
 		size_t offset = 1;
 		if (the_colour == c_red3)
 			elglColourI(c_selected);
 		else if (the_colour == c_green2)
 			elglColourI(c_highlighted);
+		else if (the_colour == c_orange1)
+			glColor3fv(gui_bright_color);
 		else
 			offset = 0;
-		draw_string_small_zoomed(cat_string_left_offset, cat_string_top_offset + n * cat_name_seperation, (unsigned char*)&storage_categories[i].name[offset],1, win->current_scale);
+		draw_string_small_zoomed(cat_string_left_offset, cat_string_top_offset + n * cat_name_separation, (unsigned char*)&storage_categories[i].name[offset],1, win->current_scale);
 	}
-	glColor3f(0.77f, 0.57f, 0.39f);
+	glColor3fv(gui_color);
 	if(storage_text[0]){
 		if (strcmp(storage_text, last_storage_text) != 0) {
 			safe_strncpy(last_storage_text, storage_text, sizeof(last_storage_text));
-			put_small_text_in_box_zoomed ((Uint8 *)storage_text, strlen(storage_text), win->len_x - 4 * border_size, wrapped_storage_text, win->current_scale);
+			put_small_text_in_box_zoomed((Uint8 *)storage_text, strlen(storage_text),
+				win->len_x - 4 * border_size, wrapped_storage_text, win->current_scale);
 		}
-		draw_string_small_zoomed(desc_string_left_offset, desc_string_top_offset, (unsigned char*)wrapped_storage_text, 2, win->current_scale);
+		draw_string_small_zoomed(desc_string_left_offset, desc_string_top_offset,
+			wrapped_storage_text, 2, win->current_scale);
 	}
 
 	glColor3f(1.0f,1.0f,1.0f);
-	for(i = pos = item_grid_size * vscrollbar_get_pos(storage_win, STORAGE_SCROLLBAR_ITEMS); i < pos + item_grid_size*item_grid_size && i < no_storage; i++){
+	for(i = pos = item_grid_size * vscrollbar_get_pos(win->window_id, STORAGE_SCROLLBAR_ITEMS); i < pos + item_grid_size*item_grid_size && i < no_storage; i++){
 		GLfloat u_start, v_start, u_end, v_end;
 		int x_start, x_end, y_start, y_end;
 		int cur_item;
@@ -540,7 +616,7 @@ int display_storage_handler(window_info * win)
 		if(!storage_items[i].quantity)continue;
 		cur_item=storage_items[i].image_id%25;
 		get_item_uv(cur_item, &u_start, &v_start, &u_end, &v_end);
-		
+
 		this_texture=get_items_texture(storage_items[i].image_id/25);
 
 		if (this_texture != -1)
@@ -567,14 +643,14 @@ int display_storage_handler(window_info * win)
 		if (show_item_desc_text && item_info_available() && (get_item_count(item_id, image_id) == 1))
 			show_help(get_item_description(item_id, image_id), 0, win->len_y + 10 + (help_text_line++) * win->small_font_len_y, win->current_scale);
 	}
-	
+
 	// Render the grid *after* the images. It seems impossible to code
-	// it such that images are rendered exactly within the boxes on all 
+	// it such that images are rendered exactly within the boxes on all
 	// cards
 	glDisable(GL_TEXTURE_2D);
-	
-	glColor3f(0.77f, 0.57f, 0.39f);
-	
+
+	glColor3fv(gui_color);
+
 	glBegin(GL_LINE_LOOP);
 		glVertex2i(border_size,  border_size);
 		glVertex2i(border_size,  bottom_offset);
@@ -584,11 +660,11 @@ int display_storage_handler(window_info * win)
 
 	glBegin(GL_LINE_LOOP);
 		glVertex2i(border_size, desc_box_top_offset);
-		glVertex2i(border_size, desc_box_botton_offset);
-		glVertex2i(desc_box_right_offset, desc_box_botton_offset);
+		glVertex2i(border_size, desc_box_bottom_offset);
+		glVertex2i(desc_box_right_offset, desc_box_bottom_offset);
 		glVertex2i(desc_box_right_offset, desc_box_top_offset);
 	glEnd();
-	
+
 	if (view_only_storage)
 	{
 		Uint32 currentticktime = SDL_GetTicks();
@@ -629,25 +705,25 @@ int click_storage_handler(window_info * win, int mx, int my, Uint32 flags)
 {
 	if(flags&ELW_WHEEL_UP) {
 		if(mx>border_size && mx<cat_right_offset) {
-			vscrollbar_scroll_up(storage_win, STORAGE_SCROLLBAR_CATEGORIES);
+			vscrollbar_scroll_up(win->window_id, STORAGE_SCROLLBAR_CATEGORIES);
 		} else if(mx>item_left_offset && mx<item_right_offset){
-			vscrollbar_scroll_up(storage_win, STORAGE_SCROLLBAR_ITEMS);
+			vscrollbar_scroll_up(win->window_id, STORAGE_SCROLLBAR_ITEMS);
 		}
 	} else if(flags&ELW_WHEEL_DOWN) {
 		if(mx>border_size && mx<cat_right_offset) {
-			vscrollbar_scroll_down(storage_win, STORAGE_SCROLLBAR_CATEGORIES);
+			vscrollbar_scroll_down(win->window_id, STORAGE_SCROLLBAR_CATEGORIES);
 		} else if(mx>item_left_offset && mx<item_right_offset){
-			vscrollbar_scroll_down(storage_win, STORAGE_SCROLLBAR_ITEMS);
+			vscrollbar_scroll_down(win->window_id, STORAGE_SCROLLBAR_ITEMS);
 		}
 	}
 	else if ( (flags & ELW_MOUSE_BUTTON) == 0) {
 		return 0;
 	}
 	else {
-		if((my > cat_string_top_offset) && (my < (cat_string_top_offset + STORAGE_CATEGORIES_DISPLAY * cat_name_seperation))){
+		if((my > cat_string_top_offset) && (my < (cat_string_top_offset + storage_categories_display * cat_name_separation))){
 			if(mx>border_size && mx<cat_right_offset){
 				int cat=-1;
-				cat=(my - cat_string_left_offset) / cat_name_seperation + vscrollbar_get_pos(storage_win, STORAGE_SCROLLBAR_CATEGORIES);
+				cat=(my - cat_string_left_offset) / cat_name_separation + vscrollbar_get_pos(win->window_id, STORAGE_SCROLLBAR_CATEGORIES);
 				move_to_category(cat);
 				do_click_sound();
 			}
@@ -659,11 +735,12 @@ int click_storage_handler(window_info * win, int mx, int my, Uint32 flags)
 				} else if(!view_only_storage && item_dragged!=-1 && left_click){
 					Uint8 str[6];
 
+					reset_storage_rate_limit();
 					str[0]=DEPOSITE_ITEM;
 					str[1]=item_list[item_dragged].pos;
 					*((Uint32*)(str+2))=SDL_SwapLE32(item_quantity);
 
-					my_tcp_send(my_socket, str, 6);
+					my_tcp_send(str, 6);
 					do_drop_item_sound();
 
 					if(item_list[item_dragged].quantity<=item_quantity) item_dragged=-1;//Stop dragging this item...
@@ -676,9 +753,9 @@ int click_storage_handler(window_info * win, int mx, int my, Uint32 flags)
 
 						str[0]=LOOK_AT_STORAGE_ITEM;
 						*((Uint16*)(str+1))=SDL_SwapLE16(storage_items[cur_item_over].pos);
-	
-						my_tcp_send(my_socket, str, 3);
-	
+
+						my_tcp_send(str, 3);
+
 						active_storage_item=storage_items[cur_item_over].pos;
 						do_click_sound();
 					}
@@ -705,13 +782,13 @@ int mouseover_storage_handler(window_info *win, int mx, int my)
 	else
 		mouse_over_storage = 1;
 
-	if((my > cat_string_top_offset) && (my < (cat_string_top_offset + STORAGE_CATEGORIES_DISPLAY * cat_name_seperation))){
+	if((my > cat_string_top_offset) && (my < (cat_string_top_offset + storage_categories_display * cat_name_separation))){
 		if(mx>border_size && mx<cat_right_offset){
 			int i;
-			int pos=last_pos=(my - cat_string_top_offset) / cat_name_seperation;
+			int pos=last_pos=(my - cat_string_top_offset) / cat_name_separation;
 			int p;
 
-			for(i=p=vscrollbar_get_pos(storage_win,STORAGE_SCROLLBAR_CATEGORIES);i<no_storage_categories;i++){
+			for(i=p=vscrollbar_get_pos(win->window_id, STORAGE_SCROLLBAR_CATEGORIES);i<no_storage_categories;i++){
 				if(i==selected_category) {
 				} else if(i!=p+pos) {
 					storage_categories[i].name[0]  = to_color_char (c_orange1);
@@ -719,17 +796,17 @@ int mouseover_storage_handler(window_info *win, int mx, int my)
 					storage_categories[i].name[0] = to_color_char (c_green2);
 				}
 			}
-			
+
 			return 0;
 		}
 	}
 	if ((my > border_size) && (my<bottom_offset) && (mx > item_grid_left_offset) && (mx < item_right_offset)) {
-		cur_item_over = get_mouse_pos_in_grid(mx, my, item_grid_size, item_grid_size, item_grid_left_offset, border_size, item_box_size, item_box_size)+vscrollbar_get_pos(storage_win, STORAGE_SCROLLBAR_ITEMS)*item_grid_size;
+		cur_item_over = get_mouse_pos_in_grid(mx, my, item_grid_size, item_grid_size, item_grid_left_offset, border_size, item_box_size, item_box_size)+vscrollbar_get_pos(win->window_id, STORAGE_SCROLLBAR_ITEMS)*item_grid_size;
 		if(cur_item_over>=no_storage||cur_item_over<0||!storage_items[cur_item_over].quantity) cur_item_over=-1;
 	}
-	
-	last_category = last_pos+vscrollbar_get_pos(storage_win,STORAGE_SCROLLBAR_CATEGORIES);
-	if(last_pos>=0 && last_pos<STORAGE_CATEGORIES_DISPLAY && last_category != selected_category) {
+
+	last_category = last_pos+vscrollbar_get_pos(win->window_id,STORAGE_SCROLLBAR_CATEGORIES);
+	if(last_pos>=0 && last_pos<storage_categories_display && last_category != selected_category) {
 		storage_categories[last_category].name[0] = to_color_char (c_orange1);
 		last_pos=-1;
 	}
@@ -771,14 +848,14 @@ void print_items(void)
 			LOG_TO_CONSOLE(c_red1, "You can't do this during combat!");
 			return;
 		}
-	
+
 	/* request the description for each item */
 	number_to_print = next_item_to_print = 0;
 	printing_category = selected_category;
 	for (i = 0; i < no_storage && i < STORAGE_ITEMS_SIZE; i++)
 	{
 		if (storage_items[i].quantity)
-		{		
+		{
 			Uint8 str[3];
 			print_info[number_to_print].quantity = storage_items[i].quantity;
 			print_info[number_to_print].id = storage_items[i].id;
@@ -786,7 +863,7 @@ void print_items(void)
 			number_to_print++;
 			str[0]=LOOK_AT_STORAGE_ITEM;
 			*((Uint16*)(str+1))=SDL_SwapLE16(storage_items[i].pos);
-			my_tcp_send(my_socket, str, 3);
+			my_tcp_send(str, 3);
 		}
 	}
 }
@@ -799,12 +876,14 @@ static int context_storage_handler(window_info *win, int widget_id, int mx, int
 	{
 		case ELW_CM_MENU_LEN+1: print_items(); break;
 		case ELW_CM_MENU_LEN+2: safe_strncpy(storage_text, reopen_storage_str, MAX_DESCR_LEN) ; break;
+		case ELW_CM_MENU_LEN+3: move_to_category(selected_category); break;
 	}
 	return 1;
 }
 
 void display_storage_menu()
 {
+	int storage_win = get_id_MW(MW_STORAGE);
 	int i;
 
 	/* Entropy suggested hack to determine if this is the view only "#sto" opened storage */
@@ -819,32 +898,32 @@ void display_storage_menu()
 	}
 
 	if(storage_win<=0){
-		int our_root_win = -1;
-		if (!windows_on_top) {
-			our_root_win = game_root_win;
-		}
-		storage_win=create_window(win_storage, our_root_win, 0, storage_win_x, storage_win_y, 0, 0, ELW_USE_UISCALE|ELW_WIN_DEFAULT|ELW_TITLE_NAME);
-		set_window_custom_scale(storage_win, &custom_scale_factors.storage);
+		storage_win = create_window(win_storage, (not_on_top_now(MW_STORAGE) ?game_root_win : -1), 0,
+			get_pos_x_MW(MW_STORAGE), get_pos_y_MW(MW_STORAGE), 0, 0, ELW_USE_UISCALE|ELW_WIN_DEFAULT|ELW_TITLE_NAME);
+		set_id_MW(MW_STORAGE, storage_win);
+		set_window_custom_scale(storage_win, MW_STORAGE);
 		set_window_handler(storage_win, ELW_HANDLER_DISPLAY, &display_storage_handler);
 		set_window_handler(storage_win, ELW_HANDLER_POST_DISPLAY, &post_display_storage_handler);
 		set_window_handler(storage_win, ELW_HANDLER_CLICK, &click_storage_handler);
 		set_window_handler(storage_win, ELW_HANDLER_MOUSEOVER, &mouseover_storage_handler);
 		set_window_handler(storage_win, ELW_HANDLER_KEYPRESS, (int (*)())&keypress_storage_handler );
 		set_window_handler(storage_win, ELW_HANDLER_UI_SCALE, &ui_scale_storage_handler );
+		set_window_handler(storage_win, ELW_HANDLER_FONT_CHANGE, &change_storage_font_handler);
+
+		cat_scrollbar_id = vscrollbar_add_extended(storage_win, STORAGE_SCROLLBAR_CATEGORIES, NULL, 0, 0, 0, 0, 0, 1.0, 0, 1,
+				max2i(no_storage_categories - storage_categories_display, 0));
+		items_scrollbar_id = vscrollbar_add_extended(storage_win, STORAGE_SCROLLBAR_ITEMS, NULL, 0, 0, 0, 0, 0, 1.0, 0, 1, 0);
 
-		cat_scrollbar_id = vscrollbar_add_extended(storage_win, STORAGE_SCROLLBAR_CATEGORIES, NULL, 0, 0, 0, 0, 0, 1.0, 0.77f, 0.57f, 0.39f, 0, 1, 
-				max2i(no_storage_categories - STORAGE_CATEGORIES_DISPLAY, 0));
-		items_scrollbar_id = vscrollbar_add_extended(storage_win, STORAGE_SCROLLBAR_ITEMS, NULL, 0, 0, 0, 0, 0, 1.0, 0.77f, 0.57f, 0.39f, 0, 1, 28);
-		
 		cm_add(windows_list.window[storage_win].cm_id, cm_storage_menu_str, context_storage_handler);
 		cm_add(windows_list.window[storage_win].cm_id, cm_dialog_options_str, context_storage_handler);
 		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+2, &sort_storage_categories, NULL);
-		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+3, &disable_storage_filter, NULL);
-		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+4, &autoclose_storage_dialogue, NULL);
-		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+5, &auto_select_storage_option, NULL);
+		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+3, &sort_storage_items, NULL);
+		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+4, &disable_storage_filter, NULL);
+		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+5, &autoclose_storage_dialogue, NULL);
+		cm_bool_line(windows_list.window[storage_win].cm_id, ELW_CM_MENU_LEN+6, &auto_select_storage_option, NULL);
 	} else {
 		no_storage=0;
-		
+
 		for(i = 0; i < no_storage_categories; i++)
 			storage_categories[i].name[0] = to_color_char (c_orange1);
 
@@ -855,15 +934,21 @@ void display_storage_menu()
 		vscrollbar_set_pos(storage_win, STORAGE_SCROLLBAR_ITEMS, 0);
 	}
 
+	cur_item_over = -1;
 	storage_text[0] = '\0';
 	set_window_name("", "");
+	storage_use_allowed = 1;
 
 	ui_scale_storage_handler(&windows_list.window[storage_win]);
+	check_proportional_move(MW_STORAGE);
 }
 
 void close_storagewin()
 {
-	if(storage_win >= 0 && !view_only_storage) {
-		hide_window(storage_win);
+	if(!view_only_storage)
+	{
+		hide_window_MW(MW_STORAGE);
+		clear_was_open_MW(MW_STORAGE);
+		storage_use_allowed = 0;
 	}
 }
