@@ -19,7 +19,9 @@
 #include "highlight.h"
 #include "init.h"
 #include "items.h"
+#include "knowledge.h"
 #include "lights.h"
+#include "main.h"
 #include "misc.h"
 #include "multiplayer.h"
 #include "password_manager.h"
@@ -49,27 +51,52 @@ text_message input_text_line;
 
 char last_pm_from[32];
 
-Uint32 last_server_message_time;
-int lines_to_show=0;
+static Uint32 last_server_message_time;
 
 int show_timestamp = 0;
 
 int dark_channeltext = 0;
 
+#define rate_limit_count 4
+static const char* rate_limit_msgs[rate_limit_count] = {
+	"You are too far away! Get closer!",
+	"Can't do, your target is already fighting with someone else,",
+	"You need to equip a quiver first!",
+	"You cannot access the storage from here!"
+};
+static Uint32 rate_limit_last_time[rate_limit_count] = { 0, 0, 0, 0 };
+static int rate_limit_done_one[rate_limit_count] = { 0, 0, 0, 0 };
+
+void reset_storage_rate_limit()
+{
+	rate_limit_done_one[3] = 0;
+}
+
 /* Impliment the glow perk hud indicator state. */
+
 static int glow_perk_check_state = 0;
 static int glow_perk_active = 0;
 static int glow_perk_unavailable = 1;
+static int glow_perk_do_check_when_active = 1;
 static Uint32 glow_perk_timer = 0;
 
-void check_glow_perk(void) { glow_perk_check_state = 3; }
-int glow_perk_is_active(void) { return glow_perk_active; };
+void check_glow_perk(void) { glow_perk_do_check_when_active = 1; }
+int glow_perk_is_active(void) { return glow_perk_active; }
 
 /*
  * Called each frame by the hud indicator code if the glow perk indcator is enabled.
 */
 int glow_perk_is_unavailable(void)
 {
+	/*
+	 * If this is the first time since loging, start the state check.
+	*/
+	if (glow_perk_do_check_when_active)
+	{
+		glow_perk_check_state = 3;
+		glow_perk_do_check_when_active = 0;
+	}
+
 	/*
 	 * The first #glow will be sent when a new login happens.  When the
 	 * message text arrives, another #glow will be sent after a timeout.
@@ -87,7 +114,7 @@ int glow_perk_is_unavailable(void)
 		glow_perk_timer = 0;
 	}
 	return glow_perk_unavailable;
-};
+}
 
 /*
  * Called when we receive #glow command text.
@@ -106,6 +133,7 @@ static int set_glow_status(int value)
 	glow_perk_unavailable = 0;
 	return 0;
 }
+
 /* End glow perk hud indicator state. */
 
 static int is_special_day = 0;
@@ -121,12 +149,9 @@ void clear_seen_pm_count(void) { seen_pm_count = 0; }
 
 int log_chat = LOG_SERVER;
 
-float	chat_zoom=1.0;
 FILE	*chat_log=NULL;
 FILE	*srv_log=NULL;
 
-ec_reference harvesting_effect_reference = NULL;
-
 #ifndef NEW_SOUND
 int afk_snd_warning = 0;
 #endif
@@ -199,7 +224,7 @@ void open_chat_log(){
 		char logsuffix[7];
 		strftime(logsuffix, sizeof(logsuffix), "%Y%m", l_time);
 		safe_snprintf (chat_log_file, sizeof (chat_log_file),  "chat_log_%s.txt", logsuffix);
-		safe_snprintf (srv_log_file, sizeof (srv_log_file), "srv_log_%s.txt", logsuffix); 
+		safe_snprintf (srv_log_file, sizeof (srv_log_file), "srv_log_%s.txt", logsuffix);
 	}
 	else
 	{
@@ -276,20 +301,37 @@ void write_to_log (Uint8 channel, const Uint8* const data, int len)
 	// The file we'll write to
 	fout = (channel == CHAT_SERVER && log_chat >= 3) ? srv_log : chat_log;
 
+	time (&c_time);
+	l_time = localtime (&c_time);
 	if(!show_timestamp)
 	{
 		// Start filling the buffer with the time stamp
-		time (&c_time);
-		l_time = localtime (&c_time);
-		j = strftime (str, sizeof(str), "[%H:%M:%S] ", l_time);
+		j = strftime (str, sizeof(str), "[%Y-%m-%d %H:%M:%S] ", l_time);
+		i = 0;
 	}
 	else
 	{
-		//we already have a time stamp
-		j=0;
+		// There should be a timestamp, see if it's there at the start of the line
+		for (i = 0; i < len; ++i)
+		{
+			if (!is_color(data[i]))
+				break;
+		}
+		if (i+9 < len && data[i] == '[' && data[i+9] == ']')
+		{
+			// That should be it, copy the time part from the string, but prepend the year
+			j = strftime(str, sizeof(str), "[%Y-%m-%d ", l_time);
+			++i;
+		}
+		else
+		{
+			// Cannot find a timestamp, though there should be one. Give up in confusion.
+			j = 0;
+			i = 0;
+		}
 	}
 
-	i = 0;
+	// Now copy the data to the output buffer
 	while (i < len)
 	{
 		for ( ; i < len && j < sizeof (str) - 1; i++)
@@ -349,7 +391,7 @@ void send_input_text_line (char *line, int line_len)
 	str[j] = 0;	// always a NULL at the end
 
 	len = strlen (&str[1]);
-	if (my_tcp_send (my_socket, (Uint8*)str, len+1) < len+1)
+	if (my_tcp_send((Uint8*)str, len+1) < len+1)
 	{
 		//we got a nasty error, log it
 	}
@@ -361,9 +403,9 @@ int match_emote(emote_dict *command, actor *act, int send)
 {
 	hash_entry *match;
 
-	// Try to match the input against an emote command and actor type	
+	// Try to match the input against an emote command and actor type
 	match=hash_get(emote_cmds,(void*)command->command);
-	
+
 	if(match){
 		//printf("Emote <%s> sent (%p)\n",((emote_dict*)match->item)->command,((emote_dict*)match->item)->emote);
 		//SEND emote to server
@@ -382,7 +424,7 @@ int parse_text_for_emote_commands(const char *text, int len)
 	emote_dict emote_text;
 	actor *act;
 
-	
+
 	//printf("parsing local for emotes\n");
 	//extract name
 	while(text[i]&&i<20){
@@ -392,7 +434,7 @@ int parse_text_for_emote_commands(const char *text, int len)
 			name[j]=0;
 			if(text[i]==':') i++;
 			break;
-		} 
+		}
 		i++;j++;
 	}
 
@@ -408,11 +450,11 @@ int parse_text_for_emote_commands(const char *text, int len)
 		return 1;		// Eek! We don't have an actor match... o.O
 	}
 
-	if (!(!strncasecmp(act->actor_name, name, strlen(name)) && 
+	if (!(!strncasecmp(act->actor_name, name, strlen(name)) &&
 			(act->actor_name[strlen(name)] == ' ' ||
 			act->actor_name[strlen(name)] == '\0'))){
 		//we are not saying this text, return
-		//UNLOCK_ACTORS_LISTS();			
+		//UNLOCK_ACTORS_LISTS();
 		//return 0;
 			itsme=0;
 	} else itsme=1;
@@ -432,37 +474,14 @@ int parse_text_for_emote_commands(const char *text, int len)
 				emote_text.command[j]=text[i];
 			j++;
 		}
-	} while(text[i++]);	
+	} while(text[i++]);
 	//printf("ef=%i, wf=%i, filter=>%i\n",ef,wf,emote_filter);
-	UNLOCK_ACTORS_LISTS();			
+	UNLOCK_ACTORS_LISTS();
 
 	return  ((ef==wf) ? (emote_filter):(0));
 
 }
 
-
-/* stop or restart the harvesting eye candy effect depending on the harvesting state */
-void check_harvesting_effect(void)
-{
-	/* if the harvesting effect is on but we're not harvesting, stop it */
-	if ((!now_harvesting() || !use_harvesting_eye_candy) && (harvesting_effect_reference != NULL))
-	{
-		ec_recall_effect(harvesting_effect_reference);
-		harvesting_effect_reference = NULL;
-	}
-	/* but if we are harvesting but there is no effect, start it if wanted */
-	else if (now_harvesting() && use_eye_candy && use_harvesting_eye_candy && (harvesting_effect_reference == NULL))
-	{
-		actor *act;
-		LOCK_ACTORS_LISTS();
-		act = get_actor_ptr_from_id(yourself);
-		if (act != NULL)
-			harvesting_effect_reference = ec_create_ongoing_harvesting2(act, 1.0, 1.0, (poor_man ? 6 : 10), 1.0);
-		UNLOCK_ACTORS_LISTS();
-	}
-}	
-
-
 int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 {
 	int l, idx;
@@ -505,7 +524,7 @@ int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 	ed (ttlanhil): made it check if it's a server colour. still not perfect
 	(this should have been done server-side instead of parsing the date), but safer
 	*/
-	if (from_color_char (text_to_add[0]) == c_green1 && my_strncompare(text_to_add+1,"Game Date", 9))
+	if (from_color_char (text_to_add[0]) == c_green1 && !strncasecmp(text_to_add+1, "Game Date", 9))
 	{
 		//we assume that the server will still send little-endian dd/mm/yyyy... we could make it safer by parsing the format too, but it's simpler to assume
 		const char * const month_names[] = { "Aluwia", "Seedar", "Akbar", "Zartia", "Elandra", "Viasia", "Fruitfall", "Mortia", "Carnelar", "Nimlos", "Chimar", "Vespia" };
@@ -554,7 +573,7 @@ int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 		}
 	}
 
-	if (from_color_char (text_to_add[0]) == c_green1 && my_strncompare(text_to_add+1,"Game Time", 9))
+	if (from_color_char (text_to_add[0]) == c_green1 && !strncasecmp(text_to_add+1, "Game Time", 9))
 	{
 		real_game_second = atoi(&text_to_add[18]);
 		set_real_game_second_valid();
@@ -569,36 +588,37 @@ int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 	}
 
 	if (channel == CHAT_SERVER) {
-		if (my_strncompare(text_to_add+1, "You started to harvest ", 23)) {
+		if (!strncasecmp(text_to_add+1, "You started to harvest ", 23)) {
 			strncpy(harvest_name, text_to_add+1+23, len-1-23-1);
 			harvest_name[len-1-23-1] = '\0';
 			set_now_harvesting();
 		}
-		else if ((my_strncompare(text_to_add+1, "You stopped harvesting.", 23)) ||
-			(my_strncompare(text_to_add+1, "You can't harvest while fighting (duh)!", 39)) ||
-			(my_strncompare(text_to_add+1, "You can't do that while trading!", 32)) ||
-			(my_strncompare(text_to_add+1, "You can't harvest here", 22)) ||
-			(my_strncompare(text_to_add+1, "You lack the knowledge of ", 26)) ||
-			((my_strncompare(text_to_add+1, "You need to wear ", 17) && strstr(text_to_add, "order to harvest") != NULL)) ||
-			((my_strncompare(text_to_add+1, "You need to have a ", 19) && strstr(text_to_add, "order to harvest") != NULL)))
+		else if ((!strncasecmp(text_to_add+1, "You stopped harvesting.", 23)) ||
+			(!strncasecmp(text_to_add+1, "You can't harvest while fighting (duh)!", 39)) ||
+			(!strncasecmp(text_to_add+1, "You can't do that while trading!", 32)) ||
+			(!strncasecmp(text_to_add+1, "You can't harvest here", 22)) ||
+			(!strncasecmp(text_to_add+1, "You lack the knowledge of ", 26)) ||
+			((!strncasecmp(text_to_add+1, "You need to wear ", 17) && strstr(text_to_add, "order to harvest") != NULL)) ||
+			((!strncasecmp(text_to_add+1, "You need to have a ", 19) && strstr(text_to_add, "order to harvest") != NULL)))
 		{
 			clear_now_harvesting();
 		}
-		else if (my_strncompare(text_to_add+1, "Great, you changed your password!", 33))
+		else if (!strncasecmp(text_to_add+1, "Great, you changed your password!", 33))
 		{
-			passmngr_confirm_pw_change();
+			if (!passmngr_confirm_pw_change())
+				LOG_TO_CONSOLE(c_red1, passmngr_error_str);
 		}
-		else if (my_strncompare(text_to_add+1, "Glow on!", 8))
+		else if (!strncasecmp(text_to_add+1, "Glow on!", 8))
 		{
 			if (set_glow_status(1))
 				return 0;
 		}
-		else if (my_strncompare(text_to_add+1, "Glow off!", 9))
+		else if (!strncasecmp(text_to_add+1, "Glow off!", 9))
 		{
 			if (set_glow_status(0))
 				return 0;
 		}
-		else if (my_strncompare(text_to_add+1, "You need the I glow in the dark perk in order to use this.", 58))
+		else if (!strncasecmp(text_to_add+1, "You need the I glow in the dark perk in order to use this.", 58))
 		{
 			glow_perk_unavailable = 1;
 			if (glow_perk_check_state)
@@ -610,17 +630,17 @@ int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 		else if (is_death_message(text_to_add+1)) {
 			// nothing to be done here cause all is done in the test function
 		}
-		else if (my_strncompare(text_to_add+1, "You found ", 10) && strstr(text_to_add+1, " coins.")) {
+		else if (!strncasecmp(text_to_add+1, "You found ", 10) && strstr(text_to_add+1, " coins.")) {
 			decrement_harvest_counter(atoi(text_to_add+11));
-		} 
-		else if (my_strncompare(text_to_add+1, "Send Item UIDs ", 15)) {
+		}
+		else if (!strncasecmp(text_to_add+1, "Send Item UIDs ", 15)) {
 			if (text_to_add[1+15] == '0')
 				item_uid_enabled = 0;
 			else if (text_to_add[1+15] == '1')
 				item_uid_enabled = 1;
 			printf("item_uid_enabled=%d\n", item_uid_enabled);
 		}
-		else if ((copy_next_LOCATE_ME > 0) && my_strncompare(text_to_add+1, "You are in ", 11)) {
+		else if ((copy_next_LOCATE_ME > 0) && !strncasecmp(text_to_add+1, "You are in ", 11)) {
 			char buffer[4096];
 			switch (copy_next_LOCATE_ME)
 			{
@@ -635,11 +655,11 @@ int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 			copy_next_LOCATE_ME = 0;
 			return 0;
 		}
-		else if (my_strncompare(text_to_add+1, "You see: ", 9)) {
+		else if (!strncasecmp(text_to_add+1, "You see: ", 9)) {
 			achievements_player_name(text_to_add+10, len-10);
 		}
-		else if ((my_strncompare(text_to_add+1, "You just got food poisoned!", 27)) ||
-			(my_strncompare(text_to_add+1, "Oh well, no invisibility, but we got poisoned.", 46)))
+		else if ((!strncasecmp(text_to_add+1, "You just got food poisoned!", 27)) ||
+			(!strncasecmp(text_to_add+1, "Oh well, no invisibility, but we got poisoned.", 46)))
 		{
 			increment_poison_incidence();
 		}
@@ -659,43 +679,50 @@ int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 			set_today_is_special_day();
 		}
 		else if (strstr(text_to_add+1, "You'd need a pair of binoculars to read the book from here - get closer!")) {
-			if (book_opened == -1)
+			if (!book_window_is_open())
 				return 0;
 		}
+		else if (!strncasecmp(text_to_add+1, "You are researching ", 20)) {
+			if (get_true_knowledge_info(text_to_add+1))
+				return 0;
+		}
+		else if (!strncasecmp(text_to_add+1, "Your buddy list is now empty.", 29)) {
+			clear_buddy();
+		}
 		else {
-			static Uint32 last_time[] = { 0, 0 };
-			static int done_one[] = { 0, 0 };
-			int match_index = -1;
-			if (my_strncompare(text_to_add+1, "You are too far away! Get closer!", 33))
-				match_index = 0;
-			else if (my_strncompare(text_to_add+1, "Can't do, your target is already fighting with someone else,", 60))
-				match_index = 1;
-			if (match_index > -1)
+			int match_index;
+			for (match_index = 0; match_index < rate_limit_count; ++match_index)
+			{
+				const char* msg = rate_limit_msgs[match_index];
+				if (!strncasecmp(text_to_add+1, msg, strlen(msg)))
+					break;
+			}
+			if (match_index < rate_limit_count)
 			{
 				Uint32 new_time = SDL_GetTicks();
 				clear_now_harvesting();
 				if(your_actor != NULL)
 					add_highlight(your_actor->x_tile_pos,your_actor->y_tile_pos, HIGHLIGHT_SOFT_FAIL);
 				/* suppress further messages within for 5 seconds of last */
-				if (done_one[match_index] && ((new_time - last_time[match_index]) < 5000))
+				if (rate_limit_done_one[match_index] && new_time - rate_limit_last_time[match_index] < 5000)
 					return 0;
-				done_one[match_index] = 1;
-				last_time[match_index] = new_time;
+				rate_limit_done_one[match_index] = 1;
+				rate_limit_last_time[match_index] = new_time;
 			}
 		}
 
 	} else if (channel == CHAT_LOCAL) {
-		if (now_harvesting() && my_strncompare(text_to_add+1, get_username(), strlen(get_username()))) {
+		if (now_harvesting() && !strncasecmp(text_to_add+1, get_username(), strlen(get_username()))) {
 			char *ptr = text_to_add+1+strlen(get_username());
-			if (my_strncompare(ptr, " found a ", 9)) {
+			if (!strncasecmp(ptr, " found a ", 9)) {
 				ptr += 9;
-				if (my_strncompare(ptr, "bag of gold, getting ", 21)) {
+				if (!strncasecmp(ptr, "bag of gold, getting ", 21)) {
 					decrement_harvest_counter(atoi(ptr+21));
 				} else if (!strstr(ptr, " could not carry ")) {
 					decrement_harvest_counter(1);
 				}
 			}
-		} else if (my_strncompare(text_to_add+1, "(*) ", 4)) {
+		} else if (!strncasecmp(text_to_add+1, "(*) ", 4)) {
 			increment_summon_counter(text_to_add+1+4);
 			if (summoning_filter) return 0;
 		}
@@ -704,8 +731,8 @@ int filter_or_ignore_text (char *text_to_add, int len, int size, Uint8 channel)
 	catch_counters_text(text_to_add+1);
 
 	/* put #mpm in a popup box, on top of all else */
-	if ((channel == CHAT_MODPM) && (my_strncompare(text_to_add+1, "[Mod PM from", 12))) {
-		display_server_popup_win(text_to_add);
+	if ((channel == CHAT_MODPM) && (!strncasecmp(text_to_add+1, "[Mod PM from", 12))) {
+		display_server_popup_win((const unsigned char*)text_to_add);
 	}
 
 	// look for astrology messages
@@ -1171,124 +1198,24 @@ void put_colored_text_in_buffer (Uint8 color, Uint8 channel, const Uint8 *text_t
 	return;
 }
 
-void put_small_text_in_box_zoomed (const Uint8 *text_to_add, int len, int pixels_limit, char *buffer, float text_zoom)
-{
-	put_small_colored_text_in_box_zoomed (c_grey1, text_to_add, len, pixels_limit, buffer, text_zoom);
-}
-
-void put_small_colored_text_in_box_zoomed (Uint8 color, const Uint8 *text_to_add, int len, int pixels_limit, char *buffer, float text_zoom)
-{
-	int i;
-	Uint8 cur_char;
-	int last_text = 0;
-	int x_chars_limit;
-
-	// force the color
-	if (!is_color (text_to_add[0]))
-		buffer[last_text++] = to_color_char (color);
-
-	//see if the text fits on the screen
-	x_chars_limit = pixels_limit / (int)(0.5 + SMALL_FONT_X_LEN * text_zoom);
-	if (len <= x_chars_limit)
-	{
-		for (i = 0; i < len; i++)
-		{
-			cur_char = text_to_add[i];
-
-			if (cur_char == '\0')
-				break;
-
-			buffer[last_text++] = cur_char;
-		}
-		if (last_text > 0 && buffer[last_text-1] != '\n')
-			buffer[last_text++] = '\n';
-		buffer[last_text] = '\0';
-	}
-	else //we have to add new lines to our text...
-	{
-		int k;
-		int new_line_pos = 0;
-		char semaphore = 0;
-		Uint8 current_color = to_color_char (color);
-
-		// go trought all the text
-		for (i = 0; i < len; i++)
-		{
-			if (!semaphore && new_line_pos + x_chars_limit < len) //don't go through the last line
-			{
-				//find the closest space from the end of this line
-				//if we have one really big word, then parse the string from the
-				//end of the line backwards, untill the beginning of the line +2
-				//the +2 is so we avoid parsing the ": " thing...
-				for (k = new_line_pos + x_chars_limit - 1; k > new_line_pos + 2; k--)
-				{
-					cur_char = text_to_add[k];
-					if (k > len) continue;
-					if (cur_char == ' ' || cur_char == '\n')
-					{
-						k++; // let the space on the previous line
-						break;
-					}
-				}
-				if (k == new_line_pos + 2)
-					new_line_pos += x_chars_limit;
-				else
-					new_line_pos = k;
-				semaphore = 1;
-			}
-
-			cur_char = text_to_add[i];
-			if (cur_char == '\0') break;
-
-			if (is_color (cur_char)) // we have a color, save it
-			{
-				current_color = cur_char;
-				if (last_text > 0 && is_color (buffer[last_text-1]))
-					last_text--;
-			}
-			else if (cur_char == '\n')
-			{
-				new_line_pos = i;
-			}
-
-			if (i == new_line_pos)
-			{
-				buffer[last_text++] = '\n';
-				// don't add color codes after the last newline
-				if (i < len-1)
-					buffer[last_text++] = current_color;
-				semaphore = 0;
-			}
-			//don't add another new line, if the current char is already a new line...
-			if (cur_char != '\n')
-				buffer[last_text++] = cur_char;
-
-		}
-		// don't add extra newlines if there already is one
-		if (last_text > 0 && buffer[last_text-1] != '\n')
-			buffer[last_text++] = '\n';
-		buffer[last_text] = '\0';
-	}
-}
-
-
 // find the last lines, according to the current time
 int find_last_lines_time (int *msg, int *offset, Uint8 filter, int width)
 {
 	// adjust the lines_no according to the time elapsed since the last message
 	if ( (cur_time - last_server_message_time) / 1000 > 3)
 	{
-		if (lines_to_show > 0)
-			lines_to_show--;
+		dec_lines_to_show();
 		last_server_message_time = cur_time;
 	}
-	if (lines_to_show <= 0) return 0;
+	if (get_lines_to_show() <= 0) return 0;
 
-	return find_line_nr (get_total_nr_lines(), get_total_nr_lines() - lines_to_show, filter, msg, offset, chat_zoom, width);
+	find_line_nr(get_total_nr_lines(), get_total_nr_lines() - get_lines_to_show(),
+		filter, msg, offset, CHAT_FONT, 1.0, width);
+	return 1;
 }
 
-
-int find_line_nr (int nr_lines, int line, Uint8 filter, int *msg, int *offset, float zoom, int width)
+void find_line_nr(int nr_lines, int line, Uint8 filter, int *msg, int *offset,
+	font_cat font, float zoom, int width)
 {
 	int line_count = 0, lines_no = nr_lines - line;
 	int imsg, ichar;
@@ -1298,7 +1225,7 @@ int find_line_nr (int nr_lines, int line, Uint8 filter, int *msg, int *offset, f
 	if ( imsg<0 ) {
 		/* No data in buffer */
 		*msg = *offset = 0;
-		return 1;
+		return;
 	}
 	do
 	{
@@ -1321,7 +1248,7 @@ int find_line_nr (int nr_lines, int line, Uint8 filter, int *msg, int *offset, f
 				// happening.
 				break;
 
-			rewrap_message(&display_text_buffer[imsg], zoom, width, NULL);
+			rewrap_message(&display_text_buffer[imsg], font, zoom, width, NULL);
 
 			for (ichar = display_text_buffer[imsg].len - 1; ichar >= 0; ichar--)
 			{
@@ -1332,7 +1259,7 @@ int find_line_nr (int nr_lines, int line, Uint8 filter, int *msg, int *offset, f
 					{
 						*msg = imsg;
 						*offset = ichar+1;
-						return 1;
+						return;
 					}
 				}
 			}
@@ -1342,7 +1269,7 @@ int find_line_nr (int nr_lines, int line, Uint8 filter, int *msg, int *offset, f
 			{
 				*msg = imsg;
 				*offset = 0;
-				return 1;
+				return;
 			}
 		}
 
@@ -1352,7 +1279,6 @@ int find_line_nr (int nr_lines, int line, Uint8 filter, int *msg, int *offset, f
 
 	*msg = 0;
 	*offset = 0;
-	return 1;
 }
 
 void clear_display_text_buffer ()
@@ -1375,21 +1301,22 @@ void clear_display_text_buffer ()
 	}
 }
 
-int rewrap_message(text_message * msg, float zoom, int width, int * cursor)
+int rewrap_message(text_message* msg, font_cat cat, float text_zoom, int width, int *cursor)
 {
 	int nlines;
-	float max_line_width = 0;
 
 	if (msg == NULL || msg->data == NULL || msg->deleted)
 		return 0;
 
-	if (msg->wrap_width != width || msg->wrap_zoom != zoom)
+	if (msg->wrap_width != width || msg->wrap_zoom != text_zoom)
 	{
- 		nlines = reset_soft_breaks(msg->data, msg->len, msg->size, zoom, width, cursor, &max_line_width);
+		int max_line_width = 0;
+ 		nlines = reset_soft_breaks((unsigned char*)msg->data, msg->len, msg->size,
+			cat, text_zoom, width, cursor, &max_line_width);
 		msg->len = strlen(msg->data);
 		msg->wrap_lines = nlines;
 		msg->wrap_width = width;
-		msg->wrap_zoom = zoom;
+		msg->wrap_zoom = text_zoom;
 		msg->max_line_width = max_line_width;
 	} else {
 		nlines = msg->wrap_lines;
