@@ -187,6 +187,7 @@ extern char 	tt_walk[30],
 		tt_use_witem[30], 
 		tt_trade[30], 
 		tt_attack[30],
+		tt_chat[30],
 		tt_inventory[30],
 		tt_spell[30],
 		tt_manufacture[30],
@@ -263,6 +264,16 @@ extern char
 		invalid_spell_string_str[40],
 		command_too_long_str[40],
 		item_list_learn_cat_str[90],
+		item_uid_help_str[80],
+		item_use_not_unique_str[80],
+		item_use_get_failed_str[80],
+		item_info_load_failed_str[80],
+		commands_help_prefix_str[20],
+		commands_search_prefix_str[40],
+		commands_help_not_loaded_str[60],
+		commands_help_not_recognsed_str[40],
+		commands_help_description_help_str[100],
+		commands_help_search_help_str[100],
 		cmd_ignores[20],
 		cmd_ignore[20],
 		cmd_unignore[20],
@@ -291,6 +302,8 @@ extern char
 		cmd_relogin[20],
 		cmd_disconnect[20],
 		cmd_disco[20],
+		cmd_summon_attack_short[10],
+		cmd_summon_attack[20],
 		help_cmd_markpos_str[50],
 		location_info_str[40],
 		knowledge_cmd_str[40],
@@ -306,6 +319,9 @@ extern char
 		urlwin_clear_str[30],
 		/*draw_scene.c*/
 		low_framerate_str[100],
+		/* elconfig */
+		multiselect_find_prompt_str[40],
+		multiselect_find_help_str[80],
 		/* encyclopedia */
 		encycl_search_prompt_str[25],
 		/*filter.c*/
@@ -316,12 +332,16 @@ extern char
 		ranginglock_disabled_str[50],
 		close_click_targetting_on_str[50],
 		close_click_targetting_off_str[50],
-		/*gl_init.c*/
+		/* gl_init.c and window size strings */
 		window_size_adjusted_str[50],
+		reset_res_str[50],
+		set_res_str[50],
+		save_res_str[50],
+		show_res_str[50],
 		/*hud.c*/
 		no_open_on_trade[100], 
 		stats_scroll_help_str[100],
-		cm_action_points_str[30],
+		cm_statsbar_str[70],
 		hud_timer_cm_str[100],
 		hud_timer_popup_title_str[25],
 		/* hud_indicators.cpp */
@@ -333,6 +353,7 @@ extern char
 		messages_indicator_str[60],
 		ranginglock_indicator_str[70],
 		glowperk_indicator_str[100],
+		summon_attack_indicator_str[100],
 		/*ignore.c*/
 		no_ignores_str[50],
 		ignores_str[50],
@@ -342,6 +363,9 @@ extern char
 		login_rules_str[120],
 		passmngr_enabled_str[70],
 		passmngr_disabled_str[70],
+		passmngr_remember_details_str[70],
+		passmngr_error_str[70],
+		password_format_str[150],
 		show_passwords_str[30],
 		login_select_window_str[30],
 		/*items.c*/
@@ -392,6 +416,9 @@ extern char
 		minute_str[15],
 		idle_str[15],
 		knowledge_read_book[15],
+		knowledge_read_book_tag[15],
+		knowledge_unread_book_tag[15],
+		knowledge_reading_book_tag[15],
 		knowledge_param_read[15],
 		knowledge_param_unread[15],
 		knowledge_param_total[15],
@@ -455,6 +482,7 @@ extern char
 		char_help[200],
 		invalid_pass[30],
 		show_password[30],
+		remember_details_str[30],
 		hide_password[30],
 		char_done[15],
 		char_back[15],
@@ -504,7 +532,7 @@ extern char
 		cm_title_menu_str[150],
 		cm_title_help_str[50],
 		cm_items_menu_str[190],
-		cm_storage_menu_str[90],
+		cm_storage_menu_str[110],
 		cm_astro_menu_str[80],
 		cm_ranging_menu_str[50],
 		cm_dialog_options_str[80],
@@ -520,6 +548,11 @@ extern char
 		cm_recipe_menu_str[100],
 		cm_manuwin_menu_str[50],
 		cm_encycl_base_str[150],
+		cm_options_default_str[50],
+		cm_options_initial_str[50],
+#ifdef JSON_FILES
+		cm_options_per_character_str[50],
+#endif
 		/* user_menus.cpp */
 		um_invalid_command_str[50],
 		um_invalid_line_str[50],
@@ -540,19 +573,27 @@ extern char
 		questlog_deleted_str[20],
 		/* item lists */
 		cm_item_list_selected_str[40],
-		cm_item_list_names_str[160],
+		cm_item_list_names_str[200],
 		item_list_use_help_str[40],
 		item_list_pickup_help_str[40],
 		item_list_edit_help_str[40],
 		item_list_add_help_str[40],
 		item_list_drag_help_str[40],
 		item_list_create_help_str[40],
-		item_list_magic_str[80],
 		item_list_find_str[20],
 		item_list_find_help_str[40],
 		item_list_find_help_disabled_str[40],
 		/* new_character.c */
-		use_appropriate_name[500];
+		use_appropriate_name[500],
+		/* summoning attack command */
+		no_attack_str[50],
+		attack_my_opponent_str[50],
+		do_not_attack_my_opponent_str[50],
+		attack_only_summoned_str[50],
+		do_not_attack_summoned_str[50],
+		attack_at_will_str[50],
+		summon_attack_set_mode_str[50],
+		summon_attack_help_str[50];
 #endif  //DOXYGEN_SKIP_THIS
 #endif  //ELC
 
@@ -586,6 +627,20 @@ extern char	reg_error_str[15],
 		cache_size_str[20],
 		/* cal.c */
 		no_animation_err_str[30],
+#ifdef USE_SSL
+		/* connection.cpp */
+		warning_str[30],
+		hostname_mismatch_str[500],
+		unverified_certificate_str[400],
+		close_connection_str[30],
+		continue_str[30],
+		no_encryption_support_str[100],
+		no_encryption_response_str[100],
+		now_encrypted_str[100],
+		encryption_failed_str[100],
+		cert_verification_err_str[100],
+		send_failed_str[100],
+#endif // USE_SSL
 		/*console.c*/
 		invalid_location_str[30],
 		name_too_long[75], 
@@ -609,6 +664,7 @@ extern char	reg_error_str[15],
 		video_card_str[20],
 		video_vendor_str[20],
 		opengl_version_str[20],
+		context_version_str[30],
 		supported_extensions_str[30],
 		help_request_str[20],
 		help_cmd_str[10],
@@ -680,7 +736,6 @@ extern char	reg_error_str[15],
 		invalid_video_mode[75], 
 		failed_sdl_net_init[30],
 		failed_sdl_timer_init[30],
-		cant_read_elini[50],
 		must_use_tabs[80],
 		init_opengl_str[35],
 		init_random_str[35],
@@ -799,6 +854,7 @@ extern char	reg_error_str[15],
 		snd_media_music_stopped[50],
 		snd_media_ogg_info_noartist[50],
 		snd_media_ogg_info[50], //sound.c
+		totalxp_str[20], //session.c
 		stat_no_invalid[50], //stats.c
 		timer_lagging_behind[100], //timers.c
 		/*spells.c*/
