@@ -26,6 +26,7 @@
 #include "hud.h"
 #include "interface.h"
 #include "main.h"
+#include "misc.h"
 #include "translate.h"
 #include "io/elpathwrapper.h"
 #include "threads.h"
@@ -86,7 +87,7 @@ void init_update()
 		download_cur_file= NULL;
 		download_cur_md5= NULL;
 	}
-	
+
 	// load the server list
 	num_update_servers= 0;
 	update_server[0]= '\0';
@@ -96,11 +97,11 @@ void init_update()
 	} else {
 		char    buffer[1024];
 		char	*ptr;
-		
+
 		ptr= fgets(buffer, sizeof(buffer), fp);
 		while(ptr && !ferror(fp) && num_update_servers < sizeof(update_servers)){
 			int len= strlen(buffer);
-			
+
 			// is this line worth handling?
 			if(len > 6 && *buffer > ' ' && *buffer != '#'){
 				while(isspace(buffer[len-1])){
@@ -123,7 +124,7 @@ void init_update()
 		update_servers[0]= "";
 		return;
 	}
-	
+
 	// start the process
 	if(download_mutex){
 		strcpy(files_lst, "files.lst");
@@ -174,7 +175,7 @@ static void do_handle_update_download(struct http_get_struct *get)
 			} else {
 				LOG_ERROR("Unable to finish %s processing (%d)", files_lst, errno);
 			}
-			
+
 			// and go back to normal processing
 			return;
 		}
@@ -183,7 +184,7 @@ static void do_handle_update_download(struct http_get_struct *get)
 		assert(get->thread_index<MAX_THREADS);
 		SDL_WaitThread(thread_list[get->thread_index], NULL);
 		thread_list[get->thread_index] = NULL;
-		
+
 		//no, we need to free the memory and try again
 		if(get->fp){
 			fclose(get->fp);
@@ -195,7 +196,7 @@ static void do_handle_update_download(struct http_get_struct *get)
 	if(update_attempt_count++ < 3){
 		char	filename[1024];
 		FILE    *fp;
-		
+
 		// select a server
 		if(num_update_servers > 1){
 			int num;
@@ -289,7 +290,7 @@ static int do_threaded_update(void *ptr)
 		char	filename[1024];
 		char    asc_md5[256];
 		Uint8	md5[16];
-		
+
 		// parse the line
 		filename[0]= '\0';
 		asc_md5[0]= '\0';
@@ -307,7 +308,7 @@ static int do_threaded_update(void *ptr)
 				// convert the md5 to binary
 				for(i=0; i<16; i++){
 					int val;
-					
+
 					strncpy(buffer, asc_md5+i*2, 2);
 					buffer[2]= '\0';
 					sscanf(buffer, "%x", &val);
@@ -321,7 +322,7 @@ static int do_threaded_update(void *ptr)
 				}
 			}
 		}
-			
+
 		// read the next line, if any
 		buf= gzgets(fp, buffer, 1024);
 	}
@@ -347,7 +348,7 @@ void add_to_download(const char *filename, const Uint8 *md5)
 		download_MD5s[download_queue_size]= calloc(1, 16);
 		memcpy(download_MD5s[download_queue_size], md5, 16);
 		download_queue_size++;
-		
+
 		// start a thread if one isn't running
 		if(!download_cur_file){
 			char	buffer[1024];
@@ -385,7 +386,7 @@ void handle_file_download(struct http_get_struct *get)
 	if(!get){   // huh? what are you doing?
 		return;
 	}
-	
+
 	// lock the mutex
 	CHECK_AND_LOCK_MUTEX(download_mutex);
 	if(get->status == 0){
@@ -425,7 +426,7 @@ void handle_file_download(struct http_get_struct *get)
 
 	// now, release everything
 	free(get);
-	
+
 	// lock the mutex
 	CHECK_AND_LOCK_MUTEX(download_mutex);
 	if(download_queue_size > 0 && !download_cur_file){
@@ -511,7 +512,7 @@ static int http_get_file_thread_handler(void *specs){
 	spec->status= http_get_file(spec->server, spec->path, spec->fp);
 	fclose(spec->fp);
 	spec->fp= NULL;
-	
+
 	// check to see if the file is correct
 	if (spec->md5 && *spec->md5)
 	{
@@ -561,9 +562,9 @@ static int http_get_file(char *server, char *path, FILE *fp)
 	if(!http_sock){
 		return(2);  // failed to open the socket
 	}
-	
-	// send the GET request, try to avoid ISP caching	
-	
+
+	// send the GET request, try to avoid ISP caching
+
 	safe_snprintf(message, sizeof(message), "GET %s HTTP/1.1\r\nHost: %s\r\nCONNECTION:CLOSE\r\nCACHE-CONTROL:NO-CACHE\r\nREFERER:%s\r\nUSER-AGENT:AUTOUPDATE %s\r\n\r\n", path, server, "autoupdate", FILE_VERSION);
 	//printf("%s",message);
 	len= strlen(message);
@@ -577,7 +578,7 @@ static int http_get_file(char *server, char *path, FILE *fp)
 	while(len > 0)
 	{
 		char buf[1024];
-		
+
 		memset(buf, 0, 1024);
 		// get a packet
 		len= SDLNet_TCP_Recv(http_sock, buf, 1024);
@@ -585,7 +586,7 @@ static int http_get_file(char *server, char *path, FILE *fp)
 		if(!got_header)
 		{
 			int i;
-			
+
 			// check for http status
 			sscanf(buf, "HTTP/%*s %i ", &http_status);
 
@@ -614,7 +615,7 @@ static int http_get_file(char *server, char *path, FILE *fp)
 		}
 	}
 	SDLNet_TCP_Close(http_sock);
-	
+
 	if(http_status != 200) {
 		if(http_status != 0){
 			return(http_status);
@@ -637,12 +638,13 @@ static void draw_update_interface (window_info *win)
 	char str[200];
 	float window_ratio = (float) win->len_y / 480.0f;
 
-	draw_string_zoomed ((win->len_x - (strlen(update_complete_str) * win->default_font_len_x)) / 2, 200 * window_ratio, (unsigned char*)update_complete_str, 0, win->current_scale);
+	draw_string_zoomed_centered(win->len_x/2, 200 * window_ratio,
+		(const unsigned char*)update_complete_str, 0, win->current_scale);
 
 /*	Possibly use this box to display the list of files updated?
-	
+
 	glDisable(GL_TEXTURE_2D);
-	glColor3f(0.77f,0.57f,0.39f);
+	glColor3fv(gui_color);
 	glBegin(GL_LINES);
 	glVertex3i(diff + 30 * window_ratio, 50 * window_ratio, 0);
 	glVertex3i(len_x - (diff + 30 * window_ratio) - 20, 50 * window_ratio, 0);
@@ -653,7 +655,7 @@ static void draw_update_interface (window_info *win)
 	glEnd();
 	glEnable(GL_TEXTURE_2D);
 */
-	
+
 	if (update_countdown != 0)
 	{
 		safe_snprintf (str, sizeof(str), client_restart_countdown_str, update_countdown);
@@ -663,9 +665,10 @@ static void draw_update_interface (window_info *win)
 		safe_strncpy (str, client_restarting_str, sizeof(str));
 		exit_now = 1;
 	}
-		
-	draw_string_zoomed ((win->len_x - (strlen (str) * win->default_font_len_x)) / 2, win->len_y - (200 * window_ratio), (unsigned char*)str, 0, win->current_scale);
-	
+
+	draw_string_zoomed_centered(win->len_x/2, win->len_y - (200 * window_ratio),
+		(const unsigned char*)str, 0, win->current_scale);
+
 	glDisable (GL_ALPHA_TEST);
 #ifdef OPENGL_TRACE
 CHECK_GL_ERRORS();
@@ -721,7 +724,8 @@ void create_update_root_window (int width, int height, int time)
 			return;
 		win = &windows_list.window[update_root_win];
 
-		update_root_restart_id = button_add_extended (update_root_win, update_root_restart_id, NULL, 0, 0, 0, 0, 0, win->current_scale, 1.0f, 1.0f, 1.0f, restart_now_label);
+		update_root_restart_id = button_add_extended (update_root_win, update_root_restart_id, NULL, 0, 0, 0, 0, 0, win->current_scale, restart_now_label);
+		widget_set_color(update_root_win, update_root_restart_id, 1.0f, 1.0f, 1.0f);
 		widget_move(update_root_win, update_root_restart_id,
 			(width - widget_get_width(update_root_win, update_root_restart_id)) /2,
 			height - 2 * widget_get_height(update_root_win, update_root_restart_id));
@@ -731,6 +735,6 @@ void create_update_root_window (int width, int height, int time)
 		set_window_handler (update_root_win, ELW_HANDLER_SHOW, &show_update_handler);
 		widget_set_OnClick(update_root_win, update_root_restart_id, &click_update_root_restart);
 	}
-	
+
 	update_countdown = time;
 }
