@@ -6,8 +6,11 @@
 #ifndef	__WIDGETS_H
 #define	__WIDGETS_H
 
+typedef struct select_info select_info;
+
 #include <SDL_types.h>
 #include <SDL_keycode.h>
+#include "font.h"
 #include "text.h"
 
 #ifdef __cplusplus
@@ -15,19 +18,23 @@ extern "C" {
 #endif
 
 typedef struct {
-	Sint8 label[64];
+	unsigned char label[64];
 	int content_id;
 	Uint16 tag_width;
+	Uint16 min_tag_width;
 	float label_r, label_g, label_b;
 	char closable;
 } tab;
 
 typedef struct {
-	int tag_height, button_size;
+	int tag_height, button_size, tabs_right_margin;
 	int nr_tabs, max_tabs, cur_tab, tab_offset, tab_last_visible;
 	tab *tabs;
 } tab_collection;
 
+// Forward declaration
+struct wl;
+
 // The purpose of this implementation if to remove the need to edit
 // the implementation of the widgets to add a new client widget. It
 // also allows clients to be dynamically created and populated.
@@ -42,6 +49,9 @@ struct WIDGET_TYPE {
 	int (*key)();
 	int (*destroy)();
 	int (*move)();
+	int (*font_change)();
+	int (*paste)();
+	int (*color_change)(struct wl*, float, float, float);
     // We can conceivably store other generic info here too
 } ;
 
@@ -65,6 +75,7 @@ typedef struct wl{
 	Uint32 Flags;  /*!< Status flags... visible, enabled, etc */
 	float size;    /*!< Size of text, image, etc */
 	float r, g, b; /*!< Associated color */
+	font_cat fcat; /*!< Font category for drawing text contents in */
     /*! @} */
 
 	/*! \name The specific widget handlers */
@@ -77,6 +88,7 @@ typedef struct wl{
 	int (*OnResize)();
 	int (*OnKey)();
 	int (*OnDestroy)();
+	int (*OnFontChange)();
 	/*! \} */
 
 	void *widget_info; /*!< Pointer to specific widget data */
@@ -96,22 +108,34 @@ typedef struct wl{
  * \name	Flags for the buttons
  */
 /*! \{ */
-#define BUTTON_ACTIVE 0x400
-#define BUTTON_SQUARE 0x800
+#define BUTTON_ACTIVE          0x0400
+#define BUTTON_SQUARE          0x0800
+#define BUTTON_VCENTER_CONTENT 0x1000
+/*! \} */
 
 /*!
  * \name	Flags for the text field
  */
 /*! \{ */
-#define TEXT_FIELD_BORDER	0x01
-#define TEXT_FIELD_EDITABLE	0x02
-#define TEXT_FIELD_NO_KEYPRESS	0x04
-#define TEXT_FIELD_CAN_GROW	0x08
-#define TEXT_FIELD_SCROLLBAR	0x10
-#define TEXT_FIELD_IGNORE_RETURN 0x20
+#define TEXT_FIELD_BORDER          0x01
+#define TEXT_FIELD_EDITABLE        0x02
+#define TEXT_FIELD_NO_KEYPRESS     0x04
+#define TEXT_FIELD_CAN_GROW        0x08
+#define TEXT_FIELD_SCROLLBAR       0x10
+#define TEXT_FIELD_IGNORE_RETURN   0x20
 #define TEXT_FIELD_MOUSE_EDITABLE 0x200
 /*! \} */
 
+/*!
+ * \name Flags for the password field
+ */
+/*! \{ */
+#define PWORD_FIELD_NO_KEYPRESS TEXT_FIELD_NO_KEYPRESS
+#define PWORD_FIELD_NO_BORDER   0x2000
+#define PWORD_FIELD_DRAW_CURSOR 0x4000
+#define PWORD_FIELD_NO_CURSOR   0x8000
+/*! \} */
+
 #define TF_BLINK_DELAY 500
 
 /*!
@@ -125,11 +149,11 @@ typedef struct
 /*!
  * Contains selection information for text_field.
  */
-typedef struct
+struct select_info
 {
 	text_field_line* lines;
 	int sm, sc, em, ec;
-} select_info;
+};
 
 /*!
  * Checks if selection is empty.
@@ -148,7 +172,7 @@ typedef struct
 {
 	int msg, offset;
 	int cursor, cursor_line;
-	int buf_size, buf_fill;
+	int buf_size;
 	int nr_lines, nr_visible_lines;
 	int update_bar;
 	int scroll_id;
@@ -185,21 +209,17 @@ typedef struct {
  * \param   OnInit The function used for initiating the label
  * \param   x The x location
  * \param   y The y location
- * \param   lx The width 
+ * \param   lx The width
  * \param   ly The height
  * \param   Flags The flags
  * \param   size The text size
- * \param   r (0<=r<=1)
- * \param   g (0<=g<=1)
- * \param   b (0<=b<=1)
  * \param   type The widget type
  * \param   T Pointer to specific widget data
  * \param   S Pointer to specific implementation info
- * \retval int Returns the new widgets unique ID 
+ * \retval int Returns the new widgets unique ID
  */
 Uint32 widget_add (int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y,
-	Uint16 lx, Uint16 ly, Uint32 Flags, float size, float r, float g, float b,
-	const struct WIDGET_TYPE *type, void *T, void *S);
+	Uint16 lx, Uint16 ly, Uint32 Flags, float size, const struct WIDGET_TYPE *type, void *T, void *S);
 
 /*!
  * \ingroup	widgets
@@ -299,6 +319,20 @@ int widget_set_OnMouseover(int window_id, Uint32 widget_id, int (*handler)());
  * \sa widget_find
  */
 int widget_set_OnKey ( int window_id, Uint32 widget_id, int (*handler)() );
+/*!
+ * \ingroup	widgets
+ * \brief 	Sets the widget's destroy handler
+ *
+ * Sets \a handler to be executed on destruction of the widget.
+ *
+ * \param   	window_id The location of the window in the windows_list.window[] array
+ * \param   	widget_id The widget's unique ID
+ * \param   	handler A function pointer to the handler.
+ * \retval int  	Returns 1 on succes or 0 on failure (when the widget was not found in the given window)
+ *
+ * \sa widget_find
+ */
+int widget_set_OnDestroy (int window_id, Uint32 widget_id, int (*handler)());
 
 /*!
  * \ingroup	widgets
@@ -427,18 +461,48 @@ int widget_set_size(int window_id, Uint32 widget_id, float size);
  * \ingroup	widgets
  * \brief 	Sets the widget colour
  *
- * 		Finds the widget in the given window and sets the r g b colour.
+ * 		Finds the widget in the given window and sets the r g b foreground colour.
  *
  * \param   	window_id The location of the window in the windows_list.window[] array
  * \param   	widget_id The widget's unique ID
  * \param   	r (0<=r<=1)
  * \param   	g (0<=g<=1)
  * \param   	b (0<=b<=1)
- * \retval int  	Returns 1 on succes or 0 on failure (when the widget was not found in the given window)
+ * \retval int  Returns 1 on succes or 0 on failure (when the widget was not found in the given window)
  *
  * \sa widget_find
  */
 int widget_set_color(int window_id, Uint32 widget_id, float r, float g, float b);
+/*!
+ * \ingroup	widgets
+ * \brief 	Unsets the widget colour
+ *
+ * Finds the widget in the given window and removes the color. The widget will be drawn in
+ * the last color used.
+ *
+ * \param   	window_id The location of the window in the windows_list.window[] array
+ * \param   	widget_id The widget's unique ID
+ * \retval int  Returns 1 on succes or 0 on failure (when the widget was not found in the given window)
+ *
+ * \sa widget_find
+ */
+static __inline__ int widget_unset_color(int window_id, Uint32 widget_id)
+{
+	return widget_set_color(window_id, widget_id, -1.0f, -1.0f, -1.0f);
+}
+
+/*!
+ * \ingroup widgets
+ * \brief Set the font category
+ *
+ * Set the font category for the textual elements in this widget to \a fcat
+ *
+ * \param window_id The location of the window in the windows_list.window[] array
+ * \param widget_id The widget's unique ID
+ * \param fcat      The new font category for this widget
+ * \return 1 on succes or 0 on failure
+ */
+int widget_set_font_cat(int window_id, int widget_id, font_cat fcat);
 
 /*!
  * \ingroup	widgets
@@ -487,11 +551,11 @@ int widget_get_height (int window_id, Uint32 widget_id);
  * \param   	g (0<=g<=1)
  * \param   	b (0<=b<=1)
  * \param   	text The text
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa lable_add
  */
-int label_add_extended(int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y, Uint32 Flags, float size, float r, float g, float b, const char *text);
+int label_add_extended(int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y, Uint32 Flags, float size, const char *text);
 
 /*!
  * \ingroup	labels
@@ -504,24 +568,12 @@ int label_add_extended(int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uin
  * \param   	text The text
  * \param   	x The x position
  * \param   	y The y position
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa		label_add_extended
  */
 int label_add(int window_id, int (*OnInit)(), const char *text, Uint16 x, Uint16 y);
 
-/*!
- * \ingroup	labels
- * \brief	Draws a label
- *
- * 		Draws the label given by the widget.
- *
- * \param   	W The widget that is to be drawn
- * \retval int  	Returns true
- * \callgraph
- */
-int label_draw(widget_list *W);
-
 /*!
  * \ingroup	labels
  * \brief 	Sets the text of the given widget
@@ -530,7 +582,7 @@ int label_draw(widget_list *W);
  *
  * \param   	window_id The location of the window in the windows_list.window[] array
  * \param   	widget_id The widget's unique ID
- * \param   	text The new text 
+ * \param   	text The new text
  * \retval int  	Returns 1 on succes, 0 on failure (if the widget is not found in the given window)
  *
  * \sa widget_find
@@ -556,20 +608,17 @@ int label_set_text(int window_id, Uint32 widget_id, const char *text);
  * \param   	ly The height
  * \param   	Flags The flags
  * \param   	size The text size
- * \param   	r (0<=r<=1)
- * \param   	g (0<=g<=1)
- * \param   	b (0<=b<=1)
  * \param   	id The offset in the texture_cache
  * \param   	u1 The start u texture coordinate
  * \param   	v1 The start v texture coordinate
  * \param   	u2 The end u texture coordinate
  * \param   	v2 The end v texture coordinate
  * \param   	alpha The alpha value for the image
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa image_add
  */
-int image_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, float r, float g, float b, int id, float u1, float v1, float u2, float v2, float alpha);
+int image_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, int id, float u1, float v1, float u2, float v2, float alpha);
 
 /*!
  * \ingroup	images
@@ -594,33 +643,6 @@ int image_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Ui
  */
 int image_add(int window_id, int (*OnInit)(), int id, Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, float u1, float v1, float u2, float v2);
 
-/*!
- * \ingroup	images
- * \brief 	Draws the image widget 
- * 
- * 		Draws an image widget as given by the widget *.
- *
- * \param   	W A pointer to the widget that should be drawn
- * \retval int  	Returns true
- * \callgraph
- */
-int image_draw(widget_list *W);
-
-/*!
- * \ingroup	images
- * \brief 	Sets the texture ID
- *
- * 		The function sets the texture ID (or rather, the location in the texture_cache) of the given widget.
- *
- * \param   	window_id The location of the window in the windows_list.window[] array
- * \param   	widget_id The widgets unique ID
- * \param   	id The location in the texture_cache array
- * \retval int  	Returns 1 on succes, 0 on failure (if the widget is not found in the given window)
- *
- * \sa widget_find
- */
-int image_set_id(int window_id, Uint32 widget_id, int id);
-
 /*!
  * \ingroup 	images
  * \brief 	Sets the UV coordinates of the image widget
@@ -658,15 +680,12 @@ int image_set_uv(int window_id, Uint32 widget_id, float u1, float v1, float u2,
  * \param   	ly The height
  * \param   	Flags The flags
  * \param   	size The text size
- * \param   	r (0<=r<=1)
- * \param   	g (0<=g<=1)
- * \param   	b (0<=b<=1)
  * \param   	checked Specifies if the widget is checked or not
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa checkbox_add
  */
-int checkbox_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, float r, float g, float b, int *checked);
+int checkbox_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, int *checked);
 
 /*!
  * \ingroup	checkboxes
@@ -687,18 +706,6 @@ int checkbox_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x,
  */
 int checkbox_add(int window_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, int *checked);
 
-/*!
- * \ingroup	checkboxes
- * \brief 	Draws a checkbox
- *
- * 		Draws the checkbox pointed to by *W.
- *
- * \param   	W The widget you wish to draw
- * \retval int  	Returns true
- * \callgraph
- */
-int checkbox_draw(widget_list *W);
-
 /*!
  * \ingroup	checkboxes
  * \brief 	Checks if the given checkbox is checked
@@ -744,7 +751,7 @@ extern int disable_double_click;
  *		tests that option and impliments the double click test if needed.
  *
  * \param last_click	The SDL_GetTicks() value from the last click
- * \retval int			Returns 1 if the button press should be actioned, else 0. 
+ * \retval int			Returns 1 if the button press should be actioned, else 0.
  */
 int safe_button_click(Uint32 *last_click);
 
@@ -763,15 +770,12 @@ int safe_button_click(Uint32 *last_click);
  * \param   	ly The height
  * \param   	Flags The flags
  * \param   	size The text size
- * \param   	r (0<=r<=1)
- * \param   	g (0<=g<=1)
- * \param   	b (0<=b<=1)
  * \param   	text The button label
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa button_add
  */
-int button_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, float r, float g, float b, const char *text);
+int button_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, const char *text);
 
 /*!
  * \ingroup	buttons
@@ -784,69 +788,68 @@ int button_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, U
  * \param   	text The button label
  * \param   	x The x position
  * \param   	y The y position
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa button_add_extended
  */
 int button_add(int window_id, int (*OnInit)(), const char *text, Uint16 x, Uint16 y);
 
 /*!
- * \ingroup	buttons
- * \brief 	Resize a button widget
+ * \ingroup buttons
+ * \brief Resize a button widget
  *
- * 		Resize a button widget using the specified scale. If x or y length ar zero, calculate them.
+ * Resize a button widget using to the specified dimensions. If \a lx or \a ly
+ * are zero, the corresponding dimension is calculated from the button's contents.
  *
- * \param   	window_id The location of the window in the windows_list.window[] array
- * \param   	lx The x length
- * \param   	ly The y length
- * \param   	size the new font and button size
- * \retval int  	Returns the new widgets unique ID
+ * \param window_id The location of the window in the windows_list.window[] array
+ * \param wid       The unique widget ID for the button
+ * \param lx        The new width
+ * \param ly        The new height
+ * \param size      the new font and button size
+ * \retval int Returns the new widgets unique ID
  *
  * \sa button_add_extended
  */
 int button_resize(int window_id, Uint32 wid, Uint16 lx, Uint16 ly, float size);
 
-
 /*!
- * \ingroup	buttons
- * \brief 	Draws a smooth button
+ * \ingroup buttons
+ * \brief Draws a button with round corners.
  *
- * 		Draws the smooth button widget pointed to by W.
+ * 	Draws a button with round corners. The box can be highlighted with the chosen highlight colors (r,g,b,a).
  *
- * \param   	W The button widget
- * \retval int  	Returns true
- * \callgraph
+ * \param str The name to write within the button, optional
+ * \param cat The category for the font with which to draw \a str
+ * \param size The size of the text
+ * \param x The start x position
+ * \param y The start y position
+ * \param w The width
+ * \param lines The number of lines (determines the height)
+ * \param r The red color for border and text
+ * \param g The green color for border and text
+ * \param b The blue color for border and text
+ * \param highlight If the button is highlighted or not
+ * \param hr The red color for highlighted buttons
+ * \param hg The green color for highlighted buttons
+ * \param hb The blue color for highlighted buttons
+ * \param ha The alpha color for highlighted buttons
  */
-int button_draw(widget_list *W);
-
+void draw_smooth_button(const unsigned char* str, font_cat cat, float size,
+	int x, int y, int w, int lines, float r, float g, float b,
+	int highlight, float hr, float hg, float hb, float ha);
 /*!
- * \ingroup	buttons
- * \brief 	Draws a square button
+ * \ingroup buttons
+ * \brief Compute the width of a button
  *
- * 		Draws the square button widget pointed to by W.
+ * Calculate the normal width of a button of size \a size, with label \a label drawn in the font
+ * for category \a cat.
  *
- * \param   	W The button widget
- * \retval int  	Returns true
- * \callgraph
+ * \param label The text to draw on the button
+ * \param cat   The font category for the button
+ * \param size  The size scale factor for the button
+ * \return The width of the button, in pixels
  */
-int square_button_draw(widget_list *W);
-
-/*!
- * \ingroup	buttons
- * \brief 	Sets the button text
- *
- * 		Finds the given button widget and sets the button text
- *
- * \param   window_id The location of the window in the windows_list.window[] array
- * \param   widget_id The unique widget ID
- * \param   text The button label
- * \retval int  Returns 1 on succes, 0 on failure (if the widget is not found in the given window)
- *
- * \sa widget_find
- */
-int button_set_text(int window_id, Uint32 widget_id, char *text);
-
-
+int calc_button_width(const unsigned char* label, font_cat cat, float size);
 
 // Progressbar
 
@@ -865,16 +868,13 @@ int button_set_text(int window_id, Uint32 widget_id, char *text);
  * \param   	ly The height
  * \param   	Flags The flags
  * \param   	size The text size
- * \param   	r (0<=r<=1)
- * \param   	g (0<=g<=1)
- * \param   	b (0<=b<=1)
  * \param   	progress The current progress
  * \param     colors The colors of the four corners of the bar. Pointer to an array of 12 floats (4 consecutive RGB colors). May be NULL.
  * \retval int  	Returns the new widgets unique ID
  *
  * \sa progressbar_add
  */
-int progressbar_add_extended(int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, float r, float g, float b, float progress, const float * colors);
+int progressbar_add_extended(int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, float progress, const float * colors);
 
 /*!
  * \ingroup	progressbars
@@ -888,38 +888,12 @@ int progressbar_add_extended(int window_id, Uint32 wid, int (*OnInit)(), Uint16
  * \param   	y The y position
  * \param   	lx The width
  * \param   	ly The height
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa progressbar_add_extended
  */
 int progressbar_add(int window_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly);
 
-/*!
- * \ingroup	progressbars
- * \brief 	Draws a progressbar
- *
- * 		The function draws the progressbar pointed to by *W
- *
- * \param   	W The progressbar widget that is going to be drawn
- * \retval int  	Returns false
- * \callgraph
- */
-int progressbar_draw(widget_list *W);
-
-/*!
- * \ingroup	progressbars
- * \brief 	Gets the progress from a progressbar
- *
- * 		Finds the progressbar widget and returns the current progress
- *
- * \param   	window_id The location of the window in the windows_list.window[] array
- * \param   	widget_id The unique widget ID
- * \retval float  	Returns -1 on failure, otherwise the current progress.
- *
- * \sa widget_find
- */
-float progressbar_get_progress(int window_id, Uint32 widget_id);
-
 /*!
  * \ingroup	progressbars
  * \brief 	Sets the current progress in the progressbar
@@ -954,17 +928,14 @@ int progressbar_set_progress(int window_id, Uint32 widget_id, float progress);
  * \param   	ly The height
  * \param   	Flags The flags
  * \param   	size The text size
- * \param   	r (0<=r<=1)
- * \param   	g (0<=g<=1)
- * \param   	b (0<=b<=1)
  * \param   	pos
  * \param   	pos_inc
  * \param		bar_len
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa vscrollbar_add
  */
-int vscrollbar_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, float r, float g, float b, int pos, int pos_inc, int bar_len);
+int vscrollbar_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, int pos, int pos_inc, int bar_len);
 
 /*!
  * \ingroup	scrollbars
@@ -978,24 +949,12 @@ int vscrollbar_add_extended(int window_id, Uint32 wid,  int (*OnInit)(), Uint16
  * \param   	y The y position
  * \param   	lx The width
  * \param   	ly The height
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa vscrollbar_add_extended
  */
 int vscrollbar_add(int window_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly);
 
-/*!
- * \ingroup	scrollbars
- * \brief 	Draws a vertical scrollbar
- *
- * 		Draws the vertical scrollbar given by *W
- *
- * \param   	W A pointer to the vertical scrollbar widget you wish to draw
- * \retval int  	Returns false
- * \callgraph
- */
-int vscrollbar_draw(widget_list *W);
-
 /*!
  * \ingroup	scrollbars
  * \brief 	Sets the position of the vertical scrollbar
@@ -1091,7 +1050,7 @@ int vscrollbar_get_pos(int window_id, Uint32 widget_id);
  * \brief 	Returns the number of the currently selected tab
  *
  * 		Returns the number of the currently selected tab in a tabbed window collection. Numbers are in the range 0...nr_tabs-1.
- *		
+ *
  * \param   	window_id The location of the window in the windows_list.window[] array
  * \param   	widget_id The unique widget ID of the tab collection
  * \retval int  	Returns the tab number on succes, -1 on failure
@@ -1105,7 +1064,7 @@ int tab_collection_get_tab (int window_id, Uint32 widget_id);
  * \brief 	Returns the window ID of the currently selected tab
  *
  * 		Returns the window ID of the currently selected tab in a tabbed window collection.
- *		
+ *
  * \param   	window_id The location of the window in the windows_list.window[] array
  * \param   	widget_id The unique widget ID of the tab collection
  * \retval int  	Returns the tab's window ID number on succes, -1 on failure
@@ -1117,7 +1076,7 @@ int tab_collection_get_tab_id (int window_id, Uint32 widget_id);
  * \brief 	Returns the position of a tab in the collection from its window ID
  *
  * 		Returns the position of a tab in the collection from its window ID
- *		
+ *
  * \param   	window_id The location of the window in the windows_list.window[] array
  * \param   	col_id The unique widget ID of the tab collection
  * \param   	tab_id The tab's window ID
@@ -1125,24 +1084,12 @@ int tab_collection_get_tab_id (int window_id, Uint32 widget_id);
  */
 int tab_collection_get_tab_nr (int window_id, Uint32 col_id, int tab_id);
 
-/*!
- * \ingroup	tabs
- * \brief 	Returns the number of tabs in this collection
- *
- * 		Returns the number of tabs in this collection
- *		
- * \param   	window_id The location of the window in the windows_list.window[] array
- * \param   	widget_id The unique widget ID of the tab collection
- * \retval int  	Returns the number of tabs, or -1 on failure
- */
-int tab_collection_get_nr_tabs (int window_id, Uint32 widget_id);
-
 /*!
  * \ingroup	tabs
  * \brief 	Sets the label color for a tab
  *
  * 		Sets the color with which the label of the tab belonging to the window with ID \a tab_id is drawn.
- *		
+ *
  * \param   	window_id The location of the window in the windows_list.window[] array
  * \param   	col_id The unique widget ID of the tab collection
  * \param	tab_id The window ID of the tab window
@@ -1185,14 +1132,15 @@ int tab_collection_close_tab (int window_id, Uint32 widget_id, int tab);
  * \ingroup	tabs
  * \brief 	Calculate the tab tag height
  *
- * 		Calculate the tab tag height given the specified size.
+ * Calculate the tab tag height given the specified size and font category for
+ * the label.
  *
- * \param   	size the scale factor
+ * \param cat  the category for the font with which the label is drawn
+ * \param size the scale factor
  * \retval int  	Returns the calculate tag tag height.
  * \callgraph
  */
-int tab_collection_calc_tab_height(float size);
-
+int tab_collection_calc_tab_height(font_cat cat, float size);
 
 /*!
  * \ingroup	tabs
@@ -1206,7 +1154,7 @@ int tab_collection_calc_tab_height(float size);
  * \param   	y The y position
  * \param   	lx The width
  * \param   	ly The height
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa tab_collection_add_extended
  */
@@ -1227,29 +1175,14 @@ int tab_collection_add (int window_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint
  * \param   	ly The height
  * \param   	Flags The flags
  * \param   	size The text size
- * \param   	r (0<=r<=1)
- * \param   	g (0<=g<=1)
- * \param   	b (0<=b<=1)
- * \param	max_tabs The largest number of tabs this collection will hold
- * \retval int  	Returns the new widgets unique ID 
+ * \param   	max_tabs The largest number of tabs this collection will hold
+ * \param   	right_margin space to leave to the right of the line of tabs (for example, for a close box)
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa tab_collection_add
  */
-int tab_collection_add_extended (int window_id, Uint32 wid, int 
-(*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, 
-float size, float r, float g, float b, int max_tabs);
-
-/*!
- * \ingroup	tabs
- * \brief 	Draws a tabbed window collection
- *
- * 		Draws the vertical tabbed window collection given by *W
- *
- * \param   	W A pointer to the tabbed window collection you wish to draw
- * \retval int  	Returns 1 on success, 0 on error
- * \callgraph
- */
-int tab_collection_draw (widget_list *W);
+int tab_collection_add_extended (int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y,
+	Uint16 lx, Uint16 ly, Uint32 Flags, float size, int max_tabs, int right_margin);
 
 /*!
  * \ingroup	tabs
@@ -1260,7 +1193,6 @@ int tab_collection_draw (widget_list *W);
  * \param   	W The widget
  * \param   	w the new width
  * \param   	h the new height
- * \param   	tab_tag_height the height of the tab tags
  * \retval int  	Returns 1 on success, 0 on failure
  * \callgraph
  */
@@ -1284,7 +1216,7 @@ int tab_collection_move (widget_list *W, Uint32 pos_x, Uint32 pos_y);
  * \ingroup	tabs
  * \brief 	Creates a new tabbed window
  *
- * 		Creates a new tabbed window 
+ * 		Creates a new tabbed window
  *
  * \param   	window_id The location of the parent window in the windows_list.window[] array
  * \param   	col_id The unique widget id of the tabbed window collection in which this tab is created
@@ -1313,7 +1245,7 @@ int tab_add (int window_id, Uint32 col_id, const char *label, Uint16 tag_width,
  * \param 	buf_size the size of the message buffer
  * \param	x_space the number of pixels in the x-direction between the border and the text
  * \param	y_space the number of pixels in the y-direction between the border and the text
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa text_field_add_extended
  */
@@ -1333,32 +1265,32 @@ int text_field_add (int window_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 l
  * \param   	lx The width
  * \param   	ly The height
  * \param   	Flags The flags
+ * \param		fcat Font category for the text
  * \param   	size The text size
- * \param   	r (0<=r<=1)
- * \param   	g (0<=g<=1)
- * \param   	b (0<=b<=1)
  * \param	buf the text buffer
  * \param 	buf_size the size of the text buffer
  * \param	chan_filt the channel of which messages are drawn
  * \param	x_space the number of pixels in the x-direction between the border and the text
  * \param	y_space the number of pixels in the y-direction between the border and the text
- * \retval int  	Returns the new widgets unique ID 
+ * \retval int  	Returns the new widgets unique ID
  *
  * \sa text_field_add
  */
-int text_field_add_extended (int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, float size, float r, float g, float b, text_message *buf, int buf_size, Uint8 chan_filt, int x_space, int y_space);
+int text_field_add_extended (int window_id, Uint32 wid, int (*OnInit)(),
+	Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint32 Flags, font_cat fcat,
+	float size, text_message *buf, int buf_size, Uint8 chan_filt, int x_space, int y_space);
 
 /*!
- * \ingroup	textfields
- * \brief 	Draws a text field
+ * \ingroup textfields
+ * \brief Return the number of visible lines
  *
- * 		Draws the vertical textfield given by \a *w
+ * Return the number of lines shown by a text field.
  *
- * \param   	w A pointer to the text field you wish to draw
- * \retval int  	Returns 1 on success, 0 on error
- * \callgraph
+ * \param window_id The location of the window in the windows_list.window[] array
+ * \param widget_id The unique widget ID
+ * \retval int Return the number of visible lines on success, 0 on failure
  */
-int text_field_draw (widget_list *w);
+int text_field_get_nr_visible_lines(int window_id, int widget_id);
 
 /*!
  * \ingroup	textfields
@@ -1374,14 +1306,28 @@ int text_field_draw (widget_list *w);
  * \callgraph
  */
 int text_field_set_buf_pos (int window_id, Uint32 widget_id, int msg, int offset);
+/*!
+ * \ingroup	textfields
+ * \brief Scroll to a line
+ *
+ * Set the drawing offset of the text field such that it starts drawing at line \a line_nr.
+ *
+ * \param window_id The location of the window in the windows_list.window[] array
+ * \param widget_id The unique widget ID
+ * \param line_nr   The line number to scroll to (zero based)
+ * \retval int Returns 1 on success, 0 on error
+ * \sa text_field_set_buf_pos.
+ * \callgraph
+ */
+int text_field_scroll_to_line(int window_id, Uint32 widget_id, int line_nr);
 
 /*!
  * \ingroup	textfields
  * \brief       Clear an editable text field
- * 
+ *
  *              Clear an editable text field, erasing its current buffer and
  *              moving the cursor to the start
- * 
+ *
  * \param   	window_id The location of the window in the windows_list.window[] array
  * \param	widget_id The unique widget ID
  * \retval int  	Returns 1 on success, 0 on error
@@ -1389,22 +1335,6 @@ int text_field_set_buf_pos (int window_id, Uint32 widget_id, int msg, int offset
  */
 int text_field_clear (int window_id, Uint32 widget_id);
 
-/*!
- * \ingroup	textfields
- * \brief 	Sets the text color
- *
- * 		Sets the color with which the text is drawn. Not that color characters in the text override this setting.
- *
- * \param   	window_id The location of the window in the windows_list.window[] array
- * \param	widget_id The unique widget ID
- * \param	r the red component of the text color
- * \param	g the green component of the text color
- * \param	b the blue component of the text color
- * \retval int  	Returns 1 on success, 0 on error
- * \callgraph
- */
-int text_field_set_text_color (int window_id, Uint32 widget_id, float r, float g, float b);
-
 /*!
  * \ingroup	widgets
  * \brief	Is called on keypress in the given widget
@@ -1421,25 +1351,15 @@ int text_field_set_text_color (int window_id, Uint32 widget_id, float r, float g
  */
 int text_field_keypress (widget_list *w, int mx, int my, SDL_Keycode key_code, Uint32 key_unicode, Uint16 key_mod);
 
-/*!
- * \brief set cursor_line according to cursor position in current message.
- *
- * \param[in] tf text_field where we set cursor_line.
- */
-void text_field_find_cursor_line(text_field* tf);
-
-
 //FIXME: Write documentation for these...
 #define P_NORMAL    0
 #define P_TEXT      1
 #define P_NONE      2
 
-int pword_keypress (widget_list *w, int mx, int my, SDL_Keycode key_code, Uint32 key_unicode, Uint16 key_mod);
-unsigned char * pword_field_get(widget_list *w);
 int pword_field_add (int window_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint8 status, unsigned char *buffer, int buffer_size);
-int pword_field_add_extended (int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint8 status, float size, float r, float g, float b, unsigned char *buffer, int buffer_size);
-int pword_field_click(widget_list *w, int mx, int my, Uint32 flags);
-void pword_set_status(widget_list *w, Uint8 status);
+int pword_field_add_extended (int window_id, Uint32 wid, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint8 status, float size, unsigned char *buffer, int buffer_size);
+int pword_field_set_content(int window_id, Uint32 widget_id, const unsigned char* buf, size_t len);
+int pword_clear(int window_id, Uint32 widget_id);
 
 int multiselect_add(int window_id, int (*OnInit)(), Uint16 x, Uint16 y, int width);
 int multiselect_add_extended(int window_id, Uint32 widget_id, int (*OnInit)(), Uint16 x, Uint16 y, int width, Uint16 max_height, float size, float r, float g, float b, float hr, float hg, float hb, int max_buttons);
@@ -1447,13 +1367,17 @@ int multiselect_button_add(int window_id, Uint32 multiselect_id, Uint16 x, Uint1
 int multiselect_button_add_extended(int window_id, Uint32 multiselect_id, Uint16 x, Uint16 y, int width, const char *text, float size, const char selected);
 int multiselect_get_selected(int window_id, Uint32 widget_id);
 int multiselect_set_selected(int window_id, Uint32 widget_id, int button_id);
+int multiselect_get_scrollbar_pos(int window_id, Uint32 widget_id);
+int multiselect_set_scrollbar_pos(int window_id, Uint32 widget_id, int pos);
 int multiselect_get_height(int window_id, Uint32 widget_id);
+int multiselect_clear(int window_id, Uint32 widget_id);
+int multiselect_set_scrollbar_inc(int window_id, Uint32 multiselect_id, int inc);
 
 #define SPIN_FLOAT 0
 #define SPIN_INT 1
 
 int spinbutton_add(int window_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint8 data_type, void *data, float min, float max, float interval);
-int spinbutton_add_extended(int window_id, Uint32 widget_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint8 data_type, void *data, float min, float max, float interval, float size, float r, float g, float b);
+int spinbutton_add_extended(int window_id, Uint32 widget_id, int (*OnInit)(), Uint16 x, Uint16 y, Uint16 lx, Uint16 ly, Uint8 data_type, void *data, float min, float max, float interval, float size);
 
 /*!
  * \ingroup	widgets
@@ -1513,20 +1437,48 @@ int widget_handle_drag (widget_list *widget, int mx, int my, Uint32 flags, int d
  * \retval 	1 if the event is handled, 0 otherwise
  */
 int widget_handle_keypress (widget_list *widget, int mx, int my, SDL_Keycode key_code, Uint32 key_unicode, Uint16 key_mod);
+/*!
+ * \ingroup widgets
+ *
+ * Handle a change in font
+ *
+ * Handle a change in font or font size in font category \a cat for widget \a widget.
+ *
+ * \param widget The widget to handle the font change for.
+ * \param cat    The font category that was changed.
+ *
+ * \return 1 if the widget handled the change, 0 otherwise
+ */
+int widget_handle_font_change(widget_list *widget, font_cat cat);
+/*!
+ * \ingroup widgets
+ *
+ * Handle a paste event
+ *
+ * Handle a text paste event, and paste text \a text into widget \a widget.
+ *
+ * \param widget The widget to handle the paste event
+ * \param text   The text to paste into the widget
+ *
+ * \return 1 if the widget handled the change, 0 otherwise
+ */
+int widget_handle_paste(widget_list *widget, const char* text);
+
 
-// XML Windows
 
 /*!
- * \ingroup 	xml_windows
- * \brief 	Adds a window from an xml-file.
+ * \ingroup	widgets
+ * \brief 	A general helper function to draw a cross.
  *
- * 		Adds a window from an xml-file.
+ * 		Draw a scalable cross centred the specified position and of the
+ * 	specified size and width.
  *
- * \param   	fn The filename
- * \retval int  	Returns 0 on failure and the window_id on succes
- * \callgraph
+ * \param	the centre x coordinate of the cross.
+ * \param	the centre y coordinate of the cross.
+ * \param	half the length in pixels of the cross width/height.
+ * \param	half the width in pixels of cross lines
  */
-int AddXMLWindow(char *fn);
+void draw_cross(int centre_x, int centre_y, int half_len, int half_width);
 
 #ifdef __cplusplus
 } // extern "C"
