@@ -496,7 +496,7 @@ static Bool Xz_CheckFooter(CXzStreamFlags flags, UInt64 indexSize, const Byte *b
 
 #define READ_VARINT_AND_CHECK(buf, pos, size, res) \
   { unsigned s = Xz_ReadVarInt(buf + pos, size - pos, res); \
-  if (s == 0) return SZ_ERROR_ARCHIVE; pos += s; }
+  if (s == 0) { return SZ_ERROR_ARCHIVE; } pos += s; }
 
 
 SRes XzBlock_Parse(CXzBlock *p, const Byte *header)
